package provide Actions


# Interface to rsync
# ==================
#	- real and dry run rsync execution
#	- filter file creation
#	- list of server files (or second local path)
#	- checking validity of remote host address



namespace eval Actions {


# Dump message for debugging
# --------------------------
#	Substitute codes recognised by pseudo terminal editor

    proc DumpMessage { p_text p_code } {

	    puts "xxx_$p_code **[ string map { \
						"\n" "\\n" \
						"\r" "\\r" \
						"\b" "\\b" \
						"\v" "\\b" \
						"\t" "\\t" } $p_text ]**"
    }



# Performe a synchronise action
# -----------------------------

    proc Synchronise { p_execute } {
			  # p_execute	dry ... only probe and list
			  #		real ... really synchronise

	global		global_vars
	global		button_value

	variable	is_ignore "ignoring "
	variable	is_delete "deleting "

	variable	::MainPanel::main_widgets
	variable	::Config::conf_descr
	variable	::Config::conf_params

	if { $p_execute == "dry" } {
	    set x_text "Nodes requiring synchronisation:"
	} else {
	    set x_text "Nodes that have been synchronised:"
	}

	$main_widgets(ACTION) configure -text $x_text

	MainProcs::VoidResult
	FilterFile

	    # rsync options used:
		# -r		recursively go down directories
		# -p		preserve access permissions
		# -t		preserve last access times
		#-g		preserve group association
		#-o		preserve ownership association
		# -u		only transfer updated files
		# -z		compress data during transfer
		# -v		verbose mode
		#
		# -n		dry-run (fake execution)
		#
		# --progress	send progress messages (implies -v..)
		# --links	copy symlinks as symlinks
		# --safe-links	ignore symlinks that point outside the tree

	MainProcs::ProgressInfo 0 ""

	set x_args1 {--log-format=%n%L\ %i\ %t}
	append x_args1 { --progress}
	set x_args2 "-rzv"
	if { $p_execute == "dry" } { append x_args2 "n" }

	for { set x_x 0 } { [ info exists conf_descr($x_x) ] } { incr x_x } {
	    set x_tag $conf_descr($x_x)
	    if { [ info exists button_value($x_tag) ] == 0 } continue
	    if { $button_value($x_tag) == 0 } continue
	    if { [ info exists conf_descr($x_tag,ARG) ] == 0 } continue
	    if { [ string length $conf_descr($x_tag,ARG) ] == 1 } {
		append x_args2 $conf_descr($x_tag,ARG)
	    } else {
		append x_args1 " -$conf_descr($x_tag,ARG)"
	    }
	}
	if { $global_vars(MODIFY_WINDOW) != 0 } {
	    append x_args1 " --modify-window=$global_vars(MODIFY_WINDOW)"
	}
	set server_path $global_vars(SERVER_BASE)$global_vars(REL_PATH)/
	set client_path $global_vars(CLIENT_BASE)$global_vars(REL_PATH)/
	if { $global_vars(SERVER_HOST) != "" } { set server_path \
				$global_vars(SERVER_HOST)::$server_path }
	if { $button_value(XFER_DIRECTION) == 0 } {
	    set x_sites "$server_path $client_path"
	} else {
	    set x_sites "$client_path $server_path"
	}
	MainProcs::WaitMessage "Analysing directories"

	eval "spawn rsync $x_args1 $x_args2 $x_sites"

	set x_phase 0
	set x_count 0
        expect {
	    -re {^\s*([0-9]+) files\s*([^\r]+)} {
		if { $expect_out(2,string) != "to consider" } exp_continue
		set x_files $expect_out(1,string)
	    }
	    -re {[^\n]+\n} {
		set x_text [ string trim $expect_out(0,string) ]
		if { [ string first "rsync:" $x_text ] == 0 } {
		    if [ regexp {No such file or directory} $x_text ] {
			set x_phase -1 
		    } else {
		        set x_phase -2
		    }
		    MainProcs::AddResult $x_text "error"
		} else {
		    exp_continue
		}
	    }
	    eof { set x_phase 1 }
	    timeout {
		MainProcs::AddResult "Timeout" "error"
		set x_phase -1
	    }
	}
	MainProcs::WaitMessage ""
	if { $x_phase < 0 }  { return { "error" {} } }
	if { $x_phase < -1 } { return { "error" { "Verify:" \
				"   - connectivity to remote host" \
				"   - state of daemon on remote host" } }
	}
	if { $x_phase == 1 } { return { "error" { \
				"No data received from rsync server " } }
	}

	set x_count 1
	MainProcs::ProgressInfo 3 "$x_files files considered"
	MainProcs::WaitMessage "Analysing the list of files to sync, wait ..."
        expect {
	    -timeout $global_vars(SYNC_TIMEOUT)

		# Messages just to skip

	    -re {^\s*\n} { exp_continue }

	    -re {^\s+\r} { exp_continue }

	    -re {([^\n]+)\s+is (newer|uptodate)\s*} {
		set x_current $expect_out(1,string)
		exp_continue
	    }

	    -re {^\s*\d+\s+files\S+\r} {
		exp_continue
	    }

	    -re {^delta-transmission [^\n\r]+\s*\n} {
		exp_continue
	    }

		# Final message, exit

	    -re {^sent [0-9]+ bytes.*\n} { }

		# Skipping non-regular file ... error message

	    -re {^(skipping non-regular[^\n\r]+)\r+\n*} {
		if { $button_value(SHOW_SKIPPED) == 0 } exp_continue

		MainProcs::AddResult $expect_out(1,string) "error"
		exp_continue
	    }

		# Rsync protocol message ... error message

	    -re {^(rsync:[^\n\r]+)\r+\n*} {
		MainProcs::AddResult $expect_out(1,string) "error"
		exp_continue
	    }

		# Ignoring ... skip message

	    -re {^ignoring ([^\n\r]+)\r+\n*} {
		MainProcs::AddResult $expect_out(1,string) "skip"
		exp_continue
	    }

		# Progress message ... copy to main panel

	    -re {^\s+\d+\s+(\S+)\s+\S+\s+([^\n\r]+)\r+\n*} {
		MainProcs::ProgressInfo 0 "$expect_out(1,string) of"
		if { [ regexp {\((\d+),} $expect_out(2,string)  "" x_count ] } \
								{ incr x_count }
		exp_continue
	    }

		# A synced file ... message to result panel and log file

	    -re {^([^\n\r]+)\s+(\S+)\s+(\S+\s+\S+)\s*\n} {
		if { $p_execute == "dry" } { incr x_count }
		if { $expect_out(2,string) == "*deleting" } \
				{ set x_op "delete" } else { set x_op "" }
		MainProcs::AddResult $expect_out(1,string) $x_op
		MainProcs::ProgressInfo 1 "file # $x_count :"
		regexp {(\S+)} $expect_out(1,string) "" x_temp
		MainProcs::ProgressInfo 2 $x_temp
		if { ( $p_execute != "dry" ) && ( $button_value(WRITE_LOG) \
								!= 0) } {
		    MainAux::LogItem $expect_out(1,string) \
				$expect_out(2,string) $expect_out(3,string)
		}
		exp_continue
	    }

	    timeout {
		MainProcs::AddResult "Timeout during reception" "error"
		set x_phase -1
	    }
	}
	catch close
	wait
	MainProcs::WaitMessage ""

	    # Wipe out all (except last ( = # of files considered) fields

	for { set x_x 0 } { $x_x < 3 } { incr x_x } \
					{ MainProcs::ProgressInfo $x_x "" }

	if { $x_phase < 0 } { return { "error" { "Verify:" \
				"   - adequacy of timeout-limit =\
						$global_vars(SYNC_TIMEOUT) sec\
							for very big files" \
				"   - connectivity to remote host" \
				"   - state of daemon on remote host" } }
	} else {
	    return { "done" "done" }
	}
    }



# Create the filter file
# ----------------------
#	- Include all directories (descend recursively) and files selected
#	  by checkbuttons
#	- Do not include any other files and directories
#	- when recursively descending, do no include temporary editor files
#	  (globing to *~ and #*)

    proc FilterFile { { p_direction 0 } } {
			  # p_direction	0   ...	synchronize to client
			  #		else...	synchronize from client

	global		global_vars
	global		select_list
	global		button_value

	variable	::Config::conf_params

	set x_x [ string length $global_vars(CLIENT_BASE) ]

	set x_fid [ open $global_vars(TEMP_FILE) w ]
	set x_temp ""
	for { set x_index 0 } { [ info exists select_list($x_index,TYPE) ] } \
							{ incr x_index } {
	    if { [ info exists select_list($x_index,VAR) ] == 0 } continue
	    set x_var $select_list($x_index,VAR)
	    set x_name $select_list($x_index,NAME)
	    if { $x_var == 0 } { set x_sign "-" } else  { set x_sign "+" }
	    if { $select_list($x_index,TYPE) == "tree" } {
		if { $button_value(HIDDEN_NODES) == 0 } \
					{ puts $x_fid "- /$x_name/**/.*" }	
		puts $x_fid "$x_sign /$x_name/"
		puts $x_fid "$x_sign /$x_name/**"
	    } elseif { $select_list($x_index,TYPE) == "file" } {
		puts $x_fid "$x_sign /$x_name"
	    }
	}

	puts $x_fid "- *"
	close $x_fid
    }



# Construct list of files for given path on rsync label at server
# ---------------------------------------------------------------
#	Sort list items: start with hidden nodes, continue with visible nodes

    proc RemoteList { p_host p_path } {
			  # p_path	path on server relative to label with
			  #			leading / unless p_path is void

	global		global_vars

	set x_hidden { } 
	set x_normal { }
	if { $p_host != "" } {
	    if { $global_vars(NETWORK) == 0 } { return "" }
	    set x_fid [ open "|/usr/bin/rsync -p ${p_host}::$p_path/" r ]
	} else {
	    set x_fid [ open "|/bin/ls -lgo $p_path" r ]
	}
	while { [ gets $x_fid x_line ] >= 0 } {
	    if [ regexp {^total} $x_line ] continue
	    set x_flag [ string range $x_line 0 0 ]
	    if { $x_flag == "s" } continue
	    regexp {.+\s+(\S+)\s+(\S+)\s+(\S+)} $x_line "" x_1 x_2 x_3
	    if { $x_2 == "->" } { 
		if { $x_flag == "d" } { set x_1 "${1}/" }
		set x_item "$x_1 ->"
	    } else {
		if { $x_3 == "." } continue
		if { $x_flag == "d" } { 
		    set x_3 "${x_3}/"
		} elseif { $x_flag == "l" } {
		    set x_3 "$x_3 ->"
		}
		set x_item $x_3
	    }
	    if { [ string range $x_item 0 0 ] == "." } {
		lappend x_hidden $x_item
	    } else {
		lappend x_normal $x_item
	    }
	}
	catch { close $x_fid }
	return [ concat $x_hidden $x_normal ]
    }



# Check whether rsync host exists and replies
# -------------------------------------------
#	Return 1 if ok, else -1

    proc HostIsup { p_host } {

	set x_ok 0
	spawn nmap -sP -T Aggressive $p_host
	expect {
	    -re {1 host up} { set x_ok 1 }
	    -re {0 hosts up} { set x_ok -1 }
	}
	return $x_ok
    }



# Check whether rsync host exists and is correctly addressed
# ----------------------------------------------------------
#	If OK: return empty list
#	if error: return 2-item list
#			item #0 ... error text
#			item #1	... error argument

    proc CheckHost { p_host p_path } {

	global		global_vars
	global		host_table

	if { $global_vars(NETWORK) == 0 } { return [ list \
					"Network is unreachable" "$p_host" ] }
	spawn rsync --exclude \* ${p_host}::$p_path
        expect {
	    -re {[^\n]\s+(\S+)\s+\d+\:\s+[^\n]+service not known[^\n]*\n} {
		return [ list "The following service is not known:" \
						"$expect_out(1,string)" ]
	    }
	    -re {[^\n]+Unknown module.+\'([^\']+)\'.*\n} {
		return [ list "The following module does not exist\
				on host \"$p_host\":" "$expect_out(1,string)" ]
	    }
	    -re {[^\n]+Network is unreachable.*\n} {
		set global_vars(NETWORK) 0
		foreach x_item [ array names host_table *,FLAGS ] \
						{ set host_table($x_item) "x" }
		return [ list "Network is unreachable" "$p_host" ]
	    }
	    -re {[^\n]+\n} {
		exp_continue
	    }
	}
	return ""
    }



# Check whether synchronisation zone is in read-only module
# ---------------------------------------------------------

    proc CheckZone  { p_host p_path } {

	global		global_vars

	regexp {^([^/]+)} $p_path "" x_path

	set x_readOnly 0
	spawn rsync -nv --exclude "+ /*/" $global_vars(TEMP_FILE) \
							${p_host}::$x_path
        expect {
	    -re {read\s+only} {
		set x_readOnly 1
		exp_continue
	    }
	    eof { }
	}
	return $x_readOnly
    }



# Check whether network is up
#----------------------------
#	Return 1 if it is up, 0 if it is down

    proc CheckNetwork { } {
	set x_temp [ exec /bin/grep nameserver /etc/resolv.conf ]
	regsub {\S+\s+} $x_temp "" x_temp
	catch [ list exec /bin/ping  -c 1 -W 1 $x_temp ] x_temp
	if { [ regexp {PING \d+\.\d+\.\d+\.\d+.* 1 received} $x_temp ] == 0 } {
	    return 0
	} else {
	    return 1
	}
    }
}
