package provide AuxProcs


# Common auxiliary support procedures
# ===================================



namespace eval AuxProcs {


# Auxiliary procedures
# ====================


# Define a an entry in the synchronisation-zone table
# ---------------------------------------------------
#	Return table index

    proc DefineZone { clBase svBase svHost p_flags } {

	global		sync_zones

	if { $svHost == "" } {
	    set x_zone "${svBase},${clBase}"
	} else {
	    set x_zone "${svBase}@${svHost},${clBase}"
	}
	set x_temp [ expr [ array size sync_zones ] / 5 ]
	set sync_zones($x_zone,SV_HOST) $svHost
	set sync_zones($x_zone,SV_BASE) $svBase
	set sync_zones($x_zone,CL_BASE) $clBase
	set sync_zones($x_zone,FLAGS)   $p_flags
	set sync_zones($x_zone,NUMBER)  $x_temp
	return $x_zone
    }



# Read a list of X-resources and define correpsonding parameter variables
# -----------------------------------------------------------------------
#	If resource not found:
#	    - if p_widget != "" 
#		Query whether to kill or to continue and substitute ""
#		return 1 if OK
#	    - if p_widget == ""
#		return 0
#	Else return 1

    proc DefineParameters { p_array p_widget p_list } {
			  # p_array	parameter array for storing variables
			  # p_widget	main panel widget for option get
			  # p_list	list of X-resources = parameter-array
			  #						indices

	variable	::Config::conf_params

	upvar #0 $p_array x_local

	if { $p_widget == "" } { set x_widget "." } else \
						{ set x_widget $p_widget }
	foreach x_x $p_list {
	    set x_x [ string trim $x_x ]
	    set x_temp [ string trim [ option get $x_widget \
							[ lindex $x_x 0 ] "" ] ]
	    if { $x_temp == "" } {
		if { $p_widget == "" } { return 0 }

		set x_ok [ Dialog::DialogMessage $p_widget Warning "" [ list \
			"Could not find resource \"$x_x\"" \
			{OK to define it as empty string?} ] { "OK" "Kill" } ]
		if { $x_ok != 0 } exit
	    }
	    if { [ string range $x_temp 0 0 ] == "\"" } \
				{ set x_temp [ string trim $x_temp "\"" ] }
	    set x_local([lindex $x_x 0]) $x_temp
	}
	return 1
    }



# Expand leading item of path ($xxx, ~/xxx ...)
# ---------------------------------------------

    proc ExpandPath { p_path } {
			  # p_path	path to expand

	global		env

	set x_temp $p_path
	if [ regexp {^~/(.*)$} $x_temp "" x_trail ] {
	    set x_temp $env(HOME)/$x_trail
	} elseif [ regexp {^(\$[^/]+)(.*)$} $x_temp "" x_lead x_trail ] {
	    set x_symbol [ string range $x_lead 1 end]
	    if [ info exists env($x_symbol) ] \
					{ set x_temp $env($x_symbol)$x_trail }
	}
	return $x_temp
    }



# Wait for a main-panel action completion (i.e. setting of a control variable)
# ----------------------------------------------------------------------------
#	This is an extension to the procs in the module dialog:
#	    1.  disable all children of the outer frame
#	    2.	wait for the variable to become modified
#	    3.	enable the childre again

    proc ActionWait { p_dialog } {
			  # p_dialog	toplevel dialog

	global		dialog_list

	foreach x_frame [ winfo children .outer ] \
				{ MainProcs::SetState $x_frame disabled }
	set dialog_list(wait,$p_dialog) ""
	tkwait variable dialog_list(wait,$p_dialog)
	foreach x_frame [ winfo children . ] \
				{ MainProcs::SetState $x_frame normal }
	if { [ info exists dialog_list(wait,$p_dialog) ] } {
	    set exit_code $dialog_list(wait,$p_dialog)
	    unset dialog_list(wait,$p_dialog)
	} else {
	    set exit_code ""
	}
	return $exit_code
    }



# Return argument value from a list with name-value argument pairs
# ----------------------------------------------------------------

    proc ListArgument { p_list p_name } {

	set x_x [ lsearch -exact $p_list "$p_name" ]
	return [ lindex $p_list [ expr $x_x + 1 ] ]
    }



# Generate a check button array
# -----------------------------
#	Return highest absolute column and row filled
#
#	Note: rows and columns found in the conf_descr array are considered
#	relative to baseCol and baseRow
	
    proc CheckboxArray { p_frame p_code p_baseCol p_baseRow { p_font "" } \
						{ p_state ::button_value } } {
			  # p_frame	gridded frame with checkbuttons
			  # p_code	1-character code for panel
			  # p_baseCol	first column in grid to fill
			  # p_baseRow	first row in grid to fill
			  # p_font	font for checkbutton labels
			  # p_state	if "" ... use the global "button_value"
			  #			array for control variables
			  #		else  ... use this variable as array
			  #			with the state variables; start 
			  #			by copying the variables for
			  #			this panel from button_value

	variable	::Config::conf_descr
	variable	::Config::conf_default

	global		button_value
	global		button_coord

	set max_col 0
	set max_row 0

	if { $p_state != "::button_value" } {
	    if [ info exists $p_state ] { unset $p_state }
	    upvar $p_state local_state
	}

	    # Destroy any old widgets

	set x_dialog [ winfo toplevel $p_frame ]
	set x_save [ bind $x_dialog <Destroy> ]
	bind $x_dialog <Destroy> {}
	set x_max [ lindex [ grid size $p_frame ] 0 ] 
	for { set x_x $p_baseCol } { $x_x < $x_max } { incr x_x } {
	    set x_list [ grid slaves $p_frame -column $x_x ]
	    foreach x_widget $x_list { destroy $x_widget }
	}
	eval "bind $x_dialog <Destroy> [ list $x_save ]"

	    # Create the checkbuttons

	if { $p_font == "" } { set p_font "normal" }
	foreach x_item [ array names conf_descr *,FLAGS ] {
	    regsub {,FLAGS} $x_item "" x_item
	    set x_flags $conf_descr($x_item,FLAGS)

		    # skip 
		    #	- if the item is not for the present panel
		    #	- if the item is not a checkbutton
		    #	- if a main-panel item is not configured
		    #		("M" flag, no button_coord value defined)

	    if { [ string first $p_code $x_flags ] < 0 } continue
	    if { [ string first "c" $x_flags ] < 0 } continue
	    if { ( [ string first "M" $x_flags ] >=0 ) && ( [ info exists \
				button_coord($x_item) ] == 0 ) } continue

		# This is a checkbutton for the current panel		

	    if { $p_state != "::button_value" } \
			{ set local_state($x_item) $button_value($x_item) }

	    if [ info exists button_coord($x_item) ] {
		set x_x [ lindex $button_coord($x_item) 0 ]
		set x_y [ lindex $button_coord($x_item) 1 ]
	    } else {
		set x_x [ lindex $conf_default($x_item,PROPS) 0 ]
		set x_y [ lindex $conf_default($x_item,PROPS) 1 ]
	    }
	    if { [ expr $x_x % 2 ] == 0 } { set x_pad 0 } else { set x_pad 15 }
	    incr x_x $p_baseCol
	    incr x_y $p_baseRow
	    if { $x_x > $max_col } { set max_col $x_x }
	    if { $x_y > $max_row } { set max_row $x_y }

	    eval "checkbutton $p_frame.c_${x_x}_$x_y -font $p_font \
				-variable $p_state\($x_item\) \
				-text \"$conf_descr($x_item,TEXT)\""
	    grid $p_frame.c_${x_x}_$x_y -column $x_x -row $x_y \
							-padx $x_pad -sticky w
	}

	for { set x_x $p_baseCol } { $x_x <= $max_col } { incr x_x } \
				{ grid columnconfigure $p_frame $x_x -weight 0 }
	return [ list $max_col $max_row ]
    }
}
