package provide Config



# Configuration control procedures
# ================================
#	- Initialise the configuration from the configuration file
#	- Save the configuration in the configuration file



namespace eval Config {

	# if executed from the place of an installed rpm, this string will be 
	# replaced by the build date of the rpm

    variable	version_string "V-3.8.3-2    24.02.2006 - 17h10"



# List of option checkbutton groups				group list
# =================================
#	This list only serves for initialisation, it is then destroyed.
#	    It defines groups for assembling sets of configuration descriptors 
#
#	In the checkbutton display, buttons are sorted by group. The sequence
#	    of the items in "group_list" determines the sequence in which
#	    groups appear in the display (within each group, the sequence 
#	    is determined by the order within "conf_list")
#
#	The group code serves for reference ("M<code>") in the flag fields
#	    of the conf_descr entries
#
#				code	    header title

    variable group_list {	{ X	"Selection criteria"		}
				{ R	"Filter rules"			}
				{ P	"Property preservation"		}
				{ L	"Link handling"			}
				{ V	"Various"			}
    }



# List of options and values to preset				conf_list
# ====================================
#	This list only serves for initialisation, it is then destroyed .
#	    It defines and configures option checkbuttons and control variables.
#
#	The sequence of the items determines:
#	  - the sequence of the corresponding arguments in the call to rsync
#	  - the relative sequence of the display of checkboxes within each
#		group of the "More ..." display (main panel service panel)
#
#   Sections of the list
#   --------------------
#	1 .options / checkboxes to configure in the main window		(M flag)
#						coords ... defined
#	2. remaining options						(M flag)
#						coords ... void
#	3. selection dialog checkboxes					(S flag)
#
#	4. control checkboxes for selection dialog			(G flag)
#
#	5. control variables						(v flag)
#
#   Flag characters
#   ---------------
#	see comments for "conf_descr"
#
#   Structure
#   ---------
#	List of configuration elements. Each element is a list of up to 6 items
#	    # 0	 ...	tag for the conf_descr entries
#	    # 1  ...	label text for item display
#	    # 2  ...	flag character list
#	    # 3  ...	default value
#	    # 4  ...	2-item list:
#			    checkboxes:		 array position coordinates
#			    contol variables:	lower and upper bounds
#	    # 5  ...	argument for rsync command-line (without 1st leading "-"

#	option-symbol	    widget-label		 flags	default	coords	arg

    variable conf_list {
	{ SYNC_LINKS		"sync links"		    MLcp0  1	{ 0 0 } -links			}
	{ HIDDEN_NODES		"sync hidden nodes"	    MXcp0  1	{ 1 0 }				}
	{ SKIP_AUXFILES		"skip auxilary files"	    MXcp0  1	{ 2 0 } -filter=-C		}
	{ UPDATE_ONLY		"sync updates only"	    MXcp0  1	{ 0 1 } u			}
	{ DELETE_NODES		"delete obsolete nodes"	    MVcp0  0	{ 1 1 } -delete			}

	{ CHECKSUM_OK		"compare checksums"	    MXcp0  0	{ }	-checksum		}
	{ SIZE_ONLY		"only compare size"	    MXcp0  0	{ }	-size-only		}
	{ UNSAFE_LINKS		"copy unsafe links"	    MLcp0   0	{ }	-copy-unsafe-links	}
	{ COPY_LINKS		"substitue content to link" MLcp0  0	{ }	-copy-links		}
	{ HARD_LINKS		"keep hard links"	    MLcp0  0	{ }	-hard-links		}
	{ KEEP_DIRLINKS		"match dir-links to dirs"   MLcp0  0	{ }	-keep-dirlixnks		}
	{ SAFE_LINKS		"exclude unsafe links"	    MLcp0  1	{ }	-safe-links		}
	{ SAVE_OWNER		"preserve owner"	    MPcp0  1	{ }	o			}
	{ SAVE_GROUP		"preserve group"	    MPcp0  1	{ }	g			}
	{ SAVE_PERMISSION	"preserve permissions"	    MPcp0  1	{ }	p			}
	{ SAVE_TIMES		"preserve times"	    MPcp0  1	{ }	t			}
	{ IGNORE_TIMES		"suppress quick-check"	    MXcp0  0	{ }	-ignore-times		}
	{ ONLY_EXISTING		"only existing nodes"	    MXcp0  0	{ }	-existing		}
	{ SKIP_EXISTING		"skip existing nodes"	    MXcp0  0	{ }	-ignore-existing	}
	{ DIR_FILTER		"per-directory filter"	    MRcp0  0	{ }	-filter=:\\\ $conf_params(perDirFilter) }
	{ USER_FILTER		"add user filter"	    MRcp0  0	{ }	-filter=.\\\ $conf_params(userFilter) }
	{ AUTO_FILTER		"use automatic filter"	    MRcp0  1	{ }	-filter=.\\\ $global_vars(TEMP_FILE) }
	{ SHOW_SKIPPED		""			    Mbr    0	{ }				}
	{ XFER_DIRECTION	""			    Mbp1   1					}

	{ HIDDEN_TOP		"show hidden nodes"	    Sc     1	{ 0 0 }				}
	{ LINK_TOP		"show links"		    Sc     1	{ 1 0 }				}

	{ CONSOLE_ECHO		"echo rsync output"	    Gc     0	{ 0 0 }				}
	{ WRITE_LOG		"write rsync log"	    Gc     1	{ 1 0 }				}

	{ SYNC_TIMEOUT	"Rsync server setup timeout"	    Gvs    $conf_params(syncTimeout)	{ 10 300 }  }
	{ MODIFY_WINDOW	"Time-stamp comparison tolerance"   Gvs    $conf_params(clockMargin)	{ 0 30 }    }
	{ REL_PATH		""			    vp1	   ""					}
	{ SYNC_ZONE		""			    vp1	   ""					}
    }



# Array of configuration descriptors
# ==================================
#	Initialised from conf_list and group list, personalised from configuration file

    variable	conf_descr	;# describe configuration item for tag <tag>	
			# (<tag>,TEXT)    widget text, may be ""
			# (<tag>,FLAGS)	  flag character list
			#		    - owner panel (caps used):
			#			M ... main panel
			#			    X ... sel. criteria
			#			    P ... property preserv.
			#			    D ... dir node choice
			#			    R ... sel rules
			#			    L ... link handling
			#			    V ... various
			#				must be directly after M (MX, MP, ...)
			#			S  ... select dialog
			#			G  ... general settings dialog
			#		    - item type (lower case used):
			#			b  ... normal button
			#			c  ... checkbutton
			#			v  ... control variable
			#			s  ... units are seconds
			#			r  ... (button only) subject to preset action
			#		    - profile selection (2-character sequences p<n>
			#			p0 ... always include in profile
			#			p1 ... include in profile is sync-zone selected
			# (<tag>,ARG)	  rsync option name (without first "-")
			#
			# (<nn>)	  tag for seq. number <nn> (0 and on)
			#
			# (<char>,GROUP)  group descriptor
			# (GROUPS)	  list of group-descriptor codes


# Array of default values and characteristics for conf_descr items
# ================================================================
#	Initialised from conf_list and group list, 
#	personalised from configuration file

    variable	conf_default	;# value (list)  attached to conf_descr <tag>
			# (<tag>,VALUE)   default value
			# (<tag>,PROPS)   default properties (list)
			#		    - for checkbuttons (button not 
			#			     configured if undefined
			#	item #0		column for placing checkbutton
			#	item #1		row for placing checkbutton
			#		    - for control variables
			#	item #0		minimum value
			#	item #1		maximum


# Array of configuration parameters
# =================================
#	Initialised from X-resources (see app-defaults and list at the
#	beginning of proc InitConfig
 
    variable	conf_params	;# global configuration parameters (data base)



# Initialise a variable
# ---------------------
#	Called twice for each configuration variable:
#	    1. During program setup for initialisation to default configuration
#	    2. During personalisation from the configuration file

    proc InitVariable { p_tag p_value p_check } {
			   # p_tag	variable tag
			   # p_value	default value
			   # p_check	"" if no check, else message header

	variable	conf_descr
	variable	conf_default

	global		global_vars
	global		button_value
	global		button_coord
	global		initial_value


	    # Define the variable

	if { [ string first "v" $conf_descr($p_tag,FLAGS) ] >= 0 } {

		# Initialising a global state variable

	    if [ info exists conf_default($p_tag,PROPS) ] {
		set x_min [ lindex $conf_default($p_tag,PROPS) 0 ]
		set x_max [ lindex $conf_default($p_tag,PROPS) 1 ]
	    } else {
		set x_min ""
		set x_max ""
	    }
	    set x_variable 1
	} else {

		# Initialising a button or checkbutton control variable

	    set x_min 0
	    set x_max 1
	    set x_variable 0
	    set x_col ""
	    set x_row ""
	    if [ regexp {([^,]+),([^,]+),([^,]+)$} $p_value "" \
							x_temp x_col x_row ] {
		set button_coord($p_tag) [ list $x_col $x_row ]
		set p_value $x_temp
	    }
	}

	    # Check the validity of the variable

	if { $p_check != "" } {
	    if { ( $x_min != "" ) && ( ( $p_value < $x_min ) || \
						( $p_value > $x_max ) ) } {
		set x_reply [ Dialog::DialogMessage . $p_check "" [ list \
		    "Bad value for control variable \"$p_tag\"" \
		    "\\40value = $p_value, must be >= $x_min and <= $x_max" \
		    "\\10" "Assuming value = $x_min" "\\10"] { Continue Kill } ]
		if { $x_reply != 0 } { MainAux::MainKill . 1 }
		set p_value 0
	    }
	    if { $x_variable != 0 } {
		set global_vars($p_tag) $p_value
	    } else {
		set button_value($p_tag) $p_value
	    }
	    set initial_value($p_tag) $p_value
	}
    }


# Define all needed fonts
# -----------------------

    proc SetFonts { } {

	variable	conf_params

	set x_ok [ AuxProcs::DefineParameters ::Config::conf_params "" { \
		{ fontFamilies  } { fontSizes1    } { fontSizes2    } } ]
	if { $x_ok == 0 } {
	    font create temp -family Helvetica -size 14 -weight bold
	    option add *Label.font	temp
	    option add *Button.font	temp 
	    Dialog::DialogMessage . Warning "" [ list \
		{One of the font definition resources is missing!} \
		{\40(resources fontSizes1, fontSizes2, fontFamilies)} ] "Kill"
	    exit
	}

	if { [ tk scaling ] > 1.1 } {
	    set conf_params(fontSizes) $conf_params(fontSizes1)
	} else {
	    set conf_params(fontSizes) $conf_params(fontSizes2)
	}

	unset conf_params(fontSizes1)
	unset conf_params(fontSizes2)

	font create standard -family [ lindex $conf_params(fontFamilies) 0 ] \
		-size [ lindex $conf_params(fontSizes) 1 ] -weight bold
	font create normal -family [ lindex $conf_params(fontFamilies) 0 ] \
		-size [ lindex $conf_params(fontSizes) 1 ] -weight normal
	font create title -family [ lindex $conf_params(fontFamilies) 0 ] \
		-size [ lindex $conf_params(fontSizes) 2 ] -weight bold
	font create small -family [ lindex $conf_params(fontFamilies) 0 ] \
		-size [ lindex $conf_params(fontSizes) 0 ] -weight normal
	font create fixed -family [ lindex $conf_params(fontFamilies) 1 ] \
		-size [ lindex $conf_params(fontSizes) 0 ] -weight normal
	font create marked -family [ lindex $conf_params(fontFamilies) 0 ] \
		-size [ lindex $conf_params(fontSizes) 1 ] \
					-weight normal -slant italic
	font create zapf -family Zapfdingbats -size \
			[ expr [ lindex $conf_params(fontSizes) 1 ] + 4 ]
    }



# Initialise the configuration data
# ---------------------------------
#	- get common parameters from the X-resource data base
#	- expand file names
#	- create name for temporary file
#	- digest group_list and conf_list into conf_descr, destroy these 2 lists

    proc InitConfig { } {

	variable	conf_params
	variable	conf_descr
	variable	conf_default

	variable	conf_list
	variable	group_list
	variable	version_string

	global		global_vars
	global		button_coord


	    # Initialise resource data base and global parameters
	    # ---------------------------------------------------

	option readfile [ file join $global_vars(LIB_DIR) app-defaults ] \
								startupFile
	SetFonts

	AuxProcs::DefineParameters ::Config::conf_params . { \
		{ windowMargin	} { buttonSpacing }  \
		{ configFile	} { tempFile	  } { syncTimeout   } \
		{ logFile       } \
		{ logWarning	} { userFilter    } { perDirFilter  } \
		{ clockMargin   } { rpmName	  } { dummyZone	    } \
		{ removableMedia }	}

	set conf_params(profileLabel) "profile = "

	set x_x 0
	foreach x_item { logFile configFile userFilter tempFile } { 
	    incr x_x
	    set x_file [ AuxProcs::ExpandPath $conf_params($x_item) ]
	    set conf_params($x_item) $x_file

	    if [ file exists $x_file ] {

		# Error if file exists and is not readable
		   # error if it is not writable for logfile and configFile

		if { [ file readable $x_file ] == 0 } {
		    Dialog::DialogMessage . Error "" [ list \
			"The file \"$x_file\" (X resource \"$x_item\")" \
			{\40needs read access permissions} \
			{\8} {You need to correct your configuration !} ] "Kill"
		    exit
		}
		if { ( $x_x <= 2) && ( [ file writable $x_file ] == 0 ) } {
		    Dialog::DialogMessage . Error "" [ list \
			"The file \"$x_file\" (X resource \"$x_item\")" \
			{\40needswrite access permissions} \
			{\8} {You need to correct your configuration !} ] "Kill"
		    exit
		}
	    } else {

		# Error if file does not exist and cannot be created

		set x_dir [ file dirname $x_file ]
		if { [ file writable $x_dir ] == 0 } {
		    Dialog::DialogMessage . Error "" [ list \
			"The file \"$x_file\" (X-resource \"$x_item\")" \
			{\40does not exist and cannot be created} \
			{\8} {You need to correct your configuration !} ] "Kill"
		    exit
		}
	    }
	}
	if { [ file writable [ file dirname $conf_params(configFile) ]] == 0 } {
	    Dialog::DialogMessage . Error "" [ list \
			"Cannot create files in the directory that contains\
						the configuration file !" \
			"\\40$conf_params(configFile)" \
			{\8} {You need to correct your configuration !} ] "Kill"
	    exit
	}


	    # Make name of temporary file
	    # ---------------------------

	set x_lead [ file dirname $::Config::conf_params(tempFile) ]
	set x_temp $x_lead/[ file tail $::Config::conf_params(tempFile) ]_
	foreach x_file [ glob -nocomplain ${x_temp}* ] {
	    catch { file delete $x_file }
	}
	set global_vars(TEMP_FILE) "${x_temp}[ pid ]" 

	set global_vars(POPUP) ""
	set global_vars(HOME) $::env(HOME)
	cd
	Dialog::DialogBindings


	    # Compose version string if executing from rpm install position
	    # -------------------------------------------------------------

	set x_fid [ open "|/bin/rpm -q --list $conf_params(rpmName)" r ]
	while { [ gets $x_fid x_line ] >= 0 } {
	    if { $x_line != $global_vars(MAIN_SCRIPT) } continue
	    set x_vers [ exec rpm -q usync ]
	    regsub {usync} $x_vers "V" x_vers
	    set x_temp [ exec rpm -q --info usync | grep "Build Date" ]
	    regsub {^.*Date:\s+} $x_temp "" x_temp

	    set version_string "$x_vers   [ clock format [ clock scan \
					$x_temp ] -format "%d.%m.%Y - %Hh%M" ]"
	    break
	}
	close $x_fid

	set x_vers [ exec rpm -q usync ]
	regsub {usync} $x_vers "V" x_vers
	set x_temp [ exec rpm -q --info usync | grep "Build Date" ]
	regsub {^.*Date:\s+} $x_temp "" x_temp

	set version_string "$x_vers   [ clock format [ clock scan $x_temp ] \
						-format "%d.%m.%Y - %Hh%M" ]"


	    # Initialise the configuration tables (options, variables)
	    # -----------------------------------

	set x_x 0
	foreach x_conf $conf_list {
	    set x_tag				[ lindex $x_conf 0 ]
	    set conf_descr($x_tag,TEXT)		[ lindex $x_conf 1 ]
	    set conf_descr($x_tag,FLAGS)	[ lindex $x_conf 2 ]
	    set x_value				[ lindex $x_conf 3 ]
	    set x_props				[ lindex $x_conf 4 ]
	    set x_arg				[ lindex $x_conf 5 ]

	    set x_flags $conf_descr($x_tag,FLAGS)

		# force substitution of variables in the x_arg item,
		# make sure that back-slashed space remain get the \ again

	    if { [ string first "$" $x_value ] >= 0 } {
		eval "set x_value \"$x_value\""
		regsub -all { } $x_value "\\ " x_value
	    }
	    set conf_default($x_tag,VALUE) $x_value
	    if { [ llength $x_props ] != 0 } \
				{ set conf_default($x_tag,PROPS) $x_props }
	    if { $x_arg != "" } { set conf_descr($x_tag,ARG) $x_arg }
	    set conf_descr($x_x) $x_tag
	    incr x_x
	}
	unset conf_list

	set x_x 0
	foreach x_group $group_list {
	    append conf_descr(GROUPS) [ lindex $x_group 0 ]
	    set conf_descr($x_x,GROUP) $x_group
	    incr x_x
	}
	unset group_list
    }
}
