package provide ConfigFile



# Confiuration file read and write procedures
# ===========================================

namespace eval ConfigFile {



# Personalise the option settings from the configuration file
# -----------------------------------------------------------
#	Return list of activateable synchronisation zones

    proc ReadConfig {} {

	global		sync_zones
	global		button_value
	global		button_coord
	global		global_vars
	global		host_table

	variable	::Config::conf_descr
	variable	::Config::conf_params
	variable	::MainPanel::main_widgets

	set x_activate ""	;# "" until a configurable sync-object is found
	set x_seen ""		;# list of variables seen
	set x_version ""	;# no version seen
	set x_skipProfile ""	;# not skipping old profiles

	if [ info exists sync_zones ] { unset sync_zones }
	set x_header "\"$conf_params(configFile)\" file error"


	    # Loop through the lines of the configuration file

	set x_fid [ open $conf_params(configFile) "r" ]
	while { [ gets $x_fid x_line ] >= 0 } {

		# Append continuation lines if necessary
		# --------------------------------------
		#	note: \u005C = backslash character (\)

	    while { [ string range $x_line end end ] == "\u005C" } {
		set x_line [ string trimright $x_line " \t\u005C" ]
		if { [ gets $x_fid x_temp ] < 0 } break
		append x_line [ string trimleft $x_temp ]
	    }
	    if [ regexp {^\s*$} $x_line ] continue
	    if [ regexp {^\s*#} $x_line ] continue


		# If it is a group header, just remember the label
		# ------------------------------------------------

	    if [ regexp {^\[([^\[\]]+)\]$} $x_line "" x_label ] continue


		# Not a group header, treat as attribute-value pair
		# -------------------------------------------------
		    # x_label ... current group header (previously defined)

	    if { $x_label == "SyncZones" } {
		set x_value $x_line
	    } elseif { [ regexp {^([^=]+)=(.*)$} $x_line "" \
						x_attr x_value ] == 0  } {
		set x_reply [ Dialog::DialogMessage . $x_header "" [ list \
			"Bad syntax in line" \
			"\\40$x_line" \
			"\\10" "Skipping this line" "\\10"] { Continue Kill } ]
		if { $x_reply != 0 } { MainAux::MainKill . 1 }
		continue
	    }
    if { $x_label == "SyncObjects" } {	;### temporary code for backward
	set x_line $x_value		;###			compatibility
	set x_switch "SyncZones"	;###
    } else {				;###
	set x_switch $x_label		;###
    }					;###

	    switch $x_switch {

		""	continue	;# skipping entries of non-valid group


		"ControlVariables" {

		    # Control variables (button state, direction, debug)
		    # --------------------------------------------------
			# items must be <attribute>=<value> with
			#	<attribute> must be a valid "conf_descr" index
			#	<value> must be within bounds

		    set x_descr [ lsearch [ array names conf_descr ] \
								$x_attr,FLAGS ]
		    if { $x_descr < 0 } {
			set x_reply [ Dialog::DialogMessage . $x_header "" \
								[ list \
				"Bad state variable name \"$x_attr\"" \
				"\\10" "Skipping this item" "\\10"] \
							{ Continue Kill } ] 
			if { $x_reply != 0 } { MainAux::MainKill . 1 }
			continue	;# skip the line with the bad variable
		    }
		    Config::InitVariable $x_attr $x_value $x_header
		    lappend x_seen $x_attr
		}


		"SyncZones" {

		    # Synchronisation object
		    # ----------------------
			# items must be <sync_id>=<path_1>,<path_2> with
			#   - <sync_id> is a dummy label string (not checked)
			#   - <path_1> a valid path on the client (local host)
			#   - <path_2> either
			#	    a valid path on the client (local host)
			#		 or
			#	    <rsync_label>@<host_name> with
			#		<host_name> a valid ip address
			#		<rsync_label> a valid rsync label on
			#					that host

		    set x_ok 0
		    set x_flags ""


			# De-compose into fields, check syntax

		    if { [ regexp \
			{^([^@]+)@([^@*]+)(\**),([^,]+)$|^([^,*]+),([^,]+)$} \
				$x_value "" sv_path1 x_host x_dead cl_path1 \
					sv_path2 cl_path2 ] == 0 } {
			set x_reply [ Dialog::DialogMessage . $x_header "" \
									[ list \
				"Bad syntax in sync-object declaration" \
				"\\40$x_line" \
				"\\10" "Skip this declaration?" "\\10"] \
							{ Continue Kill } ]
			if { $x_reply != 0 } { MainAux::MainKill . 1 }
			continue	;# skip the line with the invalid syntax
		    }

		    if { $x_host != "" } {
			set cl_path $cl_path1
			set sv_path $sv_path1
		    } else {
			set cl_path $cl_path2
			set sv_path $sv_path2
		    }
		    if { [ string range $cl_path 0 0 ] == "#" }	\
				{ set x_comment 1 } else { set x_comment 0 }


			# Check the client rsync argument (local path)
			# --------------------------------------------

		    if { ( [ file isdirectory [ AuxProcs::ExpandPath \
				$cl_path ] ] == 0 ) && ( $x_comment == 0 ) } {
			set x_reply [ Dialog::DialogMessage . $x_header "" \
									[ list \
			    "Client-side path does not exist" \
			    "\\40$cl_path" \
			    "\\40$x_line" \
			    "\\10" "Skipping this declaration" "\\10"] \
							{ Continue Kill } ]
			if { $x_reply != 0 } { MainAux::MainKill . 1 }
			continue	;# skip the line with the invalid path
		    }

		    if { ( $x_host != "" ) && ( $x_comment == 0 ) } {


			# rsync object (server) is on remote host, check it
			# -------------------------------------------------

			if { $global_vars(NETWORK) == 0 } {
			    set host_table($x_host,FLAGS) "x"
			} elseif { $x_dead != "" } {
			    if [ info exists host_table($x_host,FLAGS) ] {
				if { $host_table($x_host,FLAGS) == "" } \
								{ set x_ok 1 }
			    } else {
				if { [ Actions::HostIsup $x_host ] <= 0 } {
				    set host_table($x_host,FLAGS) "x"
				} else {
				    set host_table($x_host,FLAGS) ""
				    set x_ok 1
				}
			    }
			} elseif { [ info exists host_table($x_host,FLAGS) ] \
									== 0 } {
			    if { [ Actions::HostIsup $x_host ] <= 0 } {
				set x_reply [ Dialog::DialogMessage . Question \
								"" [ list \
				    "The host does not respond" \
				    "\\60service = \"${sv_path}@$x_host\"" \
				    "\\10" \
				    "\\30- accept but suspend all services\
								on this host," \
				    "\\30- nevertheless accept all services\
								on this host," \
				    "\\30- or abandon execution of Usync?" ] \
				{ Suspend Accept Kill } ]
				if { $x_reply > 1 } { 
				    MainAux::MainKill . 1
				} elseif { $x_reply == 1 } {
			            set host_table($x_host,FLAGS) ""
				} else {
			            set host_table($x_host,FLAGS) "x"
				}
			    } else {
				set host_table($x_host,FLAGS) ""
				set x_ok 1
			    }
			} else {
			    if { [ string first "x" $host_table($x_host,FLAGS) \
							] < 0 } { set x_ok 1 }
			}
			if { $x_ok } {
			    set x_error [ Actions::CheckHost $x_host $sv_path ]
			    if { $x_error != "" } {
				set x_x [ lindex $x_error 0 ]
				set x_x [ string tolower $x_x 0 0 ]
				set x_x "Bad sync-object declaration, $x_x"
				set x_error [ lreplace $x_error 0 0 $x_x ]

				set x_x "\\40[ lindex $x_error 1 ]"
				set x_error [ lreplace $x_error 1 1 $x_x ]
				lappend x_error "\\10" \
				"Skipping the declaration \"$x_line\"" "\\10"
				set x_reply [ Dialog::DialogMessage . \
				    $x_header "" $x_error { Continue Kill } ]
				if { $x_reply != 0 } { MainAux::MainKill . 1 }
				continue
			    }
			    if [ Actions::CheckZone $x_host $sv_path ] {
				append x_flags "r"
			    }
			}
		    } elseif { $x_comment == 0 } {


			# rsync object (server) is a local path, check it
			# -----------------------------------------------

			set x_path [ AuxProcs::ExpandPath $sv_path ]
			foreach x_item $conf_params(removableMedia) {
			    if { [ string first "t" $x_flags ] >= 0 } break
			    if { [ string first $x_item $x_path ] == 0 } \
							{ append x_flags "t" }
			}
			if { [ file isdirectory $x_path ] == 0 } {
			    if { [ string first "t" $x_flags ] < 0 } {

				# Invalid path

				set x_reply [ Dialog::DialogMessage . \
							$x_header "" [ list \
				    "Server-side path does not exist" \
				    "\\40$sv_path" \
				    "\\40$x_line" \
				    "\\10" "Skipping this declaration" "\\10"] \
							{ Continue Kill } ]
				if { $x_reply != 0 } { MainAux::MainKill . 1 }
				continue ;# skip the line with the invalid path
			    }
			} else {
			    set x_ok 1
			}
		    }
		    set x_zone [ AuxProcs::DefineZone $cl_path $sv_path \
							$x_host $x_flags ]
		    if { $x_ok != 0 } { lappend x_activate $x_zone }
		}


		"Profiles" {

		    # A profile
		    # ---------
			# items must be <name>=<value_list>
			#     - <name> is a label string (not checked)
			#     - <value_list> = 
			#	    "<tag>=<value>", ... ,"<tag>=<value>"
			#     - <tag> = a valid "conf_descr" tag
			#     - <value> = a valid value for this item

		    if { ( $x_version != "" ) && ( $x_numVersion < 384 ) } {
			if { $x_skipProfile != "" } continue
			set x_reply [Dialog::DialogMessage . $x_header \
								"" [ list \
			    "Version of configuration file = \"$x_version\"," \
			    {\40profiles from that version cannot be restored} \
			    {\10} {If you continue, all profiles are lost,} \
			    {\40they will need to be newly created} ] \
							{ Continue Kill } ] 
			if { $x_reply != 0 } { MainAux::MainKill . 1 }
			set x_skipProfile x
			continue
		    }

		    set x_list {}
		    append x_value ","		;# facilitate match at end
		    while { $x_value != "" } {
			set x_temp ""
			if [ regexp {^(\"[^=]+=[^\"]*\"),(.*)$} \
						$x_value "" x_chunk x_value ] {
			    regexp {^\"([^=]+)=} $x_chunk "" x_tag
			    if { [ lsearch [ array names conf_descr ] \
							$x_tag,FLAGS ] < 0 } {
				set x_reply [Dialog::DialogMessage . $x_header \
								"" [ list \
				    "Bad state variable name \"$x_tag\"" \
				    "\\10" "Skipping preset item \"$x_attr\"" \
				    "\\10"] { Continue Kill } ] 
				if { $x_reply != 0 } { MainAux::MainKill . 1 }
				set x_list ""
				break
			    }
			    lappend x_list $x_chunk
			} else {
			    set x_reply [Dialog::DialogMessage . $x_header \
								"" [ list \
				"Bad syntax in profile definition, line" \
				"\\40$x_line" \
				"\\10" "Skip this profile?" "\\10"] 
							{ Skip Kill } ]
			    if { $x_reply != 0 } { MainAux::MainKill . 1 }
			    set x_list ""
			    break
			}
		    } ; # while ... loop through profile components
		    if { $x_list != "" } {
			$main_widgets(PRESET_MENU) add command \
				-label "$conf_params(profileLabel)$x_attr" \
				-command [ list MainAux::PresetProfile $x_list ]
		    }
		}


		"Version" {
		    if { $x_attr == "Version" } {
			if [ regexp {[^-]*-([^-]+)-(\S+)} $x_value "" \
							x_version x_release ] {
			    regsub -all {\.} $x_version "" x_numVersion
			}
			
		    }
		}

		default {
		    set x_reply [ Dialog::DialogMessage . $x_header "" [ list \
			"Bad group header label \"\[$x_label\]\"" "\\10" \
			"Skipping the entire group" "\\10"] { Continue Kill } ]
		    if { $x_reply != 0 } { MainAux::MainKill . 1 }
		    set x_label ""
		}

	    } ;# end switch
	} ; # while ... loop through lines
	catch { close $x_fid }


		# Check for variables not contained in the config file

	set x_message ""
	foreach x_item [ array names conf_descr "*,FLAGS" ] {
	    regsub {,FLAGS} $x_item "" x_tag
	    if { [ lsearch -exact $x_seen $x_tag ] < 0 } {
		if { $x_message != "" } { append x_message ", " }
		append x_message $x_tag 
	    }
	}
	if { $x_message != "" } {
	    set x_reply [ Dialog::DialogMessage . $x_header "" [ list \
		    {The following checkbuttons were not defined in the\
							configuration file.} \
		    {They have now been set to their default values:} \
		    {\10} "\\40$x_message" \
		    {\10} {Consult "More ..." and its help functions for\
			    information on the buttons!} ] { Continue Kill } ]
	    if { $x_reply != 0 } { MainAux::MainKill . 1 }
	}
	return $x_activate
    }



# Save the configuration
# ----------------------
#	Move old configuration file to save name, recreate new and updated
#	configuration file 
#
# !!! if this module has compilation error, no message is given,
# !!! the exit hook just is skipped without further notice

    proc SaveConfig { } {

	global		sync_zones
	global		button_value
	global		button_coord
	global		global_vars
	global		host_table

	variable	::Config::conf_descr
	variable	::Config::conf_params
	variable	::Config::version_string

	variable	::MainPanel::main_widgets

	set x_profile [ string length $conf_params(profileLabel) ]
	set x_file $global_vars(TEMP_FILE)x
	set x_fid [ open $x_file "w" ]

	    # Initial comment

	puts $x_fid "# This file is automatically generated each time Usync\
							terminates normally !"
	puts $x_fid "# Manually modifying the file does no make sense while\
							Usync is running !\n"


	    # Version having created the configuration file

	regsub -all {\s+} $version_string " " x_temp
	puts $x_fid "\[Version\]"
	puts $x_fid "Version=$x_temp"
	puts $x_fid "Host=$global_vars(CLIENT_HOST)"


	    # Save control variables

	puts $x_fid "\n\[ControlVariables\]"

	for { set x_x 0 } { [ info exists conf_descr($x_x) ] } { incr x_x } {
	    set x_tag $conf_descr($x_x)
	    if { [ string first "v" $conf_descr($x_tag,FLAGS) ] >= 0 } {
		puts $x_fid "$x_tag=$global_vars($x_tag)"
	    } else {
		if [ info exists button_coord($x_tag) ] {
		    set x_coord ",[ lindex $button_coord($x_tag) 0 ],[ \
					lindex $button_coord($x_tag) 1 ]"
		} else {
		    set x_coord ""
		}
		puts $x_fid "$x_tag=$button_value($x_tag)$x_coord"
	    }
	}


	    # Save synchronisation zones

	puts $x_fid "\n\[SyncZones\]"
	foreach x_item [ array names sync_zones *,NUMBER ] {
	    regsub {,NUMBER} $x_item "" x_zone
	    set x_number $sync_zones($x_item)
	    set x_aux($x_number) $x_zone
	}
	for { set x_seq 0 } { [ info exists x_aux($x_seq) ] } { incr x_seq } {
	    set x_zone $x_aux($x_seq)
	    set x_host $sync_zones($x_zone,SV_HOST)
	    set sv_base $sync_zones($x_zone,SV_BASE)
	    set cl_base $sync_zones($x_zone,CL_BASE)
	    set x_text ""
	    if { $x_host != "" } { 
		set x_text "@$x_host"
		if { [ string first "x" $host_table($x_host,FLAGS) ] >= 0 } \
							{ append x_text "*" }
		set x_host $x_temp
	    }
	    puts $x_fid "${sv_base}${x_text},$cl_base"
	}


	    # Save profiles

	puts $x_fid "\n\[Profiles\]"
	set x_menu $main_widgets(PRESET_MENU)
	set x_last [ $x_menu index end ]
	set x_init "x"
	for { set x_x 0 } { $x_x <= $x_last } { incr x_x } {
	    if { [ $x_menu type $x_x ] != "command" } continue
	    set x_command [ $x_menu entrycget $x_x -command ]
	    set x_arg [ lindex $x_command 1 ]
	    if { [ llength $x_arg ] <= 1  } continue
	    if [ regexp {^\d+$} $x_arg ] continue
	    set x_line "[ $x_menu entrycget $x_x -label ]="
	    set x_line [ string range $x_line $x_profile end ]
	    foreach x_item $x_arg {
		if { [ string range $x_line end end ] != "=" } \
						{ set x_item ",$x_item" }
		set x_line "$x_line$x_item"
	    }
	    if { $x_init == "" } { 
		puts $x_fid ""
	    } else {
		set x_init ""
	    }

		# Break this line in chunks if necesary

	    while { [ string length $x_line ] > 75 } {
		set x_chunk [ string range $x_line 0 70 ]
		set x_sep [ string last "," $x_chunk ]
		if { $x_sep < 0 } break
		puts $x_fid "[ string range $x_line 0 $x_sep ] \\"
		incr x_sep
		set x_line [ string range $x_line $x_sep end ]
	    }
	    puts $x_fid $x_line
	}


	    # Finalise, switch files

	catch { close $x_fid }
	set x_config $conf_params(configFile)
	if [ file exists $x_config ] {
	    file attributes $x_file -permissions [ file attributes \
						$x_config -permissions ]
	    file attributes $x_file -owner [ file attributes $x_config -owner ]
	    file attributes $x_file -group [ file attributes $x_config -group ]
	    file rename -force $x_config ${x_config}~
	}
# Trying to catch a once-in-3-months error"
	if [ file exists $x_config ] {
	    puts " xxx old config not renamed"
	    exit
	}
	if { [ file exists $x_file ] == 0 } {
	    puts " xxx new config does not exist"
	    exit
	}
	if { [ file size $x_file ] == 0 } {
	    puts " xxx new config is empty"
	    exit
	}
	file rename -force $x_file $x_config
    }
}