package provide HelpAux



# Utilities for help service, popup help utility
# ==============================================


namespace eval HelpAux {


# Help-module internal auxiliary procedures
# =========================================


# Define list of bracketed zones
# ------------------------------
#	For each zone, add an item:
#	   item #0 ... begin of zone (character position)
#		#1 ... end of zone (character position)
#		#2 ... zone type character
#			#3 ... #5 only for link items
#		#3 ... link tag
#		#4 ... character position of target or of reference
#		#5 ... 0 / 1 if defining / referencing a link

    proc HelpZones { p_text p_tags } {
			  # p_text	paragraph text to treat !! reference !!
			  # p_tags	list of recognised tag characters

	upvar 1 $p_text x_text

	set zone_list ""
	set x_pos 0
	while { 1 } {

	        # Find begin of bracket expression

	    set x_pos [ HelpAux::HelpCode $x_text $x_pos "." ]
	    if { $x_pos == "" } break

	    set x_0 [ expr $x_pos - 3 ]
	    set x_x [ expr $x_0 + 1 ]
	    set x_type [ string range $x_text $x_x $x_x ]
	    if { [ string first $x_type $p_tags ] < 0 } continue

	        # Handle special case of hypertext link
		#	note: \u005b is "[", \u005d is "]"

	    if { $x_type == "l" } {
		if { [ string range $x_text $x_pos $x_pos ] != "\[" } continue
		if [ regexp {^\u005b(.+?)\u005d} \
			     [ string range $x_text $x_pos end ] "" x_point ] {
		    incr x_pos [ expr [ string length $x_point ] + 2 ]
		}
	    }

	        # Find end of bracket expression

	    set x_pos [ HelpAux::HelpCode $x_text $x_pos "/$x_type" ]
	    if { $x_pos == "" } break
	    set x_1 [ expr $x_pos - 4 ]


	    # Got a correctly bracketed region
	    # --------------------------------

	    set x_text [ string replace $x_text $x_1 [ expr $x_1 + 3 ] ]

		    # Treat hypertext link

	    if { $x_type == "l" } {
		set x_x [ expr $x_0 + 3 ]
		set x_y [ expr [ string length $x_point ] + 2 ]
		set x_text [ string replace $x_text $x_x \
						[ expr $x_x + $x_y - 1 ] ]
		incr x_1 -$x_y
		incr x_pos -$x_y

		if { $x_1 == [ expr $x_0 + 3 ] } {
		    set x_link $x_pos
		    set x_flag 1
		} else {
		    set x_link $x_0
		    set x_flag 0
		}
	    }

	    set x_text [ string replace $x_text $x_0 [ expr $x_0 + 2 ] ]
	    incr x_1 -3
	    incr x_pos -7
	    if { $x_type == "l" } {
		lappend zone_list [ list $x_0 $x_1 $x_type $x_point $x_link \
								$x_flag ]
	    } else {
		lappend zone_list [ list $x_0 $x_1 $x_type ]
	    }
	}
	return $zone_list
    }



# Get a paragraph from the help file
# ----------------------------------
#	Read successive lines and assemble them into a paragraph.
#	  - a paragraph is terminated by an empty line or a trailing \n
#	  - drop trailing newline
#	  - drop leading white spaces
#
#	Return a 2-item list:
#	    EOF  ... item #0 = "", item #1 = -1
#	    else ... item #0 = text, item #1 = position at begin of text


    proc HelpRead { p_fid } {
			  # p_fid	channel, must be open
	set x_text ""
	while { 1 } {
	    set x_temp [ tell $p_fid ]
	    set x_more [ gets $p_fid x_line ]
	    if { $x_more >= 0 } {
		set x_line [ string trimleft $x_line ]
		incr x_temp [ expr $x_more - [ string length $x_line ] ]
		if { [ string range $x_line 0 0 ] == "#" } continue
		set x_line [ string trimright $x_line "\n" ]
		if { $x_line == "" } {
		    if { $x_text == "" } continue
		    set x_end 1
		} elseif { [ string range $x_line end-1 end ] == "\\n" } {
		    set x_end 1
		    set x_line [ string range $x_line 0 end-2 ]
		} else {
		    set x_end 0
		}
		if { $x_text == "" } { set x_pos $x_temp }
		append x_text $x_line
		if { $x_end == 0 } continue
	    } else {
		if { $x_text == "" } { return [ list "" -1 ] }
	    }
	    return [ list $x_text $x_pos ]
	}
    }



# Find first occurrance of a zone tag [x] or [/x]
# -----------------------------------------------
#	Do not accept backslashed \[
#	Return position after zone tag

    proc HelpCode { p_text p_pos p_pattern } {
			  # p_text	text to search in
			  # p_pos	offset to first character to consider
			  # p_pattern   tag pattern (  . or /x  )

	while { 1 } {
	    if { [ regexp "^(.*?)\\u005b$p_pattern\\u005d" [ string range \
			$p_text $p_pos end ] "" x_prec ] == 0 } { return "" }
	    incr p_pos [ expr [ string length $x_prec ] + [ string length \
							$p_pattern ] + 2 ]
	    if { [ string range $x_prec end end ] != "//" } break
	}
	return $p_pos
    }



# Select-mark a text line, clear previous selection
# -------------------------------------------------

    proc HelpSelection { p_panel p_range } {
			  # p_panel	help text panel
			  # p_range	selection zone list

	$p_panel tag remove sel 1.0 end
	if { [ llength $p_range ] > 1 } {
	    set x_first [ lindex $p_range 0 ]
	    set x_last [ lindex $p_range 1 ]
	} else {
	    regexp {(\d+)\.} $p_range "" x_line
	    set x_first $x_line.0
	    set x_last $x_line.end
	}
	$p_panel tag add sel $x_first $x_last
    }



# Callback for find utility
# -------------------------

    proc DoFind { p_pattern p_text { p_mode "" } } {
			  # p_pattern	text panel iwth pattern string
			  # p_text	help text panel
			  # p_mode	function to execute
			  #		  first ... search forward from begin
			  #		  next  ... search forward
			  #		  prec  ... search backward
			  #				from current selection
			  #		  "" and else ... toggle find visibility

	switch $p_mode {
	    "first" { 
		set x_start 1.0
		set x_dir "-forward"
	    }
	    "next" {
		set x_start [ $p_text tag nextrange sel 1.0 ]
		if { [ llength $x_start ] == 0 } {
		    set x_start 1.0 
		} else {
		    set x_start [ lindex $x_start 1 ]
		}
		set x_dir "-forward"
	    }
	    "prec" {
		set x_start [ $p_text tag nextrange sel 1.0 ]
		if { [ llength $x_start ] == 0 } {
		    set x_start end 
		} else {
		    set x_start [ lindex $x_start 0 ]
		}
		set x_dir "-backward"
	    }
	    default {
		set x_parent [ winfo parent $p_text ]
		set x_scroll [ lindex [ $p_text cget -yscrollcommand ] 0 ]
		set x_frame [ winfo parent $p_pattern ]
		if [ winfo exists $x_parent.textbar ] {
		    destroy $x_parent.textbar
		    grid forget $x_frame
		    set x_span 3
		} else {
		    set x_span 1
		    frame $x_parent.textbar -height 2 -borderwidth 2 \
							-relief groove
		    grid $x_parent.textbar -column 3 -columnspan 2 \
							-row 1 -sticky ew
		    grid $x_frame -column 3 -columnspan 2 -row 2 -padx 4 \
							-sticky news
		}
		grid configure $p_text -rowspan $x_span
		grid configure $x_scroll -rowspan $x_span
		return
	    }
	}


	set x_string [ string trimright [ $p_pattern get 1.0 end ] "\n" ]
	if { $x_string == "" } return
	set x_x [ $p_text search $x_dir -count x_count -exact -- \
							$x_string $x_start ]
	if { $x_x != "" } {
	    regexp {(\d+)\.(\d+)} $x_x "" x_0 x_1
	    $p_text tag remove sel 1.0 end
	    $p_text tag add sel $x_0.$x_1 $x_0.[expr $x_1 + $x_count]
	    $p_text see $x_0.$x_1
	}
    }



# Module-internal callback procedures
# ===================================


# Scrollbar set callback handler for toc
# --------------------------------------

    proc HelpTOCscroll { p_toc p_offset p_size } {
			  # p_toc	toc listbox widget
			  # p_offset	scroll offset (added by TK)
			  # p_size	(added by TK)

	regsub {\.[^\.]+$} $p_toc "" x_dialog
	if { ( $p_offset == 0 ) && ( $p_size == 1 ) } {

		# No scrollbar - put vertical bar in place of scrollbar

	    if { [ winfo exists $x_dialog.border ] == 0 } {
		set x_list [ grid info $p_toc ]
		set x_row [ AuxProcs::ListArgument $x_list -row ]
		set x_col [ expr [ AuxProcs::ListArgument $x_list -column ] + 1]
		frame $x_dialog.border -width 2 -borderwidth 2 -relief groove
		grid  $x_dialog.border -column $x_col -row $x_row -sticky ns
	    }
	} else {

		# Need scrollbar, put it unless it exists already

	    if { [ winfo exists $x_dialog.contentscroll ] == 0 } {
		set x_list [ grid info $p_toc ]
		set x_row [ AuxProcs::ListArgument $x_list -row ]
		set x_col [ expr [ AuxProcs::ListArgument $x_list -column ] + 1]
		set x_bg [ $x_dialog cget -bg ]
		scrollbar $x_dialog.contentscroll -command "$p_toc yview"
		grid $x_dialog.contentscroll -column $x_col -row $x_row \
								-sticky ns
		$x_dialog.contentscroll set $p_offset $p_size
		$p_toc configure -yscrollcommand "$x_dialog.contentscroll set"
  	    }
	}
    }



# Button-up callback for TOC selection
# ------------------------------------
#	Jump to corresponding help text paragraph
#	Clear hypertext history

    proc HelpButtonup { p_toc p_target } {
			  # p_toc	toc listbox widget
			  # p_target	help target name

	variable	::HelpDialog::help_state

	set x_index [ lindex [ $p_toc curselection ] 0 ]
	if { $x_index == "" } return
	set x_pos $help_state($p_target,$x_index)
	set x_help [ winfo parent $p_toc ].text
	$x_help see $x_pos
	HelpSelection $x_help $x_pos
	set help_state($p_target,BACK_LIST) ""
	$help_state($p_target,BACK_BUTTON) configure -state disabled
    }



# Drag callback for toc separator bar
# -----------------------------------

    proc DragBorder { p_bar } {
			  #p_bar	separator bar frame widget

	    set x_parent [ winfo parent $p_bar  ]
	    set x_save [ bind $p_bar <B1-Motion> ]
	    bind $p_bar <B1-Motion> {}
	    set x_width [ winfo width $x_parent.contents ]
	    set x_x [ expr [ winfo pointerx $p_bar ] - [ winfo rootx $p_bar ] ]
	    grid columnconfigure $x_parent 0 -minsize [ expr $x_width + $x_x ]
	    after idle [ list bind $p_bar <B1-Motion> $x_save ]
    }



# Jump to a hypertext link, mark the destination
# ----------------------------------------------
#	Support going back to current position (but for a limited number of
#	levels

    proc HelpGo { p_panel } {
			  # p_panel	help text panel

	variable	::HelpDialog::help_links
	variable	::HelpDialog::help_state
	variable	::HelpDialog::help_params

	regexp {\.[^_]+_([^\.]+)\..+} $p_panel "" x_target

	    # Find text panel position where hit occured

	set x_pos [ $p_panel index current ]
	regexp {(\d+)\.(\d+)} $x_pos "" x_line x_char
	set x_pos $x_line.[ expr $x_char + 1 ]

	    # Get corresponding hypertext tag

	set x_range [ $p_panel tag prevrange link_tag $x_pos ]
	set x_start [ lindex $x_range 0 ]
	if { [ info exists help_links($x_target,$x_start) ] == 0 } return
	set x_point $help_links($x_target,$x_start)
	if { [ info exists help_links($x_target,$x_point) ] == 0 } return
	set x_link $help_links($x_target,$x_point)
	$p_panel see $x_link
	HelpSelection $p_panel $x_link

	$help_state($x_target,BACK_BUTTON) configure -state normal
	lappend help_state($x_target,BACK_LIST) $x_range
	set x_x [ expr [ llength $help_state($x_target,BACK_LIST) ] \
					- $help_params(gobackStack) - 1 ]
	if { $x_x >= 0 } { set help_state($x_target,BACK_LIST) \
			[ lreplace $help_state($x_target,BACK_LIST) 0 $x_x ] }
    }



# Go back from visiting a hypertext link
# --------------------------------------

    proc HelpBack { p_panel } {
			  # p_panel	help text panel

	variable	::HelpDialog::help_state

	regexp {\.[^_]+_([^\.]+)\..+} $p_panel "" x_target

	set x_x [ llength $help_state($x_target,BACK_LIST) ]
	if { $x_x > 0 } {
	    incr x_x -1
	    set x_range [ lindex $help_state($x_target,BACK_LIST) $x_x ]
	    set x_pos [ lindex $x_range 0 ]
	    set help_state($x_target,BACK_LIST) [ lreplace \
				$help_state($x_target,BACK_LIST) $x_x $x_x ]
	    $p_panel see $x_pos
	}
	HelpSelection $p_panel $x_range
	if { $x_x <= 0 } { $help_state($x_target,BACK_BUTTON) configure \
							-state disabled }
    }



# Support (handler, callbacks) for option popup help display
# ==========================================================


# Callback and utility to destroy any existing help-text panel
# ------------------------------------------------------------

    proc PopupClear { } {

	global		global_vars

	if { $global_vars(POPUP) != "" } { destroy $global_vars(POPUP) }
    }



# Popup help button-down event callback
# -------------------------------------

    proc PopupPress { p_field } {
			  # p_field	widget receiving the button-down hit

	global		global_vars

	if { $global_vars(POPUP) != "" } { destroy $global_vars(POPUP) }
	set x_dialog [ winfo toplevel $p_field ]
	$x_dialog configure -cursor hand1
    }



# Popup help button-up event callback
# -----------------------------------

    proc PopupButtonup { p_frame p_x p_y} {
			  # p_frame	frame that contains the checkbuttons
			  # p_x, p_y	mouse coordinates at event

	variable	w_data
	variable	c_state

	variable	::Config::conf_descr

	set x_dialog [ winfo toplevel $p_frame ]
	$x_dialog configure -cursor ""
	set x_widget [ winfo containing $p_x $p_y ]
	if { [ string first $p_frame $x_widget ] != 0 } return
	if { [ winfo class $x_widget] != "Checkbutton" } return
	PopupHelp $x_widget $x_dialog
    }



# Popup window vertical scroll command
# ------------------------------------

    proc PopupScroll { p_help p_offset p_size } {
			  # p_help	popup text widget
			  # p_offset	scroll offset (added by TK)
			  # p_size	(added by TK)

	regsub {\.[^\.]+$} $p_help "" x_outer
	if { ( $p_offset == 0 ) && ( ( $p_size == 0 ) || ( $p_size == 1 ) ) } \
									return

		# Need scrollbar, put it unless it exists already

	if { [ winfo exists $x_outer.scroll ] == 0 } {
	    set x_bg [ $x_outer cget -bg ]
	    scrollbar $x_outer.scroll -command "$p_help yview"
	    grid $x_outer.scroll -column 1 -row 0 -sticky ns
	    grid columnconfigure $x_outer 1 -weight 0
	    $x_outer.scroll set $p_offset $p_size
	    $p_help configure -yscrollcommand "$x_outer.scroll set"
  	}
    }



# Create the popup option help display
# ====================================

    proc PopupHelp { p_widget p_parent { p_label "" } } {
			  # p_widget	checkbutton widget that needs help
			  # p_parent	parent widget (calling panel)
			  # p_label	option label or ""

	global		global_vars

	variable	popup_params	;# parameters from resource data base

	variable	::HelpDialog::help_state
	variable	::HelpDialog::popup_table

	variable	::Config::conf_descr
	variable	::Config::conf_default
	variable	::Config::conf_params

	set	MODULE "popup"
	set	item_widgets(MODULE) $MODULE
	if { $p_parent == "." } {
	    set f_dialog .$MODULE
	} else {
	    set f_dialog $p_parent.$MODULE
	}

	set x_file [ file join $global_vars(HELP_DIR) "Popup.hlp" ]
	if { ( [ file isfile $x_file ] && [ file readable $x_file ] ) == 0 } {
	    Dialog::DialogMessage $p_widget Warning "" [ list \
			"Help file does not exist or is not a readable file" \
			"\\40$x_file" "\\8" "No help panel available !" ] "OK"
	    return
	}


	    # Define index table if it does not exist already

	if { [ array size popup_table ] == 0 } {
	    set x_fid [ open $x_file "r" ]
	    while { 1 } {
		set x_list [ HelpRead $x_fid ]
		set x_pos [ lindex $x_list 1 ]
		if { $x_pos < 0 } break
		set x_text [ lindex $x_list 0 ]
		if { [ regexp {^\[(\S\S+)\](\s+)(.+)$} $x_text "" \
					x_label x_sep x_text ] == 0 } continue
		set popup_table($x_label) [ expr $x_pos + 2 + \
			[ string length $x_label ] + [ string length $x_sep ] ]
	    }
	    close $x_fid
	}


	    # Create and position the popup window with text panel
	    # ----------------------------------------------------
	    #	Place top-left corner below and right of the calling checkbox

	if { [ Dialog::DialogCreate $f_dialog $p_widget \
		"Usync option checkbutton help" ] == 0 } { return 0 }


	    # Add a do-nothing intermediate frame to have a steady reference
	    #	for resource naming (there may be several top-level help widgets
	    #	each with another name)

	set	f_outer $f_dialog.helpPopup
	frame	$f_outer
	pack	$f_outer -side top -fill both -expand true

	if { [ array size help_params ] == 0 } {
	    AuxProcs::DefineParameters [ namespace current ]::popup_params \
						$f_outer { { popupSize} }
	}


		# Text panel with tags

	set	x_help $f_outer.help
	text	$x_help -borderwidth 2 -wrap word -relief flat -yscrollcommand \
			[ list [ namespace current ]::PopupScroll $x_help ]

	grid	$x_help -column 0 -row 0 -sticky news
	grid	columnconfigure $f_outer 0 -weight 1
	grid	rowconfigure $f_outer 0 -weight 1

	$x_help tag configure label_tag -font standard -spacing3 4
	$x_help tag configure rsync_tag -font standard -lmargin1 60 -spacing3 4
	$x_help tag configure text_tag -font normal -lmargin1 15 -lmargin2 15 \
								-spacing1 6
	    # Define text-zone tags

	foreach x_item { { b standard } { i marked } } {
	    $x_help tag configure tag_[ lindex $x_item 0 ] \
						-font [ lindex $x_item 1 ]
	}


	    # Post callback event when the popup goes away
	    # --------------------------------------------
	    #	Clear global_vars(POPUP) when the help text goes away

	bind	$f_dialog <Destroy> { set ::global_vars(POPUP) "" }
	wm	resizable $f_dialog 1 1
	set	global_vars(POPUP) $f_dialog


	    # Fill the help panel with info
	    # -----------------------------

		# 1st line = label

	if { $p_label == "" } {
	    set x_label [ $p_widget cget -variable ]
	    regexp {\((.*)\)} $x_label "" x_label
	} else {
	    set x_label $p_label
	}
	$x_help insert 1.0 "[ string toupper $conf_descr($x_label,TEXT) \
						0 0 ]   ($x_label)\n" label_tag

	set popup_table(-1) $x_label

		# 2nd line = default state

	if { $conf_default($x_label,VALUE) } { set x_state "selected" } else \
						{ set x_state "not selected" }
	$x_help insert insert "default state:  $x_state\n" rsync_tag

		# 3rd line = rsync variable text (if it exists

	set x_line 3
	if [ info exists conf_descr($x_label,ARG) ] {

	   eval set x_text $conf_descr($x_label,ARG)
	   $x_help insert insert "rsync option:  - $x_text\n" rsync_tag
	   incr x_line
	}

		# 4th line and on = information text

	set x_fid [ open $x_file "r" ]
	seek $x_fid $popup_table($x_label) start

	set x_lead "If checked: "
	while { 1 } {
	    set x_list [ HelpRead $x_fid ]
	    if { [ lindex $x_list 1 ] < 0 } break
	    if [ regexp {^\[\S\S+\]}  [ lindex $x_list 0 ] ] break

	    set x_text "$x_lead[ lindex $x_list 0 ]"
	    set x_lead ""
	    set zone_list [ HelpZones x_text "ib" ]

	    $x_help insert insert "$x_text\n" text_tag
	    foreach x_zone $zone_list {
		set x_begin [ lindex $x_zone 0 ]
		set x_end   [ lindex $x_zone 1 ]
		set x_type  [ lindex $x_zone 2 ]
		set x_tag tag_$x_type
		$x_help tag add $x_tag $x_line.$x_begin $x_line.$x_end
	    }
	    incr x_line
	}
	close $x_fid
	$x_help configure -state disabled \
				-width [ lindex $popup_params(popupSize) 0 ] \
				-height [ lindex $popup_params(popupSize) 1 ]
	$x_help see 1.0
    }
}
