package provide HelpDialog



# Help paragraph tags
# ===================
#
#   Title paragraphs and 0-level text tags (2-character code)
#	Automatically defined from paraT<xx> variables of option data base
#
#	T0 ...	header level 0
#	T1 ...	header level 1
#	T2 ...	header level 2 (with reference in toc)
#	T3 ...	header level 2 (no reference in toc)
#
#	TX (or no code) ... 0-level text paragraph
#
#	Bn ...	bullet item
#	Cn ...	enumeration item
#	In ...	indent item (normal - all lines with same indent)
#
#	J1 ...	indent item level 1 (for file names - large indent for 2nd
#			and later lines)
#	K3 ...	intent item level 3 with tight spacing



namespace eval HelpDialog {


# Define indentation, bullet etc. paragraphs
# ------------------------------------------
#	(note: \u2022 is the code for a bold bullet)

    variable	para_preset {
			{ B1	1	" \u2022 "	}
			{ B2	2	" -  "		}
			{ B3	3	" -  "	}
			{ C1	1	" "		}
			{ C2	2	" "		}
			{ C3	3	" "		}
			{ I1	1			}
			{ I2	2			}
			{ K3	3	""    "0"	}
			{ J1	1	""    ""    45	} }


# Tables that are common for all help windows (initialised only once)
# ===================================================================


# Paragraph descriptor table
# --------------------------

    variable	para_table	    ;# help paragraph type table (<type = [xx] )
	#	    (<type>,TAG)	text panel tag for text lines
	#	    (<type>,COLOUR)	text panel foreground colour
	#	    (<type>,FONT)	text panel font
	#	    (<type>,SPACING)	spacing on top of paragraph
	#	    (<type>,MARGIN)	with n-flag: left paragraph margin
	#				else: extra margin for lines 2 and on
	#	    (<type>,LEAD)	paragraph lead character or ""
	#						" " ... enumeration
	#	    (<type>,FLAGS)	type flag characters
	#				  l ... title item
        #				  c ... put link into TOC
	#				  n ... an indented paragraph
	#				always gets leading digit = indentation
	#				  with n-flag: 0
	#				  else: > 0


# Tag table with text tags
# ------------------------

    variable	tag_table
        set	    tag_table(i)    { normal	italic	italic_tag }
	set	    tag_table(b)    { bold	roman	bold_tag }
	set	    tag_table(l)    { normal	roman	link_tag }
#		    tag_table(STRING) ... list of tag codes


# Popup index table
# -----------------

    variable	popup_table
	#	    (-1)		option id of current popup window
	#	    (<label>)		offset into popup text file for option
	#				with identifier <label>


# Static parameters from resource data base (max_level is computed)
# -----------------------------------------

    variable	help_params
	#	    (maxLevel)		highest indentation level
	#	    (gobackStack)	max number of goback entries kept
	#	    (urlColour)		background colour of hypertext links
	#	    (minSize)		min window width and height
	#	    (textIndents)	text indentation levels
	#	    (titleIndents)	title indentation step size
	#	    (tocIndents:	TOC indentation step size



# Tables that are specific for each help window
# =============================================
#	These tables are indexed with the name of the help target


# Hypertext link table
# --------------------

    variable	help_links
	#	    (<target>,<tag>)	text index of target text for <tag>
	#	    (<target>,<index>)	tag called from zone at index


# Help state table
# ----------------

    variable	help_state	    ;# help information table
	#	    (<target>,<n>,ENUM) enumeration counter for level <n>
	#	    (<target>,<toc>)	text-panel index of for toc item <toc>
	#	    (<target>,BACK_BUTTON)
	#	    (<target>,BACK_LIST)


# Help window handler
# ===================

    proc HelpPanel { p_widget p_target } {
			  # p_widget	help-button widget
			  # p_target	help target

	global		global_vars

	variable	help_state
	variable	help_params
	variable	para_table
	variable	tag_table
	variable	help_links
	variable	para_preset

	variable	::Config::conf_params
	variable	::Config::conf_descr

	if { $p_target == "" } { 
	    set x_target "MainPanel"
	} else {
	    set x_target $p_target
	}

	set	MODULE "help"
	set	item_widgets(MODULE) $MODULE
	set	f_dialog ".${MODULE}_$x_target"
	set	help_state(DIALOG) $f_dialog
	set	help_state($x_target,BACK_LIST) ""
	set	BUTTON 2

	set x_file [ file join $global_vars(HELP_DIR) "$x_target.hlp" ]
	if { ( [ file isfile $x_file ] && [ file readable $x_file ] ) == 0 } {
	    Dialog::DialogMessage $p_widget Warning "" [ list \
			"Help file does not exist or is not a readable file" \
			"\\40$x_file" "\\8" "No help panel available !" ] "OK"
	    return
	}

	if { [ Dialog::DialogCreate $f_dialog $p_widget "" ] == 0 } { return 0 }

	    # Add a do-nothing intermediate frame to have a steady reference
	    #	for resource naming (there may be several top-level help widgets
	    #	each with another name)

	set	f_outer $f_dialog.helpPanel
	frame	$f_outer
	pack	$f_outer -side top -fill both -expand true

	if { [ array size help_params ] == 0 } {
	    AuxProcs::DefineParameters [ namespace current ]::help_params \
								$f_outer  {  \
			{ gobackStack	} { minSize	} { urlColour	} \
			{ textIndents	} { titleIndent	} { tocIndent	} }
	}


	    # Put everything in the gridded top-level frame
	    # ----------------------------------------------
	    #   		    row 0			   row 1
	    #   column 0 ...   toc list box			button frame
	    #   column 1 ...   vert. scrollbar for toc		quit button
	    #   column 2 ...		separator bar for width-drag
	    #   column 3 ...		scrolled toc text panel
	    #   column 4 ...		vert. scrollbar for text

	grid	columnconfigure $f_outer 0 -weight 0
	grid	columnconfigure $f_outer 1 -weight 0
	grid	columnconfigure $f_outer 2 -weight 0
	grid	columnconfigure $f_outer 3 -weight 1
	grid	columnconfigure $f_outer 4 -weight 0

	grid	rowconfigure $f_outer 0 -weight 1
	grid	rowconfigure $f_outer 1 -weight 0
	grid	rowconfigure $f_outer 2 -weight 0


	    # Create listbox for table of contents
	    # ------------------------------------

		# Listbox

	listbox	$f_outer.contents -font normal -borderwidth 2 -relief flat \
		-yscrollcommand [ list HelpAux::HelpTOCscroll \
							$f_outer.contents ] 

	grid	$f_outer.contents -column 0 -row 0 -sticky news
	bind	$f_outer.contents <ButtonRelease-1> [ list \
			HelpAux::HelpButtonup $f_outer.contents $x_target ]

	frame	$f_outer.listbar -height 2 -borderwidth 2 -relief groove
	grid	$f_outer.listbar -column 0 -columnspan 2 -row 1 -sticky ew


	    # Vertical separater bar
	    # ----------------------

	frame	$f_outer.sep -cursor cross -relief raised -borderwidth 2
	grid	$f_outer.sep -column 2 -row 0 -rowspan 3 -sticky news

	label	$f_outer.sep.mark -bitmap @[ file join $global_vars(LIB_DIR) \
				diamond.xbm ] -borderwidth 0
	pack	$f_outer.sep.mark -side left -anchor center

	bind	$f_outer.sep <B1-Motion> [ list \
					HelpAux::DragBorder $f_outer.sep ]
	bind	$f_outer.sep.mark <B1-Motion> [ list \
					HelpAux::DragBorder $f_outer.sep ]


	    # Create scrolled help text widget with scrollbar
	    # -----------------------------------------------
	set	f_right $f_outer.right
	set	x_pattern $f_right.text

		# Help text panel

	set	x_panel $f_outer.text
	text	$x_panel -wrap word -padx 4 -relief flat \
				-yscrollcommand "$f_outer.textscroll set"
	grid	$x_panel -column 3 -row 0 -sticky news -rowspan 3

	scrollbar $f_outer.textscroll -command "$x_panel yview"
	grid	$f_outer.textscroll -column 4 -row 0 -sticky ns -rowspan 3


		# Window menu

	set	x_menu $x_panel.menu
	menu	$x_menu -tearoff false
	$x_menu add command -label {Show / hide "Find"} -command \
			[ list HelpAux::DoFind $x_pattern $x_panel ]

	if { $p_target != "" } { $x_menu add command -label "Main-window help" \
		-command [ list [ namespace current ]::HelpPanel . "" ] }
	$x_menu add separator
	$x_menu	add command -label "Done" -command [ list destroy $f_dialog ]
	bind	$x_panel <ButtonPress-3> [ list tk_popup $x_menu %X %Y ]


	    # Create left bottom control buttons
	    # ----------------------------------

	set	f_left $f_outer.left
	frame	$f_left -borderwidth 3
	grid	$f_left -column 0 -columnspan 2 -row 2 -sticky ew


		# Go-back button

	button	$f_left.back -text "Go back" -state disabled -pady $BUTTON \
			-command [ list HelpAux::HelpBack $f_outer.text ]
	pack	$f_left.back -side left
	set	help_state($x_target,BACK_BUTTON) $f_left.back


	    # Create right bottom control buttons and find text panel
	    # -------------------------------------------------------

	frame	$f_right


		# Find button

	button	$f_right.find -text "Find" -pady $BUTTON -command [ list \
				HelpAux::DoFind $x_pattern $x_panel first ]
	pack	$f_right.find -side left


		# Pattern text

	text	$x_pattern -width 0 -height 1 -bg white
	pack	$x_pattern -side left -fill x -padx 4 -expand true
	bind	$f_dialog <Enter> [ list focus $x_pattern ]
	bind	$x_pattern <Return> [ list HelpAux::DoFind $x_pattern \
								$x_panel next ]
	bind	$x_pattern <Tab> { break }


		# Prec button

	button  $f_right.prec -text "Prec" -pady $BUTTON -command [ list \
				HelpAux::DoFind $x_pattern  $x_panel prec ]
	pack	$f_right.prec -side right -padx 4


		# Next button

	button  $f_right.next -text "Next" -pady 1 -borderwidth 1 \
		-command [ list HelpAux::DoFind $x_pattern $x_panel first ] \
								-default active
	pack	$f_right.next -side right


	    # Define paragraph table
	    # ----------------------

	if { [ info exists help_params(maxLevel) ] == 0 } {

	    foreach x_tag [ array names tag_table ] { append \
						tag_table(STRING) $x_tag }

		# Preset data for indentation, bullet etc. paragraphs

	    foreach x_para $para_preset {
		set x_tag [ lindex $x_para 0 ]
		set para_table($x_tag,FLAGS)	[ lindex $x_para 1 ]
		set para_table($x_tag,LEAD)	[ lindex $x_para 2 ]
		set para_table($x_tag,SPACING)	[ lindex $x_para 3 ]
		set para_table($x_tag,MARGIN)	[ lindex $x_para 4 ]
		if { $para_table($x_tag,MARGIN) == "" } \
					{ set para_table($x_tag,MARGIN)	0 }
	    }
	    unset para_preset

		# Setup the required fonts

	    foreach x_item [ font names ] { 
		set x_list [ font configure $x_item]
		set x_temp [ AuxProcs::ListArgument $x_list -family ]
		if { $x_temp != [ lindex $conf_params(fontFamilies) 0 ] } \
									continue
		set x_size   [ AuxProcs::ListArgument $x_list -size ]
		set x_weight [ AuxProcs::ListArgument $x_list -weight ]
		set x_slant  [ AuxProcs::ListArgument $x_list -slant ]
		set font_table($x_size,$x_weight,$x_slant) $x_item
		
	    }

	    foreach x_item [ array names font_table *,WEIGHT ] {
		regsub {,WEIGHT} $x_item "" x_size 
	    }

		# Titles and plain text paragraphs

	    for { set x_x 0 } { 1 } { incr x_x } {
		set x_label T$x_x
		set x_list [ option get $f_outer para$x_label "" ]
		if { $x_list == "" } {
		    set x_label TX
		    set x_list [ option get $f_outer para$x_label "" ]
		}
		set x_flags "0n"
		set x_slant  "roman"
		set x_weight "normal"
		foreach x_item $x_list {
		    set x_lead [ string range $x_item 0 0 ]
		    switch -regexp $x_item {
			{^toc$}  {
			    append x_flags "c"
			}
			{^normal$} -
			{^bold$} {
			    set x_weight $x_item
			}
			{^regular$} -
			{^italic$} {
			    set x_slant $x_item
			}
			{^#\d$} {
			    set x_temp [ string range $x_item 1 end ]
			    set x_size [ lindex $conf_params(fontSizes) \
								$x_temp ]
			} 
			{^\+(\d$)} {
			    set x_temp [ expr [ string range $x_item 1 end ] \
						* $help_params(titleIndent) ]
			    set para_table($x_label,MARGIN) $x_temp
			} 
			{^[\d]+$} {
			    set para_table($x_label,SPACING) $x_item
			} 
			{^\#[:xdigit:][:xdigit:]+$} -
			{^\w+$} {
			    set para_table($x_label,COLOUR) $x_item
			}
			default {
			    puts stderr "bad para code"
			}
		    }
		}
		set para_table($x_label,LEAD)	""
		set para_table($x_label,FLAGS)	$x_flags
		set para_table($x_label,TAG)	tag_$x_label
		if { [ info exists font_table($x_size,$x_weight,$x_slant) ] \
									== 0 } {
		    set x_name font_[ array size font_table ]
		    font create $x_name -family \
			[ lindex $conf_params(fontFamilies) 0 ] \
			-size $x_size -weight $x_weight -slant $x_slant
		    set font_table($x_size,$x_weight,$x_slant) $x_name
		}
		set para_table($x_label,FONT) \
					$font_table($x_size,$x_weight,$x_slant)
		if { $x_label == "TX" } break
	    }

		# Indent, enum, ...  paragraphs

	    set max_level 0
	    foreach x_item [ array names para_table *,FLAGS ] {
		if { [ string first "n" $para_table($x_item) ] >= 0 } continue
		regexp {(.*),FLAGS} $x_item "" x_label
		set x_level [ string range $para_table($x_label,FLAGS) 0 0 ]
		set para_table($x_label,TAG)	tag_$x_label
		set para_table($x_label,COLOUR)	$para_table(TX,COLOUR)
		set para_table($x_label,FONT)	$para_table(TX,FONT)
		if { $para_table($x_label,SPACING) == "" } { set \
					para_table($x_label,SPACING) \
					[ expr $para_table(TX,SPACING) - 4 ] }
		if { $x_level > $max_level } { set max_level $x_level }
	    }
	    set help_params(maxLevel) $max_level
	}


	    # Define tags for paragraphs attached to text panel
	    # -------------------------------------------------

	foreach x_item [ array names para_table *,TAG ] {
	    regexp {(.*),TAG} $x_item "" x_label
	    set x_flags $para_table($x_label,FLAGS)
	    set x_level [ string range $x_flags 0 0 ]
	    set x_spacing $para_table($x_label,SPACING)
	    if { [ string first "n" $x_flags ] < 0 } {
		set x_margin1 [ expr $para_table(TX,MARGIN) + \
				[ lindex $help_params(textIndents) $x_level ] ]
		set x_margin2  [ expr $x_margin1 + \
						$para_table($x_label,MARGIN) ]
		if { $para_table($x_label,LEAD) != "" } { incr x_margin1 -15 }
	    } else {
		set x_margin1 $para_table($x_label,MARGIN)
	        set x_margin2 $para_table($x_label,MARGIN)
	    }
	    $x_panel tag configure $para_table($x_label,TAG) \
			-foreground $para_table($x_label,COLOUR) \
			-font $para_table($x_label,FONT) -spacing1 $x_spacing \
			-lmargin1 $x_margin1 -lmargin2 $x_margin2
	}


	    # Define tags for text zones
	    # --------------------------

	set x_list [ font configure $para_table(TX,FONT) ]
	foreach x_item { family size slant weight } {
	    set x_font($x_item) [ AuxProcs::ListArgument $x_list "-$x_item" ]
	}

	foreach	x_item [ array names tag_table ] {
	    if { [ string length $x_item ] != 1 } continue
	    set x_list $tag_table($x_item)
	    set x_tag [ lindex $x_list 2 ]
	    set x_list [ lreplace $x_list 2 2 ] 
	    set x_list [ linsert $x_list 0 $x_font(family) $x_font(size) ]
	    if { $x_item == "l" } {
		$x_panel tag configure $x_tag -font $x_list \
			-underline true -foreground $help_params(urlColour)
		$x_panel tag bind $x_tag <ButtonRelease-1> [ list \
						HelpAux::HelpGo $x_panel ]
	    } else {
		$x_panel tag configure $x_tag -font $x_list
	    }
	}

		# Define aspect of selection tag

	$x_panel tag configure sel -foreground yellow -background grey60
	$x_panel tag raise sel

		# Introduce "missing-link tag"

	$x_panel tag configure tag_miss -foreground red -background yellow
	$x_panel tag raise tag_miss

	for { set x_x 0 } { $x_x <= $help_params(maxLevel) } { incr x_x } \
				{ set help_state($x_target,$x_x,ENUM) 0 }


	    # Read lines of help file and fill help panel
	    # -------------------------------------------

	set x_fid [ open $x_file "r" ]
	if [ info exists help_links ] {
	    foreach x_item [ array names help_links $x_target,* ] \
						{ unset help_links($x_item) }
	}
	while { 1 } {
	    set x_list [ HelpAux::HelpRead $x_fid ]
	    if { [ lindex $x_list 1 ] < 0 } break
	    set x_text [ lindex $x_list 0 ]

	    if { [ regexp {^\[([^\[\]][^\[\]]+)\](.*)$} $x_text "" \
				x_label x_text ] == 0 } { set x_label "TX" }
	    if { $x_label == "WINDOW" } {
		wm title $f_dialog $x_text 
		continue
	    }


		# A title or text paragraph
		# -------------------------

	    set x_link [ $x_panel index insert ]
	    regexp {(\d+)\.} $x_link "" x_line
	    set x_flags $para_table($x_label,FLAGS)
	    set x_tag $para_table($x_label,TAG)
	    set x_level [ string range $x_flags 0 0 ]

		    # Reset enum counter for all higher levels

	    for { set x_x [ expr $x_level + 1 ] } { $x_x <= \
				$help_params(maxLevel) } { incr x_x } { 
		set help_state($x_target,$x_x,ENUM) 0
	    }

		    # Prepend lead character if one is defined

	    set x_lead $para_table($x_label,LEAD)
	    if { $x_lead == " " } {
		incr help_state($x_target,$x_level,ENUM)
		set x_lead "$help_state($x_target,$x_level,ENUM). "
	    }
	    set x_text $x_lead$x_text

		    # Handle internal tags, establish the zone list

	    set zone_list [ HelpAux::HelpZones x_text $tag_table(STRING) ]
	    foreach x_zone $zone_list {
		if { [ lindex $x_zone 2 ] != "l" } continue

		    # Treat hypertext link
		    #	Add item to help_links
		    #	    definition: help_links(<target><tag>) = text index
		    #	    reference:  help_links(<target><index> = tag

		set x_point [ lindex $x_zone 3 ]
		set x_pos [ lindex $x_zone 4 ]
		if [ lindex $x_zone 5 ] {
		    set help_links($x_target,$x_point) $x_line.$x_pos
		} else {
		    set help_links($x_target,$x_line.$x_pos) $x_point
		}
	    }

		    # Force spacing to 0 if this is the first line

	    if { $x_link == "1.0" } {
		set x_command "$x_panel tag configure first_tag"
		foreach x_item { -font -foreground -lmargin1 -lmargin2 } {
		    set x_value  [ $x_panel tag configure $x_tag $x_item ]
		    set x_temp [ expr [ llength $x_value ] - 1 ]
		    set x_value [ lindex $x_value $x_temp ]
		    append x_command " $x_item $x_value"
		}
		append x_command" -spacing1 0"
		eval $x_command
		set x_tag first_tag 
	    }

		    # Store text

	    $x_panel insert end "$x_text\n" $x_tag

	    foreach x_zone $zone_list {
		set x_begin [ lindex $x_zone 0 ]
		set x_end   [ lindex $x_zone 1 ]
		set x_type  [ lindex $x_zone 2 ]
		set x_tag   [ lindex $tag_table($x_type) 2 ]
		$x_panel tag add $x_tag $x_line.$x_begin $x_line.$x_end
	    }

	    if { [ string first "c" $x_flags ] >= 0 } {


		# toc- flag is set, add item to the toc listbox
		# ---------------------------------------------

		regexp {^.(\d)} $x_label "" x_level
		set x_index [ $f_outer.contents index end ]
		set x_lead [ string repeat " " [ expr $x_level * \
						$help_params(tocIndent) ] ]
		$f_outer.contents insert $x_index $x_lead$x_text
		$f_outer.contents itemconfigure $x_index \
						-fg $para_table($x_label,COLOUR)
		set help_state($x_target,$x_index) $x_link
	    }
	}
	close $x_fid

		# Check whether all hypertext targets are defined
		# -----------------------------------------------

	foreach x_item [ array names help_links $x_target,* ] {
	    regsub "$x_target," $x_item "" x_temp
	    if [ regexp {\d+\.\d+} $x_temp ] {
		set x_point $help_links($x_item)
		if [ info exists help_links($x_target,$x_point) ] continue

		    # Found a reference to an undefined label

		regexp {(\d+)\.(\d+)} $x_temp "" x_line x_char
		set x_range [ $x_panel tag prevrange link_tag \
						$x_line.[ expr $x_char + 1 ] ]
		set x_0 [ lindex $x_range 0 ]
		set x_1 [ lindex $x_range 1 ]
		$x_panel tag add tag_miss $x_0 $x_1
		puts stderr "****"
		puts stderr "**** Hypertext tag \"$x_point\" is not defined in\
						help window \"$x_target\""
		puts stderr "****     Referenced in paragraph number $x_line\
						of the help file $x_target.hlp"
		puts stderr "****"
	    }
	}

	$x_panel configure -state disabled
	update idletasks
	wm minsize $f_dialog [ lindex $help_params(minSize) 0 ] \
					[ lindex $help_params(minSize) 1 ]
    }
}
