package provide MainAux



# Callback handlers for main panel
# ================================



namespace eval MainAux {



# Callback handlers
# =================


# Main-panel destroy callback
# ---------------------------
#	Write configuration file. In order to avoid losing it in case of an
#	abandon (.e.g. SIGKILL), write and rename a temporary file

    proc MainKill { p_dialog { p_abort "" } } {
			  # p_dialog	widget of main-panel (i.e. "." )
			  # p_abort	if non-"" ... exit without re-generating
			  #		the configuration file

	variable	::Config::conf_params

# !!! if this module has compilation error, no message is given,
# !!! the exit hook just is skipped without further notice

# If that happens: to debug try calling SaveConfig early in proc InitPanel

	bind . <Destroy> {} 
	LogItem "" close 
	if { $p_abort == "" } ConfigFile::SaveConfig
	Dialog::DialogKill $p_dialog
    }



# Main-panel configure event callback (includes resize events)
# ------------------------------------------------------------

    proc ResizePanel { p_height p_widget } {
			  # p_height	new height of f_data frame
			  # p_widget	widget with resize event

	variable	::MainPanel::main_widgets
	variable	::MainPanel::main_params


	    # Initialise variables
	    # --------------------
	    #	x_canvas	... canvas widget
	    #   f_data		... outer data-field panel (sunken frame)

	set f_data $main_widgets(DATA)
	if { $p_widget != $f_data } return
	set x_canvas [ winfo parent $main_widgets(CANVAS) ]
	if { [ info exists main_widgets(STEP) ] == 0 } return
	set x_step $main_widgets(STEP)


	    # Determine new number of view rows
	    # ---------------------------------

	set x_temp [ expr ( [ $f_data cget -borderwidth ] + \
				[ $f_data cget -highlightthickness ] ) * 2 ]
	if [ winfo exists $f_data.xscroll ] \
				{ incr x_temp $main_widgets(SCROLL_WIDE) }
	set v_rows [ expr ( $p_height - $x_temp ) / $x_step ]
		if { $v_rows < $main_params(resultRows) } \
					{ set v_rows $main_params(resultRows) }
	set main_widgets(ROW_NUMB) $v_rows


	    # If no vertical scrollbar needed any more, destroy it
	    # ----------------------------------------------------

	if { ( $v_rows >= $main_widgets(ITEM_ROW) ) && \
					[ winfo exists $f_data.yscroll ] } {
	    $f_data.canvas yview moveto 0.0
	    set x_save [ bind . <Destroy> ]
	    bind . <Destroy> {}
	    destroy $f_data.yscroll
	    eval "bind . <Destroy> [ list $x_save ]"
	    $x_canvas configure -yscrollcommand "" -yscrollincrement $x_step
	}


	    # Introduce new settings into canvas and scrollbar
	    # ------------------------------------------------

	if [ winfo exists $f_data.yscroll ] {
	    set t_height [ expr $main_widgets(ITEM_ROW) * $x_step ]
	    set t_width [ winfo width $main_widgets(CANVAS) ]
	    if { $t_width <= $main_params(resultMax) } {
		set v_width $t_width
	    } else {
		set v_width $main_params(resultMax)
	    }
	    $x_canvas configure -width $v_width -height [ expr $v_rows * \
				$x_step ] -scrollregion "0 0 $t_width $t_height"
	} 
	MainProcs::ShowAdapt
    }



# Post main-panel Config-menu callback
# ------------------------------------
#	Control state of profile editor items

    proc PostConfig { p_menu } {

	variable	::MainPanel::main_widgets

	    # Set x_state to normal if a profile exists

	set x_menu $main_widgets(PRESET_MENU)
	set x_last [ $x_menu index end ]
	set x_state disabled
	for { set x_x 0 } { $x_x <= $x_last } { incr x_x } {
	    if { [ $x_menu type $x_x ] != "command" } continue
	    set x_command [ $x_menu entrycget $x_x -command ]
	    set x_arg [ lindex $x_command 1 ]
	    if { [ llength $x_arg ] > 1  } { set x_state normal }
	}

	    # Set update and delete menu items to normal / disabled if
		# at least one profile exists / does not exist

	for { set x_x 0 } { $x_x <= $x_last } { incr x_x } {
	    if { [ $p_menu type $x_x ] != "command" } continue
	    set x_command [ $p_menu entrycget $x_x -command ]
	    if { [ regexp {EditProfile\s+\S+\s+(\S+)} $x_command "" x_mode ]
								== 0 } continue
	    if { $x_mode == "create" } continue
	    $p_menu entryconfigure $x_x -state $x_state
	}
    }



# Preset profile menu-button callback: preset from profile
# --------------------------------------------------------

    proc PresetProfile { p_arg { p_frame "" } } {
			  # p_arg	"0"  ... clear checkbuttons in frame
			  #		"1"  ... mark checkbuttons in frame
			  #		"2"  ... set checkbuttons in frame
			  #					to default
			  #		"3"  ... set checkbuttons in frame
			  #					to initial value
			  #		"+"  ... set all checkbuttons and 
			  #			    config variables to default
			  #		"*"  ... set all checkbuttons to
			  #					initial value
			  #		else ... preset all according to profile
			  # p_frame	frame that contains the checkbuttons
			  #		    (only used for 0, 1, 2, 3)

	global		button_value
	global		initial_value
	global		global_vars

	variable	::Config::conf_descr
	variable	::Config::conf_default
	variable	::MainPanel::main_widgets

	switch -- $p_arg {
	    "0" -
	    "1" -
	    "2" -
	    "3" {

		# Set options of frame to default (-) or 1 or 0

		foreach x_widget [ grid slaves $p_frame ] {
		    if { [ winfo class $x_widget ] == "Frame" } {
			PresetProfile $p_arg $x_widget
		    } elseif { [ winfo class $x_widget ] == "Checkbutton" } {
			set x_variable [ $x_widget cget -variable ]
			regexp {\((.*)\)} $x_variable "" x_tag

			if { $p_arg == "2" } {
			    set x_value $conf_default($x_tag,VALUE)
			} elseif { $p_arg == "3" } {
			    set x_value $initial_value($x_tag)
			} else {
			    set x_value $p_arg
			}
			upvar #0 $x_variable x_local
			set x_local $x_value
		    } else {
			continue
		    }
		}
	    }

	    "*" -
	    "+" {

		# Set all parameters to default / to initial value

		foreach x_tag [ array names conf_descr *,FLAGS ] {
		    regsub {,FLAGS} $x_tag "" x_tag
		    if { $p_arg == "+" } {
			if { [ info exists conf_default($x_tag,VALUE) ] == 0 } \
									continue
			set x_value $conf_default($x_tag,VALUE)
		    } else {
			if { [ info exists initial_value($x_tag) ] == 0 } \
									continue
			set x_value $initial_value($x_tag)
		    }
		    set x_flags $conf_descr($x_tag,FLAGS)
		    if { [ string first "c" $x_flags ] >= 0 } {
			
			set button_value($x_tag) $x_value
		    } elseif { ( $p_arg == "*" ) && \
				( [ string first "v" $x_flags ] >= 0 ) } {
			set global_vars($x_tag) $x_value
		    } elseif { [ string first "b" $x_flags ] >= 0 } {
			if { [ string first "r" $x_flags ] >= 0 } {
			    set button_value($x_tag) $x_value
			}
		    }
		}
		MainProcs::SetDirection
	    }

	    default {

		# Set to values according to a profile (also direction button)

		set x_zone $global_vars(SYNC_ZONE)
		set x_path $global_vars(REL_PATH)
		foreach x_item $p_arg {
		    if [ regexp {^\"([^=]+)=(.*)\"$} $x_item "" \
							x_arg x_value ] {
			if { [ string first "c" \
					$conf_descr($x_arg,FLAGS) ] >= 0 } {
			    set button_value($x_arg) $x_value
			} elseif { [ string first "b" \
					$conf_descr($x_arg,FLAGS) ] >= 0 } {
			    set button_value($x_arg) $x_value
			} elseif { [ string first "v" \
					$conf_descr($x_arg,FLAGS) ] >= 0 } {
			    set global_vars($x_arg) $x_value
			}
		    }
		}
		if { ( $x_zone != $global_vars(SYNC_ZONE) ) ||
				( $x_path != $global_vars(REL_PATH) ) } {
		    regsub {^/} $global_vars(REL_PATH) "" x_temp
		    SelectAux::NewTop $global_vars(SYNC_ZONE) "" $x_temp
		    MainProcs::PathLabels
		}
		MainProcs::SetDirection
	    }
	}
    }



# Service button callback
# -----------------------
#	Launch synchronisation action

    proc DoAction { p_execute { p_check "" } } {
			  # p_execute	dry ... only probe and list
			  #		real ... really synchronise
			  # p_check	not "" ... ask for confirmation, value=
			  #				widget 

	global		global_vars
	global		button_value

	variable	::MainPanel::main_widgets
	variable	::MainPanel::main_params

	if { $p_check != "" } {
	    set x_temp $global_vars(REL_PATH)
	    set x_text "$global_vars(CLIENT_BASE)$x_temp"
	    if { $button_value(XFER_DIRECTION) } {
		append x_text "  ->  "
	    } else {
		append x_text "  <-  "
	    }
	    append x_text "$global_vars(SERVER_BASE)$x_temp"
	    if { $global_vars(SERVER_HOST) != "" } { append x_text \
						"@$global_vars(SERVER_HOST)" }

	    set x_reply [ Dialog::DialogMessage $p_check Confirm "" [ list \
			{Going to synchronise} \
			"\\40$x_text"  \
			{\8} {OK to go ahead ?} ] \
			{ "OK" "Skip" } ]
	    if { $x_reply != 0 } return
	}
	update idletasks

	wm maxsize . 0 0
	wm resizable . 0 0

	foreach x_frame [ winfo children .outer ] \
				{ MainProcs::SetState $x_frame disabled }

	set f_data $main_widgets(DATA)
	set x_save [ bind $f_data <Configure> ]
	bind $f_data <Configure> {}
	set x_list [ Actions::Synchronise $p_execute ]
	if { $p_execute != "dry" } { LogItem "" flush }
	set x_code [ lindex $x_list  0]
	foreach x_item [ lindex $x_list 1 ] \
					{ MainProcs::AddResult $x_item $x_code }
	set x_list [ MainProcs::ShowAdapt ]

	bind $f_data <Configure> $x_save


	foreach x_frame [ winfo children .outer ] \
					{ MainProcs::SetState $x_frame normal }

	if [ winfo exists $main_widgets(DATA).yscroll ] {
	    set x_temp [ expr $main_widgets(ROW_BASE) + \
						$main_widgets(ITEM_ROW) ]
	    set y_temp [ expr $main_widgets(ROW_BASE) + \
						$main_params(resultRows) ]
	    if { [ winfo exists $main_widgets(DATA).xscroll ] == 0  } {
		incr x_temp -1
		incr y_temp -1
	    }
	    wm maxsize . 0 $x_temp
	    wm minsize . 0 $y_temp
	    wm resizable . 0 1
	} else {
	    wm resizable . 0 0
	}
    }



# Log file actions
# ----------------
#   a. write log items to the log file
#   b. do service operations (open, close flush)

    proc LogItem { p_file p_arg { p_stamp "" } } {
			  # p_file	if not "":
			  #		    write rsync log, p_file = file name
			  #		if "" 
			  #		    p_arg = command: 
			  #			- file ops: open, close, flush
			  #			- log ops: session, base
			  # p_arg	if p_file != ""
			  #		    action description for log record
			  #		if p_file == ""
			  #		   open  ... open the log file 
			  #		   close ... close the log file 
			  #		   flush ... flush the log file 
			  #		   session ... put session start log
			  #		   base    ... put base change log
			  # p_stamp	time-stamp for log record (not used
			  #			if a service action)

	global		button_value
	global		global_vars

	variable	x_fid
	variable	x_format "%Y/%m/%d %H:%M:%S"

	variable	::Config::conf_params

	if { $p_file != "" } {
	    if [ info exists x_fid ] { puts $x_fid "$p_stamp $p_arg $p_file" }
	    return
	}

	switch $p_arg {
	    "open" {
		if { $conf_params(logFile) == "" } return
		if { $button_value(WRITE_LOG) == 0 } return
		if [ info exists x_fid ] return
		if [ catch { open $conf_params(logFile) "a" } x_fid ] return
		set x_size [ file size $conf_params(logFile) ]
		if { $x_size > $conf_params(logWarning) } {
		    Dialog::DialogMessage . Warning "" [ list \
			"The log file \"$conf_params(logFile)\" has a size of" \
			"\\40 $x_size bytes" \
			"A new log file should be started soon !" ] "OK"
		}
	    }
	    "flush" {
		if { [ info exists x_fid ] == 0 } return
		catch { flush $x_fid }
	    }
	    "close" {
		if { [ info exists x_fid ] == 0 } return
		catch { close $x_fid }
		unset x_fid
	    }
	    "base" -
	    "session" {
		if { [ info exists x_fid ] == 0 } return
		set x_clock [ clock format [ clock seconds ] -format $x_format ]

		    # If "session" ... put session-start log-item

		if { $p_arg == "session" } {
		    regsub {\s+.*$} [ exec grep " / " /etc/fstab ] "" x_temp
		    puts $x_fid "$x_clock Session start $x_temp"
		}


		    # Put synchronisation-zone log-item

		if { $global_vars(SYNC_ZONE) != "" } {
		    if { $global_vars(SERVER_HOST) == "" } {
			set x_server "$global_vars(SERVER_BASE)/"
			append x_server $global_vars(REL_PATH)
		    } else {
			set x_server "$global_vars(SERVER_BASE)/"
			append x_server "$global_vars(REL_PATH)@"
			append x_server $global_vars(SERVER_HOST)
		    }
		    set x_client "$global_vars(CLIENT_BASE)/"
		    append x_client $global_vars(REL_PATH)
		    if { $button_value(XFER_DIRECTION) } {
			set x_dir "->"
		    } else {
			set x_dir "<-"
		    }
		    puts $x_fid "$x_clock $x_client $x_dir $x_server"
		    }
		catch { flush $x_fid }
	    }
	}
    }
}
