package provide MainPanel



# Main panel creation and initilisation
# =====================================



namespace eval MainPanel {


    variable	main_widgets	    ;# widget data
		    # (TOP)		... top frame with buttons
		    # (DATA)		... frame with result panel
		    # (PATH)		... path label frame
		    # (CANVAS)		... gridded frame with result items
		    # (ITEM_COL)	... result panel column for next item
		    # (ITEM_ROW)	... result panel row for next item
		    # (RESULT_ROW)	... data panel row for result panel
		    # (ACTION)		... title label widget: current action
		    # (PRESET_MENU)	... preset option profile menu
		    # (PROGRESS)	... progress information frame
		    # (BOXES)		... frame with option checkbuttons
		    # (MESSAGE)		... message text widget
		    # (WAIT_TEXT)	... wait message text or ""
		    # (<type>,COLOUR)   ... result-panel for nodes of <type>

    variable	main_params	    ;# resource data-base params for main-widget
		    # (resultSize)	... result panel # of columns and rows
		    # (resultMin)	... max width of result panel
		    # (resultMax)	... min width of result panel
		    # (selCol)		... initial number of res. panel rows



	# See the module main.tcl for comments on the global variables



# Create the main panel
# ---------------------

    proc InitPanel { } {

	global		global_vars
	global		button_value

	variable	main_widgets
	variable	main_params	

	variable	::Config::conf_params
	variable	::Config::conf_descr
	variable	::Config::conf_default

	set BORDERS	$conf_params(windowMargin)
	
	set MODULE	[ string trim [namespace current] ":" ]

	if { [ array size select_table ] == 0 } {
	    AuxProcs::DefineParameters [ namespace current]::main_params . {  \
			{ resultSize	} { resultMin		} \
			{ resultMax	} { resultRows	}  }
	}


	bind	. <ButtonPress-1> { HelpAux::PopupClear}

	wm	resizable . 0 0

	set	f_outer .outer
	frame	$f_outer
	pack	$f_outer -pady $BORDERS -fill both -expand true

	set x_colour [ $f_outer cget -background ]
	. configure -background $x_colour


	    # Frame names (order as frames appear, top to bottom)
	    # ---------------------------------------------------
	    #	(Define here to avoid forward referencing problems)

	set f_top	$f_outer.top	;# principal control buttons
	set f_xfer	$f_outer.xfer	;# source, destination, transfer direct.
	set f_middle	$f_outer.middle	;# option menu button and checkbuttons
	set f_progress	$f_outer.progress ;# rsync action progress indication
	set f_result	$f_outer.result	;# result output zone

	set main_widgets(TOP)		$f_top
	set main_widgets(PROGRESS)	$f_progress
	set main_widgets(DATA)		$f_result.data


	    # Top zone: buttons
	    # =================

		# Button row: Kill, Terminal and Quit button
	
	frame $f_top
	pack  $f_top -side top -fill x -padx $BORDERS


	        # Config menubutton

	set x_menu $f_top.config.menu
	menubutton $f_top.config -text "Config" -menu $x_menu
	pack $f_top.config -side left


		# File menu

	menu $x_menu -tearoff false -postcommand [ list \
					MainAux::PostConfig $x_menu ]
	foreach x_item { \
	    { "Kill - no save"		"bind . <Destroy> { MainAux::MainKill\
							. 1 } ; destroy ." } \
	    { } \
	    { "Option buttons, manual layout ..." "OptionDialog::BoxDialog ." }\
	    { "Option buttons, default layout" "MainProcs::BoxDefault" } \
	    { } \
	    { "Create new profile ..." \
				"ProfileDialog::EditProfile . create" } \
	    { "Re-define a profile ..."	\
				"ProfileDialog::EditProfile . re-define" } \
	    { "Delete a profile ..." \
				"ProfileDialog::EditProfile . delete" } \
	    { "Display profile settings ..." \
				"ProfileDialog::EditProfile . display" } \
	    { } \
	    { "Synchronisation-zone editor ..." "TopItems::EditItems . " } \
	    { "Various settings & parameters ..." \
					"ProfileDialog::GeneralSettings ." } } {

	    if { [llength $x_item ] == 0 } {
		$x_menu add separator
		continue
	    }
	    set x_text [ lindex $x_item 0 ]
	    set x_args [ lindex $x_item 1 ]
	    eval "$x_menu add command -label \"$x_text\" -command [ list \
								$x_args ]"
	}


		# Sync-zone and node selection 

	button	$f_top.path -text "Directory & nodes ..." -command \
					[ list SelectDialog::SelectDialog . ]
	pack	$f_top.path -side left -padx $conf_params(buttonSpacing)


		# Save & quit button

	button	$f_top.quit -text "Save & quit" -command  [ list destroy . ]
	pack	$f_top.quit -side right


		# Help button

	button	$f_top.help -text "Help" -command  [ list \
					HelpDialog::HelpPanel $f_top.help "" ]
	pack	$f_top.help -side right -padx \
				[ expr $conf_params(buttonSpacing) + 2 ]


		# Sync action buttons

	pack	[ frame $f_top.sep -width 8 ] -side right
	button	$f_top.probe -text "Dry-run" -command [ list \
						MainAux::DoAction "dry" ]
	pack	$f_top.probe -side right -padx $conf_params(buttonSpacing)

	button	$f_top.doit -text "Synchronise" -command [ list \
					MainAux::DoAction "real" $f_top.doit ]
	pack	$f_top.doit -side right


		# Root warning

	if { $global_vars(ROOT) != 0 } {
	    label   $f_top.root -text "R O O T" -foreground "#ac3062"
	    pack    $f_top.root -anchor center -pady 4
	}


	    # Xfer control zone (source, direction, destination)
	    # ==================================================

		# Top separator

	frame	$f_outer.sep1 -relief raised -height 2 -borderwidth 2
	pack	$f_outer.sep1 -fill x -pady 8

		# Label

	frame	$f_xfer
	pack	$f_xfer -side top -anchor w -padx $BORDERS

	label	$f_xfer.comment -text "Client <-> server:"
	pack	$f_xfer.comment -side left
	pack	[ frame $f_xfer.space -width 6 ] -side left

		# Outer box with ridge

	set	f_path $f_xfer.path
	set	main_widgets(PATH) $f_path
	frame	$f_path
	pack	$f_path -fill x

		    # Source

	set	x_temp $f_path.path1
	label	$x_temp -text ""
	grid 	$x_temp -column 0  -sticky w

		    # Direction

	set	x_temp $f_path.direction
	label	$x_temp
	grid 	$x_temp -column 1 -row 0 -sticky w

		    # Destination

	set	x_temp $f_path.path2
	label	$x_temp -text ""
	grid 	$x_temp -column 2 -row 0 -padx 4 -sticky w

	for { set x_x 0 } { $x_x < 3 } { incr x_x } \
				{ grid columnconfigure $f_path $x_x -weight 0 }
	grid	columnconfigure $f_path 3 -weight 1


	    # Option control zone, left part: preset button and menu
	    # ======================================================

		# Top separator

	frame	$f_outer.sep2 -relief raised -height 2 -borderwidth 2
	pack	$f_outer.sep2 -fill x -pady 8

	frame	$f_middle
	pack	$f_middle -side top -fill x -padx $BORDERS

	set	f_options $f_middle.options
	frame	$f_options
	pack	$f_options -side left -anchor n


		# Preset option button

	label	$f_options.label -text "Options for rsync: "
	pack	$f_options.label -side left

	set	x_menu $f_options.preset.menu
	set	main_widgets(PRESET_MENU) $x_menu
	menubutton $f_options.preset -text "Preset" -menu $x_menu
	pack	$f_options.preset -side left

	# Preset menu: global settings only; profiles only when read from file

	set	f_boxes $f_middle.boxes
	set	main_widgets(BOXES) $f_boxes

	menu	$x_menu -tearoff false
	foreach x_item { \
		{ "All displayed option checkboxes: clear"		  0 } \
		{ "All displayed option checkboxes: mark"		  1 } \
		{ "All displayed option checkboxes: set to default"	  2 } \
		{ "All displayed option checkboxes: set to initial value" 3 } \
		{ } \
		{ "All checkboxes and buttons: set to default"		  + } \
		{ "All options and settings: set to initial value"	  * } \
		{ } \
									    } {
	    if { [llength $x_item ] == 0 } {
		$x_menu add separator
	    } else {
		$x_menu add command -label [ lindex $x_item 0 ] -command [ list\
			MainAux::PresetProfile [ lindex $x_item 1 ] $f_boxes ]
	    }
	}


	    # Option control zone, right part: option checkbuttons
	    # ====================================================
	
		# Preset all variables to 0
		# -------------------------
		#   (configuration file only does those it contains

	foreach x_item [ array names conf_descr *,FLAGS ] {
	    regsub {,FLAGS} $x_item "" x_item

		    # Handle normal buttons and state variables

	    if { [ string first "v" $conf_descr($x_item,FLAGS) ] >= 0 } {
		set global_vars($x_item) 0
		continue
	    } elseif { [ string first "b" $conf_descr($x_item,FLAGS) ] >= 0 }  {
		set button_value($x_item) 0
		continue
	    }
	    set button_value($x_item) 0
	}

		    # Other default values

	foreach x_item [ array names conf_descr *,FLAGS ] {
	    set x_flags $conf_descr($x_item)
	    if { [ string first "v" $x_flags ] < 0 } continue
	    regsub {,FLAGS} $x_item "" x_tag
	    set global_vars($x_tag) $conf_default($x_tag,VALUE)
	}


		# Initialise the configuration
		# ----------------------------

		    # Define frame for checkboxes, and fill in checkboxes

	frame $f_boxes
	pack  $f_boxes -side right -anchor n


		    # Get the configuration data from the configuration file
			# if it does not exist, establish default values

	set global_vars(SYNC_ZONE) ""
	if { [ file exists $conf_params(configFile) ] == 0 } {
	    MainAux::PresetProfile "+"
	    MainProcs::MainCheckboxes
	    MainProcs::BoxDefault
	    if [ regexp {([^,]+),(.+)$} $conf_params(dummyZone) "" \
						x_server x_client ] {
		if { [ regexp {^([^@]+)@(.+)} $x_server "" \
						sv_base sv_host ] == 0 } {
		    set sv_base $x_server
		    set sv_host ""
		}

		set x_activate [ AuxProcs::DefineZone $x_client $sv_base \
								$sv_host "" ]
	    } else {
		set x_activate ""
	    }
	} else {
	    set zone_list [ ConfigFile::ReadConfig ]
	    if { $global_vars(SYNC_ZONE) != "" } {
		set x_activate $global_vars(SYNC_ZONE)
	    } elseif { [ llength $zone_list ] > 0 } {
		set x_activate [ lindex $zone_list 0 ]
	    } else {
		set x_activate ""
	    }
	    MainProcs::MainCheckboxes
	}
	if { $x_activate != "" } {
	    regsub {^/} $global_vars(REL_PATH) "" x_temp
	    SelectAux::NewTop $x_activate "" $x_temp
	    MainProcs::PathLabels
	}
	set x_enable [ MainProcs::EnableButtons ]
	log_user $button_value(CONSOLE_ECHO)
	MainProcs::SetDirection


		# Bottom separator

	frame	$f_outer.sep3 -relief raised -height 2 -borderwidth 2
	pack	$f_outer.sep3 -fill x -pady 6 -side top


	    # Action progress information
	    # ===========================
		#   4 fields:
		#	1st ... current file number
		#	2nd ... progress percentage
		#	3rd ... current file name
		#	4th ... count of files considered by rsync

	frame $f_progress
	pack $f_progress -side top -fill x -padx $BORDERS
	set x_x 0

		# field descriptor list:
		#		item 0	minimum width
		#		item 1	sticky code
		#		item 2	grid weight
		#		item 3  if b: bold font

	foreach x_field { { 60 e 0 } { 70 w 0 } { 100 w 1 b } { 150 e 0 b } } {
	    if { [ lindex $x_field 3 ] == "b" } { set x_font standard } else \
							{ set x_font normal }
	    label $f_progress.field_$x_x -text "" -font $x_font
	    grid  $f_progress.field_$x_x -column $x_x -row 0 \
						-sticky [ lindex $x_field 1 ]
	    grid  columnconfigure $f_progress $x_x -minsize \
			[ lindex $x_field 0 ] -weight [ lindex $x_field 2 ]
	    incr x_x
	}
	set main_widgets(MESSAGE) $f_progress.field_2
	set main_widgets(WAIT_TEXT) ""


	    # Create outer frame for result panel
	    # ===================================
		# row 0	... top space
		# row 1 ... header row
		# row 2 ... result panel
		# row 3 ... bottom space (adjusted for overall size of panel
		#					to be multiple of step)

	frame	$f_result
	pack	$f_result -side top -anchor w -fill both -expand true \
								-padx $BORDERS

	grid	columnconfigure $f_result 0 -minsize 20
	grid	columnconfigure $f_result 0 -weight 0
	grid	columnconfigure $f_result 1 -weight 1

	set	main_widgets(HEADER_ROW) 1
	set	main_widgets(RESULT_ROW) 2

	grid	rowconfigure	$f_result 0 -minsize 2
	grid	rowconfigure	$f_result 3 -minsize 0
	grid	rowconfigure	$f_result 0			    -weight 0
	grid	rowconfigure	$f_result $main_widgets(HEADER_ROW) -weight 0
	grid	rowconfigure	$f_result $main_widgets(RESULT_ROW) -weight 1
	grid	rowconfigure	$f_result 3			    -weight 0

		# Header with result label, colour and version indications

	set f_temp $f_result.header
	frame $f_temp
	grid $f_temp -row $main_widgets(HEADER_ROW) -column 0 -columnspan 2 \
								-sticky ew
	 
	set main_widgets(ACTION) $f_temp.label

	label $main_widgets(ACTION) -text "Result:"
	pack  $main_widgets(ACTION) -side left

	foreach x_item { link delete update error skip } {
	    label $f_temp.$x_item -text (${x_item}) -font normal
	    pack $f_temp.$x_item -side left
	    set main_widgets($x_item,COLOUR) \
					[ $f_temp.$x_item cget -foreground ]
	}

	checkbutton $f_temp.skipped -text "display non-regular nodes" -font \
				standard -variable button_value(SHOW_SKIPPED)
	pack	    $f_temp.skipped -side left -padx 15


	label	$f_temp.version -text $::Config::version_string -font small
	pack	$f_temp.version -side right


	    # Final set-up
	    # ------------
	    #	(all parts of panel do no exist, control variables are defined

		# Finalise title, catch destroy events
		#    catch <Destroy> only now (earlier would risk overwriting 
		#    the config file with configurtion data that has not yet 
		#    been initialised

	bind	. <Destroy> [ list MainAux::MainKill . ]
	wm	title . "Synchronise directories on local host\
						\"$global_vars(CLIENT_HOST)\""

		# Initialise the result panel

	set main_widgets(ROW_NUMB) $main_params(resultRows)
	set main_widgets(SCROLL_WIDE) 0
	MainProcs::VoidResult
	bind $main_widgets(DATA) <Configure> [ list MainAux::ResizePanel %h %W ]
	update idletasks


		# Finalise geometry settings
		#    adjust hight of final dummy row of result panel to make 
		#    the total hight a multiple of the resultpanel row height

	MainProcs::AdjustGeometry


		# Make the top-level window visible

	wm deiconify .
	tkwait visibility .


		# Pronounce a warning if there are no filters

	if { ( $button_value(AUTO_FILTER) || $button_value(USER_FILTER) || \
					$button_value(DIR_FILTER) ) == 0 } {
	    set x_reply [ Dialog::DialogMessage . Warning "" [ list \
		{All filter options are turned off ! Enable the use of, at\
							least, one of} \
		{\30- the automaticcally generated filter,} \
		{\30- a user-provided filter file,} \
		{\30- per-directory filter files.} \
		{\6} {The "More ..." button provides access to the option\
							selection window !} \
		{\12} {Do you want to continue and correct the settings,\
								or quit?} ] \
		{ Continue Quit } ]
	    if { $x_reply != 0 } exit
	}


		# Pronounce a warning if there are no enabled resources

	if { [ file exists $conf_params(configFile) ] == 0 } {
	    set x_message [ list \
	        "The configuration file \"$::Config::conf_params(configFile)\"\
							does not exist !" \
		{\6} "\\30\u2022 The option checkbuttons and their layout are\
						set to default values." \
		{\6} {\80You may want to do some adjustments and then to\
							define profiles.}
	    ]
	    if { $x_activate != "" } { lappend x_message \
		{\12} "\\30\u2022 A dummy synchronisation-zone exists to\
				facilitate rapid familarisation with Usync." \
		{\8} {\80Delete this zone or change it to a comment and\
					define your proper configuration: } \
		{\80use "Config" -> "Synchronisation-zone editor".}
	    }
	    lappend x_message \
		{\12} {If you continue, all settings will be saved to the\
			configuration file when you exit with "Save & quit"}
	    set x_reply [ Dialog::DialogMessage . Warning "" $x_message \
							{ Continue Kill } ]
	    if { $x_reply != 0 } exit
	} elseif { $x_enable != "normal" } {
	    set x_reply [ Dialog::DialogMessage . Warning "" [ list \
		{No synchronisation zone has been configured - or all hosts\
							 have been suspended.} \
		{\12} {To get the operational buttons enabled, you must define\
					at least one synchronisation-zone,} \
		{\60use:  "Config" -> "Synchronisation-zone editor".} ] \
		{ Continue Kill } ]
	    if { $x_reply != 0 } exit
	}


		# Control logging

	if { $button_value(WRITE_LOG) == 0 } {
	    set x_reply [ Dialog::DialogMessage . Warning "" [ list \
		{The logging facility is disabled !} \
		{\40(logging can be switched on and offby doing} \
		{\40"File" -> "Set timeout, console-echo, etc")} \
		{\8} {Enable loggin now, or keep it disabled?} ] \
		{ Enable Disable } ]
	    if { $x_reply == 0 } { set button_value(WRITE_LOG) 1 }
	}

	if { $button_value(WRITE_LOG) } {
	    MainAux::LogItem "" open
	    MainAux::LogItem "" session
	}
	return
    }
}
