package provide MainProcs



# Main panel support procedures
# =============================
#	- I/O to main panel widgets
#	- Result panel creation and I/O
#	- Auxiliary procedures



namespace eval MainProcs {



# Result panel handling procedures
# ================================


# Arrange visual presentation of result panel
# -------------------------------------------
#	after filling the panel without updating visual panel state

    proc ShowAdapt { } {

	variable	::MainPanel::main_widgets
	variable	::MainPanel::main_params

	    # Initialise variables
	    # --------------------
	    #	x_canvas	... canvas widget
	    #   f_data		... outer data-field panel (sunken frame)

	update idletasks
	set x_canvas [ winfo parent $main_widgets(CANVAS) ]
	set f_data $main_widgets(DATA)
	if { [ info exists main_widgets(STEP) ] == 0 } {
	    set main_widgets(STEP) [ expr \
		[ lindex [ grid bbox $main_widgets(CANVAS) 0 0 ] 3 ] - 2 ]
	}
	set x_step $main_widgets(STEP)


	    # Horizontal settings
	    # -------------------
	    #	t_width	... text field width
	    #	v_width	... view window width
	    
	set t_width [ winfo width $main_widgets(CANVAS) ]
	if { $t_width <= $main_params(resultMax) } {
	    set v_width $t_width
	} else {
	    set v_width $main_params(resultMax)
	    if { [ winfo exists $f_data.xscroll ] == 0 } {
		scrollbar $f_data.xscroll -orient horizontal \
				-command [ list $f_data.canvas xview ]
	        grid $f_data.xscroll -column 0 -row 1 -sticky ewn
		$x_canvas configure -xscrollcommand [ list $f_data.xscroll \
						set ] -xscrollincrement 1

		grid rowconfigure $f_data 0 -weight 0
		grid rowconfigure $f_data 1 -weight 1
	    }
	}


	    # Vertical settings
	    # -----------------
	    #	t_rows	  ... number of text rows
	    #	v_rows	  ... number of rows in view window
	    #	t_height  ... text field height

	set t_rows $main_widgets(ITEM_ROW)
	if { $t_rows == 0 } { set t_rows 1 }

	if { $t_rows > $main_widgets(ROW_NUMB) } {
	    set v_rows $main_widgets(ROW_NUMB)
	    if { [ winfo exists $f_data.yscroll ] == 0 } {
		scrollbar $f_data.yscroll -orient vertical \
				-command [ list $f_data.canvas yview ]
		set main_widgets(SCROLL_WIDE) [ $f_data.yscroll cget -width ]
		grid $f_data.yscroll -column 1 -row 0 -sticky nse

		$x_canvas configure -yscrollcommand [ list $f_data.yscroll \
						set ] -yscrollincrement $x_step
	    }
	} else {
	   set v_rows $t_rows
	}

	    # Resize the top-level window and wait
	    # ----------------------------------

	set t_height [ expr $t_rows * $x_step ]
	$x_canvas configure -width $v_width -height [ expr $v_rows * $x_step ] \
					-scrollregion "0 0 $t_width $t_height"
	if [ winfo exists $f_data.yscroll ] { $f_data.canvas yview \
								moveto 1.0 }
	update idletasks
	wm geometry . ""
	return [ list $v_rows $t_rows ]
    }



# Add a new item to the result panel
# ----------------------------------

    proc AddResult { p_text { p_flag "" } } {
			  # p_flag	""	... ordinary sync entry
			  #		none	... show "empty" message
			  #		skip	... a skipped entry
			  #		delete	... delete this entry
			  # p_text	item text to store
			  #		if void and first position:
			  #			show  ** none **

	variable	::MainPanel::main_widgets
	variable	::MainPanel::main_params

	set x_col $main_widgets(ITEM_COL)
	set x_row $main_widgets(ITEM_ROW)
	set x_item $main_widgets(CANVAS).r_${x_col}_$x_row

	set x_colour $main_widgets(update,COLOUR)
	set x_text $p_text
	set x_merge 1
	set x_sticky "w"
	set x_fresh 0

	switch $p_flag {
	    "done" {
		set x_text "** $p_text **"
		set x_fresh 1
		set x_sticky "ew"
	    }

	    "skip" {
		set x_colour $main_widgets(skip,COLOUR)
		if { [ regexp {.*unsafe.*link\s+\"([^\"]+)\".*} \
						$p_text "" x_text ] == 0 } {
		    puts stderr "rsync handler: skipping    $p_text"
		}
	    }

	    "delete" {
		set x_colour $main_widgets(delete,COLOUR)
	    }

	    "error" {
		set x_fresh 1
		set x_merge [ lindex $main_params(resultSize) 0 ]
		set x_colour $main_widgets(error,COLOUR)
	    }

	    default {
		if [ regexp {^(.+) \-> } $p_text "" x_text ] { set x_colour \
						$main_widgets(link,COLOUR) }
	    }
	}
	if { $x_fresh != 0 } {
	    if { $x_col != 0 } {
		set x_col 0
		incr x_row
		set main_widgets(ITEM_COL) 0
		incr main_widgets(ITEM_ROW)
	    }
	    set x_merge [ lindex $main_params(resultSize) 0 ]
	}
	label $x_item -text $x_text -font normal -fg $x_colour -pady 0
	grid  $x_item -column $x_col -row $x_row -padx 6 -sticky $x_sticky \
							-columnspan $x_merge
	if { $x_fresh != 0 } {
	    set x_scroll $main_widgets(DATA).xscroll
	    incr main_widgets(ITEM_ROW)
	    incr x_row
	    ShowAdapt
	    if { [ winfo exists $x_scroll] && [ winfo ismapped $x_scroll ] && \
						( $p_flag == "done" ) } {

		# "Done" and the horizontal scrollbar is visible:
		#	center the horizontal scrollbar

		set x_list [ $x_scroll get ]
		set x_temp [ expr [ lindex $x_list 0 ] + ( 1.0 - \
						[ lindex $x_list 1 ] ) / 2 ]
		eval "[ $x_scroll cget -command ] moveto $x_temp"
	    }
	} else {
	    incr main_widgets(ITEM_COL)
	    if { $main_widgets(ITEM_COL) >= \
				[ lindex $main_params(resultSize) 0 ] } {
		set main_widgets(ITEM_COL) 0
		incr main_widgets(ITEM_ROW)
		ShowAdapt
	    }
	}
    }



# Create a void result panel
# --------------------------
#	If a result panel already exists, destroy it

    proc VoidResult { } {

	variable	::MainPanel::main_widgets
	variable	::MainPanel::main_params

	set f_data $main_widgets(DATA)


	    # If an old panel exists, delete it
	    # ---------------------------------

	if [ winfo exists $f_data ] {
	    set y_save [ bind $f_data <Configure> ]
	    set x_save [ bind . <Destroy> ]
	    bind . <Destroy> {}
	    destroy $f_data
	    eval "bind . <Destroy> [ list $x_save ]"
	    update idletasks
	} else {
	    set y_save ""
	}


	    # Create new result panel
	    # -----------------------

	frame $f_data -relief sunken -borderwidth 2
	grid $f_data -column 1 -row $main_widgets(RESULT_ROW) -sticky ewns

	if { $y_save != "" } { bind $f_data <Configure> $y_save }
	set main_widgets(ITEM_COL) 0
	set main_widgets(ITEM_ROW) 0


	    # Create canvas
	    # -------------
	    #	column 0 ... data
	    #	column 1 ... vertical scrollbar

	canvas $f_data.canvas -height 0 -highlightthickness 0 
	grid $f_data.canvas  -column 0 -row 0 -sticky ew
	grid columnconfigure $f_data 0 -weight 1
	grid columnconfigure $f_data 1 -weight 0

	grid rowconfigure $f_data 0 -weight 1
	grid rowconfigure $f_data 1 -weight 0

	set main_widgets(CANVAS) $f_data.canvas.f
	frame $main_widgets(CANVAS) -borderwidth 0
	$f_data.canvas create window 0 0 -window $main_widgets(CANVAS) \
								-anchor nw
	set x_min [ expr $main_params(resultMin) / \
					[ lindex $main_params(resultSize) 0 ] ]
	for { set x_x 0 } { $x_x < [ lindex $main_params(resultSize) 0 ] } \
								{ incr x_x } { \
	    grid columnconfigure $main_widgets(CANVAS) $x_x -weight 0 \
								-minsize $x_min
	}

	set x_item $main_widgets(CANVAS).l_0_0
	label $x_item -text ""
	grid  $x_item -column 0 -row 0
	ShowAdapt
    }



# Main panel widget I/O procedures
# ================================


# Update the path-name frame if the main panel
# --------------------------------------------
#	Structure of the main_widgets(PATH) gridded frame:
#	    column 0 ... client label (local host)
#	    column 1 ... transfer direction arrow button
#	    column 2 ... server label (remote host)

    proc PathLabels { } {

	global		global_vars

	variable	::MainPanel::main_widgets

	set x_temp $global_vars(REL_PATH)
	set x_text(0) $global_vars(CLIENT_BASE)$x_temp
	set x_text(2) $global_vars(SERVER_BASE)$x_temp
	if { $global_vars(SERVER_HOST) != "" } { set x_text(2) \
					$x_text(2)@$global_vars(SERVER_HOST) }
	for { set x_x 0 } { $x_x < 3 } { incr x_x 2 } {
	    set x_widget [ grid slaves $main_widgets(PATH) -column $x_x ]
	    $x_widget configure  -text $x_text($x_x)
	}
    }



# Define contenst of progress indication field
# --------------------------------------------

    proc ProgressInfo { p_field p_text } {
			  # p_field	index of the field to define
			  # p_text	text to set into the field

	variable	::MainPanel::main_widgets

	if { $main_widgets(WAIT_TEXT) != "" } { WaitMessage "" }
	set x_widget [ grid slaves $main_widgets(PROGRESS) -column $p_field ]
	$x_widget configure -text $p_text

	    # Give the name a chance to become visible

	if { $p_field == 2 } { update idletasks }
    }



# Wait status message
# -------------------

    proc WaitMessage { p_text } {
			  # p_text	"" ... cancel message text

	variable	::MainPanel::main_widgets

	set x_widget $main_widgets(MESSAGE)
	if { $p_text == "" } {
	    if { $main_widgets(WAIT_TEXT) == "" } return
	    set x_sibling [ grid slaves [ winfo parent $x_widget ] -column 0 ]
	    set x_colour [ $x_sibling cget -foreground ]
	    set x_sticky "w"
	} else {
	    set x_colour red
	    set x_sticky "ew"
	}
	set main_widgets(WAIT_TEXT) $p_text
	$x_widget configure -text $p_text -foreground $x_colour
	grid configure $x_widget -sticky $x_sticky
	update idletasks
    }



# Make the size of the main-panel a multiple of the result-panel step size
# ------------------------------------------------------------------------

    proc AdjustGeometry { } {

	variable	::MainPanel::main_widgets

	set f_result [ winfo parent $main_widgets(DATA) ]
	grid rowconfigure $f_result 3 -minsize 0
	update idletasks

	regexp {(\d+)\D+(\d+)\D+} [ wm geometry . ] "" x_width x_height
	set x_temp [ expr $main_widgets(STEP) - ( $x_height % \
							$main_widgets(STEP) ) ]
	grid rowconfigure $f_result 3 -minsize $x_temp
	incr x_height $x_temp
	wm grid . $x_width $x_height 1 $main_widgets(STEP)
	set x_temp [ expr $x_height / $main_widgets(STEP) ]
	wm minsize . 0 $x_temp
	wm maxsize . 0 $x_temp
	set main_widgets(ROW_BASE) $x_temp
    }



# Auxiliary procedures
# ====================


# Enable / disable top-buttons in main panel
# ------------------------------------------
#	Enable if at least one synchronisation object is defined - if there is
#	none, disable
#
#	Return "normal" if enabled, else return "disabled"

    proc EnableButtons { } {

	global		global_vars

	variable	::MainPanel::main_widgets

	if { $global_vars(SYNC_ZONE) == "" } { set x_state disabled } else \
							{ set x_state normal }
	foreach x_widget [ winfo children $main_widgets(TOP) ] {
	    if { [ winfo class $x_widget ] != "Button" } continue
	    if { [ $x_widget cget -text ] == "Help" } continue
	    if { [ lindex [ $x_widget cget -command ] 0 ] == "destroy" } \
									continue
	    $x_widget configure -state $x_state
	}
	return $x_state
    }



# Enable / disable all children of a widget
# -----------------------------------------
#	that are of class button, menubutton, checkbutton

    proc SetState { p_frame p_state { p_inner "" } } {
			  # p_frame	fame with children to enable / disable
			  # p_state	state in which to set the children
			  #				("normal" or "disabled")
			  # p_inner	"" if outer call, 1 of recursion

	set x_children [ winfo children $p_frame ]
	foreach x_widget $x_children {
	    set x_class [ winfo class $x_widget ]
	    if { $x_class == "Frame" } {
		SetState $x_widget $p_state 1
		continue
	    } elseif { [ lsearch { Button Checkbutton Menubutton } \
							$x_class ] < 0 } {
		continue
	    }
	    $x_widget configure -state $p_state
	}
	if { ( $p_state == "normal" ) && ( $p_inner == "" ) } EnableButtons
    }



# Set the checkbuttons for the main-panel
# ---------------------------------------

    proc MainCheckboxes { } {

	variable	::MainPanel::main_widgets

	variable	::Config::conf_params

	set x_frame $main_widgets(BOXES)
	set x_coord [ AuxProcs::CheckboxArray $x_frame "M" 0 0 ]

	set x_col [ lindex $x_coord 0 ]
	set x_row [ expr [ lindex $x_coord 1 ] + 1 ]
	set f_temp $x_frame.temp
	frame $f_temp
	grid $f_temp -column $x_col -row $x_row -sticky e

	button $f_temp.more -text "More ..." -padx 3 -pady 1 \
				-command { SettingDialog::MoreOptions . }
	pack $f_temp.more -side right

	set x_temp $f_temp.help
	label $x_temp -text "?" -padx 2 -pady 2 -relief raised -borderwidth 2
	pack $x_temp -side right -padx $conf_params(buttonSpacing)

	bind $x_temp <ButtonPress-1>   [ list HelpAux::PopupPress $x_temp ]
	bind $x_temp <ButtonRelease-1> [ list HelpAux::PopupButtonup \
								$x_frame %X %Y ]

    }



# Set the layout of the main-panel checkbuttons to default
# --------------------------------------------------------

    proc BoxDefault { } {

	global		button_coord

	variable	::Config::conf_descr
	variable	::Config::conf_default

	if [ info exists button_coord ] { unset button_coord }
	foreach x_item [ array names conf_descr *,FLAGS ] {
	    regsub {,FLAGS} $x_item "" x_item
	    if { [ string first "M" $conf_descr($x_item,FLAGS) ] < 0 } continue
	    if { [ string first "c" $conf_descr($x_item,FLAGS) ] < 0 } continue
	    if { [ info exists conf_default($x_item,PROPS) ] == 0 } continue
	    set button_coord($x_item) $conf_default($x_item,PROPS)
	}
	MainCheckboxes
    }



# Set transfer direction marker
# -----------------------------
#	Structure of the main_widgets(PATH) gridded frame:
#	    column 0 ... client label (local host)
#	    column 1 ... transfer direction arrow button
#	    column 2 ... server label (remote host)

    proc SetDirection { } {

	global		button_value
	global		global_vars

	variable	::MainPanel::main_widgets

	if { $button_value(XFER_DIRECTION) == 0 } \
			{ set x_label left.xbm } else { set x_label right.xbm }
	set x_widget [ grid slaves $main_widgets(PATH) -column 1 ]
	$x_widget configure -bitmap @[ file join $global_vars(LIB_DIR) \
								$x_label ]
    }
}
