package provide OptionDialog



# Select the option checkbuttons to appear in the main panel, arrange the layout
# ==============================================================================



namespace eval OptionDialog {


    variable w_data		;# widget data
		# (MODULE)	    module name
		# (DRAG)	    widget being dragged, "" if none
		# (0)		    upper zone frame widget 
		# (1)		    lower zone frame widget
		# (SELECT)	    selected field
		# (SHIFT)	    add/delete menu button widget

    variable c_state		;# state variable
		# (<n>,...)	    <n> = 0/1  upper/lower zone
		#
		# (<n>,MAX_X)	    highest grid columns
		# (<n>,MAX_Y)	    highest grid row
		#
		# (<n>,<x>,<y>	    label for field at column <x> row <y> in <n>
		# (<n>,<label>	    list { <x> <y> } (column and row) for item 
		#							<label>

    variable option_params	;# parameters from resource data base
		# (fieldWidth)	    min width of option field


# Auxiliary procedures
# ====================


# Retrieve grid values for a given widget
# ---------------------------------------
#	Return a 3-item list
#	    item 0 ...	column
#	    item 1 ...	row
#	    item 2 ...	0 if upper zone, 1 of lower zone

    proc FieldData { p_widget } {
			  # p_widget	object widget

	set x_list [ grid info $p_widget ]
	set x_x [ lsearch $x_list -column ]
	set x_x [ lindex $x_list [ expr $x_x + 1 ] ]
	set x_y [ lsearch $x_list -row ]
	set x_y [ lindex $x_list [ expr $x_y + 1 ] ]
	set x_n [ lsearch $x_list -in ]
	regexp {\.(\d+)\.} [ lindex $x_list [ expr $x_n + 1 ] ] "" x_n
	return  [ list $x_x $x_y $x_n ]
    }



# Create a new field
# ------------------

    proc NewField { p_frame p_x p_y p_text } {
			  # p_frame	frame where field is to be created
			  # p_col	grid column for new field
			  # p_row	grid row for new field
			  # p_text	text for label widget

	variable	w_data
        variable	option_params

	regexp {^(\.[^\.]+)\.} $p_frame "" x_dialog
	set x_box $p_frame.l_${p_x}_$p_y
	label $x_box -anchor w -padx 6 -pady 3 \
				-relief sunken -borderwidth 1 -text $p_text 
	grid  $x_box -column $p_x -row $p_y -sticky ew
	grid  columnconfigure $p_frame $p_x -minsize $option_params(fieldWidth)
	bind $x_box <ButtonPress-1> [ list \
				[ namespace current ]::FieldPress $x_box ]
	bind $x_box <ButtonRelease-1> [ list \
				[ namespace current ]::FieldRelease $x_box ]
    }



# Update zone to make display match the data from c_state array
# -------------------------------------------------------------

    proc UpdateZone { p_zone } {
			  # p_zone	0 or 1

	variable	c_state
	variable	w_data

	variable	::Config::conf_descr

	SelectField
	set x_frame $w_data($p_zone)
	set x_dialog [ winfo toplevel $x_frame ]

	    # For the range defined in c_state, update the frame

	set x_void 0
	for { set x_y 0 } { $x_y <= $c_state($p_zone,MAX_Y) } { incr x_y } {
	    for { set x_x 0 } { $x_x <= $c_state($p_zone,MAX_X) } { incr x_x } {
		set x_text ""
		if [ info exists c_state($p_zone,$x_x,$x_y) ] {
		    set x_label $c_state($p_zone,$x_x,$x_y)
		    set x_text $conf_descr($x_label,TEXT)
		} else {
		    incr x_void
		}
		set x_widget [ grid slaves $x_frame -column $x_x -row $x_y ]
		if { [ llength $x_widget ] == 0 } {
		    NewField $x_frame $x_x $x_y $x_text
		} else {
		    $x_widget configure -text $x_text
		}
	    }
	}

	    # Destroy excess cells (the frame may have shrunken)

	set y_max [ lindex [ grid size $x_frame ] 1 ]
	set y_min [ expr $c_state($p_zone,MAX_Y) + 1 ]
	set x_save [ bind $x_dialog <Destroy> ]
	bind $x_dialog <Destroy> {}
	for { set x_y $y_min } { $x_y < $y_max } { incr x_y } {
	    set x_list [ grid slaves $x_frame -row $x_y ]
	    foreach x_widget $x_list { destroy $x_widget }
	}
	eval "bind $x_dialog <Destroy> [ list $x_save ]"
	if { $p_zone == 0 } return

	    # Lower zone. Make sure that there is at least one empty field

	if { $x_void == 0 } {
	    incr c_state(1,MAX_Y)
	    for { set x_x 0 } { $x_x <= $c_state(1,MAX_X) } { incr x_x } \
				{ NewField $x_frame $x_x $c_state(1,MAX_Y) "" }
	} elseif { $x_void > [ expr $c_state(1,MAX_X) + 1 ]} {
	    set x_list [ grid slaves $x_frame -row $c_state(1,MAX_Y) ]
	    set x_save [ bind $x_dialog <Destroy> ]
	    bind $x_dialog <Destroy> {}
	    foreach x_widget $x_list { destroy $x_widget }
	    eval "bind $x_dialog <Destroy> [ list $x_save ]"
	}
    }



# Tidy lower frame
# ----------------
#	Squeeze out holes from the middle
#	Make sure there is at least one blank field

    proc TidyZone1 { } {

	variable	c_state
	variable	w_data

	variable	::Config::conf_descr

	set p_frame $w_data(1)
	set x_list { }
	for { set x_y 0 } { $x_y <= $c_state(1,MAX_Y) } { incr x_y } {
	    for { set x_x 0 } { $x_x < 3 } { incr x_x } {
		if { [ info exists c_state(1,$x_x,$x_y) ] == 0 } continue
		lappend x_list $c_state(1,$x_x,$x_y)
	    }
	}

	set lim_x $c_state(1,MAX_X)
	set lim_y $c_state(1,MAX_Y)
	set x_index 0
	set x_void 0
	for { set x_y 0 } { $x_y <= $lim_y } { incr x_y } {
	    for { set x_x 0 } { $x_x <= $lim_x } { incr x_x } {
		if { $x_index < [ llength $x_list ] } {
		    set x_item [ lindex $x_list $x_index ]
		    set c_state(1,$x_x,$x_y) $x_item
		    set c_state(1,$x_item) [ list $x_x $x_y ]
		    incr x_index
		    set x_text $conf_descr($x_item,TEXT)
		} else {
		    incr x_void
		    set x_text ""
		    if [ info exists c_state(1,$x_x,$x_y) ] \
						{ unset c_state(1,$x_x,$x_y) }
		}
	    }
	}
	UpdateZone 1
    }



# Select / De-select a field
# --------------------------

    proc SelectField { { p_field "" } } {
			  # p_field	if 0: deselect if something is selected
			  #		else: select that field

	variable	w_data

	    # Set selection state

	set x_shift disabled
	if { $p_field == "" } {
	    set p_field $w_data(SELECT)
	    if { $p_field == "" } return
	    set x_relief sunken
	    set w_data(SELECT) ""
	} else {
	    set x_relief raised
	    set w_data(SELECT) $p_field
	    set x_list [ FieldData $p_field ]
	    if { [ lindex $x_list 2 ] == 0 } { set x_shift normal }
	}
	$p_field configure -relief $x_relief

	    # Enable / disable the shift menu button

	$w_data(SHIFT) configure -state $x_shift
    }



# Callback procedures
# ===================


# Button-1 motion callback
# ------------------------
#	Check whether cursor is over potential drag-destination
#	    if yes:
#		- set "icon" cursor
#		- remember the drag-destination
#	    if no:
#		- claar drag-destination
#		- set "target" cursor
 	

    proc FieldMove { p_widget p_x p_y } {
			  # p_widget	widget at origin of motion
			  # p_x, p_y	current x- and y-coordinate

	variable	w_data

	    # Check if widget under the cursor
	    #   - is within upper or lower zone
	    #   - and is blank (ready to act as drag-destination)
	    #		x_widget ... widget under cursor

	if { $w_data(DRAG) == "" } return	;# return if no drag initialised
	set x_widget [ winfo containing $p_x $p_y ]
	set x_dialog [ winfo toplevel $p_widget ]
	if { ( ( [ string first $w_data(0) $x_widget ] == 0 ) || \
		 ( [ string first $w_data(1) $x_widget ] == 0 ) ) && \
					( [ $x_widget cget -text ] == "" ) } {
	    $x_dialog configure -cursor icon
	    set w_data(RECEIVE) $x_widget
	} else {
	    $x_dialog configure -cursor target
	    set w_data(RECEIVE) ""
	}
    }



# Button-down event callback
# --------------------------
#	Skip if the hit field is blank (cannot be dragged)

    proc FieldPress { p_field } {
			  # p_field	widget receiving the button-down hit

	variable	w_data
	variable	c_state

	    # Do nothing if a blank field. Else set the drag cursor ("target")

	set x_dialog [ winfo parent [ winfo parent $p_field ] ]
	if { [ $p_field cget -text ] != "" }  {
	    set w_data(DRAG) $p_field
	    $x_dialog configure -cursor target
	}

	    # De-select old selection (if there is one), set new selection

	SelectField
	SelectField $p_field
    }



# Callback for delete-row menu button
# -----------------------------------
#	Delete a row from the upper or lower zone

    proc DeleteRow { } {

	variable	w_data
	variable	c_state

	set x_list [ FieldData $w_data(SELECT) ]
	set x_zone [ lindex $x_list 2 ]
	set x_frame $w_data($x_zone)
	set x_dialog [ winfo toplevel $x_frame ]
	set x_row [ lindex $x_list 1 ]

	    # Check whether the selected row is empty, abandon if not

	set x_list [ grid slaves $x_frame -row $x_row ]
	foreach x_widget  $x_list {
	    if { [ $x_widget cget -text ] != "" } {
		Dialog::DialogMessage $x_dialog Warning "" [ list \
			"Only empty rows can be deleted" \
			"This row is not empty !" ] OK
		return
	    }
	}

	    # Shift all items beyond the deleted row one row down

	incr c_state($x_zone,MAX_Y) -1
	for { set x_y $x_row } { $x_y <= $c_state($x_zone,MAX_Y) } \
								{ incr x_y } {
	    set x_yy [ expr $x_y + 1 ]
	    for { set x_x 0 } { $x_x <= $c_state($x_zone,MAX_X) } { incr x_x } {
		if { [ info exists c_state($x_zone,$x_x,$x_yy) ] == 0 } continue
		set x_label $c_state($x_zone,$x_x,$x_yy)
		unset c_state($x_zone,$x_x,$x_yy)
		set c_state($x_zone,$x_x,$x_y) $x_label
		set c_state($x_zone,$x_label) [ list $x_x $x_y ]
	    }
	}

	    # Update the display

	UpdateZone $x_zone
    }



# Callback for add-row menu button
# -----------------------------------
#	Add a void row to the upper or lower zone

    proc AddRow { p_flag } {
			  # p_flag	0 ... insert before
			  #		1 ... insert after

	variable	c_state
	variable	w_data

	set x_list [ FieldData $w_data(SELECT) ]
	set x_zone [ lindex $x_list 2 ]
	set x_row [ expr [ lindex $x_list 1 ] + $p_flag ]

	    # Starting at the row to be inserted, shift all items one row up

	for { set x_y $c_state($x_zone,MAX_Y) } { $x_y >= 0 } { incr x_y -1 } {
	    set x_yy [ expr $x_y + 1 ]
	    if { $x_y < $x_row } break
	    for { set x_x 0 } { $x_x <= $c_state($x_zone,MAX_X) } { incr x_x } {
		if { [ info exists c_state($x_zone,$x_x,$x_y) ] == 0 } continue
		set x_label $c_state($x_zone,$x_x,$x_y)
		unset c_state($x_zone,$x_x,$x_y)
		set c_state($x_zone,$x_x,$x_yy) $x_label
		set c_state($x_zone,$x_label) [ list $x_x $x_yy ]
	    }
	}
	incr c_state($x_zone,MAX_Y)

	    # Update the display

	UpdateZone $x_zone
    }



# Help button-up event callback
# -----------------------------
#	Special version for label (not checkbutton) widgets

    proc PopupRelease { p_dialog p_x p_y} {
			  # p_dialog	option dialog widget
			  # p_x, p_y	mouse coordinates at event

	variable	w_data
	variable	c_state

	variable	::Config::conf_descr

	$p_dialog configure -cursor ""
	set x_widget [ winfo containing $p_x $p_y ]
	set x_zone ""
	for { set x_x 0 } { $x_x  < 2 } { incr x_x } {
	    if { [ string first $w_data($x_x) $x_widget ] == 0 } {
		set x_zone $x_x
		break
	    }
	}
	if { $x_zone == "" } return
	if { [ winfo class $x_widget] != "Label" } return
	set x_list [ grid info $x_widget ]
	set x_tag $x_zone
	foreach x_item { -column -row } {
	    set x_temp [ lsearch $x_list $x_item ]
	    if { $x_temp < 0 } return
	    incr x_temp
	    append x_tag ",[ lindex $x_list $x_temp ]"
	}
	set x_label $c_state($x_tag)
	HelpAux::PopupHelp $x_widget $p_dialog $x_label
    }



# Button-up event callback
# ------------------------
#	Do nothing if
#	    - drag did not start on valid field
#	    - if up-event is not on valid receiver (empty field)
#	Else
#	    - move widget contents to receiver field
#	    - update c_state array correspondingly

    proc FieldRelease { p_field } {
			  # p_field	widget receiving the button-up hit

	variable	w_data
	variable	c_state

	variable	::Config::conf_descr

	set x_dialog [ winfo parent [ winfo parent $p_field ] ]
	set x_field $w_data(DRAG)
	if { $x_field == "" } return
	set w_data(DRAG) ""
	$x_dialog configure -cursor ""
	set y_field $w_data(RECEIVE)
	set w_data(RECEIVE) ""
	if { $y_field == "" } return

		# source (x... variables) - must be non-blank

	set x_list [ FieldData $x_field ]
	set x_x [ lindex $x_list 0 ]
	set x_y [ lindex $x_list 1 ]
	set x_n [ lindex $x_list 2 ]
	set x_label $c_state($x_n,$x_x,$x_y)

		# destination (y... variables) - must be blank

	set y_list [ FieldData $y_field ]
	set y_x [ lindex $y_list 0 ]
	set y_y [ lindex $y_list 1 ]
	set y_n [ lindex $y_list 2 ]

		# clear source

	unset c_state($x_n,$x_x,$x_y)
	unset c_state($x_n,$x_label)
	$x_field configure -text ""
	if { $x_field == $w_data(SELECT) } SelectField

		# define new destination setting

	set c_state($y_n,$y_x,$y_y) $x_label
	set c_state($y_n,$x_label) [ list $y_x $y_y ]
	$y_field configure -text $conf_descr($x_label,TEXT)
	TidyZone1
    }



# Main dialog handler
# ===================

    proc BoxDialog { p_parent } {

	variable	c_state
	variable	w_data
	variable	option_params

	global		button_coord

	variable	::Config::conf_params
	variable	::Config::conf_descr
	variable	::Config::conf_default

	set	MODULE	"option"
	set	w_data(MODULE) $MODULE
	set	f_dialog .$MODULE
	set	f_control [ Dialog::DialogTop $f_dialog $p_parent \
			{ " Help" Cancel OK } "Configure option checkboxes" 1 ]
	if { $f_control == "" } { return 0 }

	AuxProcs::DefineParameters [ namespace current]::option_params \
						$f_dialog  { fieldWidth }

	bind	$f_dialog <ButtonPress-1> { HelpAux::PopupClear}

	set w_data(DRAG) ""
	set w_data(RECEIVE) ""
	if [ info exists c_state ] { unset c_state }
	set w_data(SELECT) ""

	    # Adjust Help button command

	set x_temp [ lindex [ winfo children $f_control ] 0 ]
	$x_temp configure -command [ list HelpDialog::HelpPanel \
							$x_temp OptionDialog ]

	    # Add row menu button to top button row

	set x_shift $f_control.default
	set w_data(SHIFT) $x_shift
	set x_menu $x_shift.menu
	menubutton $x_shift -text "Add / Delete a row" -state disabled \
								-menu $x_menu
	grid	$x_shift -column 0 -row 1 -sticky w


	     # Create help button

	set	x_temp $f_control.help
	label	$x_temp -text "?" -padx 2 -pady 2 -relief raised -borderwidth 2
	grid	$x_temp -column 1 -row 1
	bind	$x_temp <ButtonPress-1> [ list HelpAux::PopupPress $x_temp ]
	bind	$x_temp <ButtonRelease-1> [ list \
			[ namespace current]::PopupRelease $f_dialog %X %Y ]


	    # Create row menu

	menu $x_menu -tearoff false
	foreach x_item { \
		{ "Add row before selection"	{ AddRow 0 } } \
		{ "Add row after selection"	{ AddRow 1 } } \
		{ } \
		{ "Delete row with selection"	{ DeleteRow } }	} {

	    if { [llength $x_item ] == 0 } {
		$x_menu add separator
		continue
	    }
	    set x_text [ lindex $x_item 0 ]
	    set x_args [ lindex $x_item 1 ]
	    set x_command [ lindex $x_args 0 ]
	    set x_args [ lreplace $x_args 0 0 \
					"[ namespace current]::$x_command" ]
	    $x_menu add command -label $x_text -command $x_args
	}

	    # Set up upper zone
	    # -----------------

		# Define upper array
		    # Read values from conf_descr. For each item create
		    #		c_state(0,x_col,x_row) ... label

	set c_state(0,MAX_X) 0
	set c_state(0,MAX_Y) 0
	foreach x_item [ array names conf_descr *,FLAGS ] {
	    regsub {,FLAGS} $x_item "" x_item
	    if { [ string first "M" $conf_descr($x_item,FLAGS) ] < 0 } continue
	    if { [ string first "c" $conf_descr($x_item,FLAGS) ] < 0 } continue
	    if { [ info exists button_coord($x_item) ] == 0 } continue
	    set x_x [ lindex $button_coord($x_item) 0 ]
	    set x_y [ lindex $button_coord($x_item) 1 ]
	    if { $x_x > $c_state(0,MAX_X) } { set c_state(0,MAX_X) $x_x }
	    if { $x_y > $c_state(0,MAX_Y) } { set c_state(0,MAX_Y) $x_y }
	    set c_state(0,$x_x,$x_y) $x_item
	    set c_state(0,$x_item) [ list $x_x $x_y ]
	}

		# Define upper frame
		    # fill each grid cell with data form upper array - blank 
		    # if it is not defined

	label	$f_dialog.title0 -text \
				"Options configured as main-panel checkbuttons:"
	pack	$f_dialog.title0 -side top -anchor w \
						-padx $conf_params(windowMargin)

	frame	$f_dialog.0
	pack	$f_dialog.0 -side top -padx $conf_params(windowMargin) -anchor w

	frame	$f_dialog.0.margin
	pack	$f_dialog.0.margin -side left	

	set	f_frame(0) $f_dialog.0.fields
	set	w_data(0) $f_frame(0)
	frame	$f_frame(0)
	pack	$f_frame(0) -side left -anchor n

	UpdateZone 0

	    # Set up lower zone
	    # -----------------
		# Read values from conf_descr, skip those that exist already 
		# in the upper zone

	pack	[ frame $f_dialog.sep1 -height $conf_params(windowMargin) ] \
								-side top
	label	$f_dialog.title1 -text \
			"Additional options available for configuration:"
	pack	$f_dialog.title1 -side top -anchor w \
						-padx $conf_params(windowMargin)

	frame	$f_dialog.1
	pack	$f_dialog.1 -side top -padx $conf_params(windowMargin) -anchor w

	frame	$f_dialog.1.margin
	pack	$f_dialog.1.margin -side left	

	set	f_frame(1) $f_dialog.1.fields
	set	w_data(1) $f_frame(1)
	frame	$f_frame(1)
	pack	$f_frame(1) -side left -anchor n

	set x_x 0
	set x_y 0
	set c_state(1,MAX_X) $c_state(0,MAX_X)
	set c_state(1,MAX_Y) -1
	foreach x_item [ array names conf_descr *,FLAGS ] {
	    regsub {,FLAGS} $x_item "" x_item
	    if { [ string first "M" $conf_descr($x_item,FLAGS) ] < 0 } continue
	    if { [ string first "c" $conf_descr($x_item,FLAGS) ] < 0 } continue
	    if { [ lsearch [ array names c_state 0,* ] 0,$x_item ] >= 0 } \
								continue

	    set c_state(1,$x_x,$x_y) $x_item
	    set c_state(1,$x_item) [ list $x_x $x_y ]
	    set c_state(1,MAX_Y) $x_y
	    incr x_x
	    if { $x_x > $c_state(1,MAX_X) } {
		set x_x 0
		incr x_y
	    }
	}
	UpdateZone 1
	pack [ frame $f_dialog.bottom -height $conf_params(windowMargin) ] \
								-side top
	bind $f_dialog <B1-Motion> [ list \
				[ namespace current ]::FieldMove %W %X %Y ]
	bind $f_dialog <ButtonRelease-1> [ list $f_dialog configure -cursor "" ]


	update idletasks
	raise $f_dialog

	set x_reply [ string tolower [ AuxProcs::ActionWait $f_dialog ] ]
	if { $x_reply == "ok" } {
	    if [ info exists button_coord ] { unset button_coord }
	    foreach x_item [ array names c_state 0,*,* ] {
		regexp {\d+,(\d+),(\d+)} $x_item "" x_x x_y
		set button_coord($c_state($x_item)) [ list $x_x $x_y ]
	    }
	    MainProcs::MainCheckboxes
	}

	destroy $f_dialog
    }
}
