package provide ProfileDialog



# Profile editor dialog
# =====================


namespace eval ProfileDialog {


# Auxiliary internal procedures
# =============================


# Add the "include zone variables" checkbutton
# --------------------------------------------

    proc ZoneButton { p_dialog } {
			  # p_dialog	top-level dialog

	variable	::Config::conf_params

	set	    x_box $p_dialog.zoneBox
	checkbutton $x_box -text "Include synchronisation-zone into profile"
	pack	    $x_box -side top -padx $conf_params(windowMargin) -anchor w
	upvar #0    [ $x_box cget -variable ] x_temp
	set	    x_temp 1

	pack	    [ frame $p_dialog.space -height 8 ] -side top
    }



# Produce and return the list of options and their values
# -------------------------------------------------------

    proc PresetItems { p_zones } {
			  # p_zones	0 ... skip sync-zone variables, else
			  #					include them

	variable	::Config::conf_descr

	global		button_value
	global		global_vars

	set x_list [ list ]
	for { set x_x 0 } { [ info exists conf_descr($x_x) ] } { incr x_x } {
	    set x_tag $conf_descr($x_x)
	    set x_flags $conf_descr($x_tag,FLAGS)
	    set x_select 0
	    if { [ string first "p0" $x_flags ] >= 0 } { set x_select 1 }
	    if { ( [ string first "p1" $x_flags ] >= 0 ) &&
					( $p_zones != 0 ) } { set x_select 1 }
	    if { $x_select == 0 } continue
	    if { [ string first "c" $x_flags ] >= 0 } {
		lappend x_list "\"$x_tag=$button_value($x_tag)\""
	    } elseif { [ string first "b" $x_flags ] >= 0 } {
		lappend x_list "\"$x_tag=$button_value($x_tag)\""
	    } elseif { [ string first "v" $x_flags ] >= 0 } {
		lappend x_list "\"$x_tag=$global_vars($x_tag)\""
	    }
	}
	return $x_list
    }



# Display a modular dialogs with all settings for a profile
# ---------------------------------------------------------

    proc ShowProfile { p_parent p_name p_list } {
			  # p_parent	parent widget
			  # p_name	name of profile
			  # p_list	list of profile items

	variable	::Config::conf_descr
	variable	::Config::conf_params

	set	MODULE	"showprofile"
	set	f_dialog .$MODULE

	if { [ Dialog::DialogCreate $f_dialog $p_parent \
				"Usync profile display" ] == 0 } { return 0 }
	wm resizable $f_dialog false false

	if { [ tk scaling ] > 1.1 } {
	    set tab_list [ list 2p left 150p left 250p left 400p ]
	    set text_width 450p
	    puts desktop
	} else {
	    set tab_list [ list 2p left 180p left 300p left 500p ]
	    set text_width 550p
	    puts laptop
	}

	set	f_outer $f_dialog.outer
	frame	$f_outer
	pack	$f_outer -side top -fill both -padx $conf_params(windowMargin) \
					 -pady $conf_params(windowMargin)


	grid	rowconfigure $f_outer 1 -minsize 8
	grid	columnconfigure $f_outer 0 -minsize 20p
	grid	columnconfigure $f_outer 1 -minsize $text_width

	label	$f_outer.name -text $p_name
	grid	$f_outer.name -column 0 -columnspan 2 -row 0 -sticky ws

	set	x_done $f_outer.done
	button	$x_done -text "Done" -command \
					[ list ::Dialog::DialogDone $x_done ]
	grid	$x_done -column 2 -row 0 -sticky e


	    # Create the text widget for the profile display

	set	x_widget $f_outer.text
	text	$x_widget -font normal -relief ridge -borderwidth 2 -width -1 \
			-bg azure2 -tab $tab_list -spacing1 1 -spacing3 1
	grid	$x_widget -column 1 -columnspan 2 -row 2 -sticky ew

	set x_row 0
	set x_x 0
	foreach x_item $p_list {
	    if { [regexp {^\"([^=]+)=(.*)\"$} $x_item "" \
						x_name x_value ] == 0 } continue
	    set x_flags $conf_descr($x_name,FLAGS)
	    if { [ string first "v" $x_flags ] < 0 } {
		if { $x_name == "XFER_DIRECTION" } {
		    set x_value [ lindex { "<-" "->" } $x_value ]
		} else {
		    set x_value [ lindex { off on } $x_value ]
		}
		$x_widget insert end "\t$x_name\t$x_value"
		incr x_x
		if { $x_x > 1 } {
		    $x_widget insert end "\n"
		    set x_x 0
		    incr x_row
		}
	    } else {
		if { $x_value == "" } { set x_value \"\" }
		if { $x_x > 0 } {
		    $x_widget insert end "\n"
		    set x_x 0
		    incr x_row
		}
		$x_widget insert end "\t$x_name\t$x_value\n"
		incr x_row
	    }
	}
	$x_widget configure -height $x_row -state disabled	

	update idletasks
	raise $f_dialog
	set exit_code [ Dialog::DialogWait $p_parent $f_dialog ]
	destroy $f_dialog

    }



# Panel for editing setting profiles
# ----------------------------------

    proc EditProfile { p_parent p_mode } {
			  # p_parent	parent widget
			  # p_mode	editing mode: update, delete, create

	variable	::MainPanel::main_widgets

	variable	::Config::conf_params

	global		button_value
	global		global_vars

	switch $p_mode {
	    "re-define" {
		set x_title "Select a profile for updating"
		set x_doit "Re-define"
	    }

	    "delete" {
		set x_title "Select a profile for deletion"
		set x_doit "Delete"
	    }

	    "create" {
		set x_title "Create new profile with current settings"
		set x_doit "Create"
	    }

	    "display" {
		set x_title "Display settings of a profile"
		set x_doit "Display"
	    }

	    default {
		return
	    }
	}

	set	MODULE	"profile"
	set	f_dialog .$MODULE
	set	f_control [ Dialog::DialogTop $f_dialog $p_parent \
					[ list Cancel "${x_doit}+" ] "" ]
	if { $f_control == "" } { return 0 }

	set	x_help $f_control.help
	button	$x_help -text "Help" -command [ list \
				HelpDialog::HelpPanel $x_help Profiles  ]
	grid	$x_help -column 0 -row 1 -sticky w


		# Bottom row operational data
		# ---------------------------

	switch $p_mode {

	    "create" {

		    # Create a profile

		set	x_title "Create a new profile"

		ZoneButton $f_dialog

		set	f_name $f_dialog.name
		frame	$f_name
		pack	$f_name -side top -padx $conf_params(windowMargin)

		set	x_temp $f_name.label
		label	$x_temp -text "Profile name: "
		pack	$x_temp -side left

		set	x_text $f_name.text
		text	$x_text -height 1 -font normal -pady 2
		pack	$x_text -side left
		bind	$x_text <Enter> [ list focus $x_text ]
		bind	$x_text <Return> [ list \
					Dialog::DialogDefault $f_dialog ]
		bind	$x_text <Tab>	{ break }
		bind    $f_dialog <Enter> [ list focus $x_text ]
	    }

	    "delete" -
	    "display" -
	    "re-define" {

		    # Delete or update a profile

		if { $p_mode == "delete" } {
		    set x_title "Delete a profile"
		} else {
		    set x_title "Re-define a profile"
		}
		if { $p_mode == "re-define" } { ZoneButton $f_dialog }

		set	x_box $f_dialog.list
		listbox	$x_box -height 0 -selectborderwidth 2 \
			-selectbackground azure4 -selectforeground yellow
		pack $x_box -side top -padx $conf_params(windowMargin)

		set x_menu $main_widgets(PRESET_MENU)
		set x_last [ $x_menu index end ]
		if [ info exists preset_table ] { unset preset_table }
		set x_lead [ string length $conf_params(profileLabel) ]
		for { set x_x 0 } { $x_x <= $x_last } { incr x_x } {
		    if { [ $x_menu type $x_x ] != "command" } continue
		    set x_command [ $x_menu entrycget $x_x -command ]
		    set x_arg [ lindex $x_command 1 ]
		    if { [ llength $x_arg ] <= 1  } continue
		    if [ regexp {^\d+$} $x_arg ] continue
		    set preset_table([ $x_box index end ]) $x_x
		    set x_temp [ $x_menu entrycget $x_x -label ]
		    $x_box insert end [ string range $x_temp $x_lead end ]
		}
		$x_box selection set 0
		bind $f_dialog <Enter> [ list focus $x_box ]
		bind $x_box <KeyPress> [ list if { "%K" == "Return" } [ list \
					Dialog::DialogDefault $f_dialog ] ]
	    }
	}
	wm title $f_dialog $x_title
	pack [ frame $f_dialog.bottom -height $conf_params(windowMargin) ] \
								-side top


	    # Wait for user interaction
	    # -------------------------

	update idletasks
	raise $f_dialog

	while { 1 } {
	    set x_reply [ string tolower [ AuxProcs::ActionWait $f_dialog ] ]
	    if [ winfo exists $f_dialog.zoneBox ] {
		upvar #0 [ $f_dialog.zoneBox cget -variable ] x_zones
	    } else {
		set x_zones 0
	    }

	    switch $x_reply {

		"create" {
		    set x_list [ PresetItems $x_zones ]
		    set x_temp [ string trim [ $x_text get 1.0 end ] ]
		    if { $x_temp == "" } {
			Dialog::DialogMessage . Warning "" [ list \
				{The profile needs a name !} ] Continue
			continue 
		    }
		    $main_widgets(PRESET_MENU) add command \
				-label "$conf_params(profileLabel)$x_temp" \
				-command [ list MainAux::PresetProfile $x_list ]
		}

		"re-define" {
		    set x_list [ PresetItems $x_zones ]
		    set x_temp [ $x_box curselection ]
		    if { $x_temp != "" } { $x_menu entryconfigure \
				$preset_table($x_temp) -command \
				[ list MainAux::PresetProfile $x_list ] }
		}

		"delete" {
		    set x_temp [ $x_box curselection ]
		    if { $x_temp != "" } { $x_menu delete \
							$preset_table($x_temp) }
		}

		"display" {
		    set x_temp [ $x_box curselection ]
		    if { $x_temp == "" } continue
		    set x_label [ $x_menu entrycget \
					$preset_table($x_temp) -label ]
		    regsub {^[^=]+=\s*} $x_label "" x_label
		    set x_list [ lindex [ $x_menu entrycget \
					$preset_table($x_temp) -command ] 1 ]
		    ShowProfile $f_dialog $x_label $x_list
		}

		default {
		    set x_reply ""
		}
	    }
	    break
	}
	destroy $f_dialog
	return $x_reply
    }
}
