package provide SelectAux



# Select panel auxiliary procedures
# =================================
#	- Callback handlers
#	- Directory and sync-zone initialisation



namespace eval SelectAux {


# Callback handlers
# =================


# Toggle direction button callback
# --------------------------------

    proc ToggleDirection { p_widget { p_direction "" } } {
			  # p_widget		toggle button widget
			  # p_direction		"" ... toggle
			  #			else ... preset to value:
			  #			0 = to client, 1 = from client

	variable	::SelectDialog::button_temp
	global		global_vars

	if { $p_direction == "" } {
	    if { $button_temp(XFER_DIRECTION) == 0 } \
				{ set p_direction 1 } else { set p_direction 0 }
	}
	set button_temp(XFER_DIRECTION) $p_direction
	if { $p_direction == 0 } { set x_label up.xbm } else \
						{ set x_label down.xbm }
	$p_widget configure -bitmap @[ file join $global_vars(LIB_DIR) \
								$x_label ]
    }



# Preset selection checkbutton callback
# ------------------------------------

    proc PresetSelect { p_frame p_value } {
			  # p_frame	selection title frame
			  # p_value	0 ... clear all checkbuttons
			  #		1 ... set all checkbuttons

	foreach x_item [ winfo children $p_frame ] {
            if { [ winfo class $x_item ] != "Checkbutton" } continue
	    upvar #0 [ $x_item cget -variable ] y_temp
	    set y_temp $p_value
	}
    }



# Save / restore global state callback & service procedure
# --------------------------------------------------------

    proc CopyState { p_flag } {
			  # p_flag	0    ... accept temporary state
			  #		else ... initialise temp. state

	global		global_vars
	global		select_list
	global		select_temp
	variable	::SelectDialog::temp_save

	set save_variable \
		{ SYNC_ZONE REL_PATH CLIENT_BASE SERVER_BASE SERVER_HOST }

	if { $p_flag == 0 } {

	    # Transfer temporary state to global state
	    # ----------------------------------------

		# Copy the current select_list

	    if [ info exists select_list ] { unset select_list }
	    foreach x_item [ array names select_temp *,TYPE ] {
		regsub {,TYPE} $x_item "" x_index
		set select_list($x_index,TYPE) $select_temp($x_index,TYPE)
		set select_list($x_index,NAME) $select_temp($x_index,NAME)
		if [ info exists select_temp($x_index,VAR) ] {
		    set select_list($x_index,VAR) $select_temp($x_index,VAR)
		}
	    }

		# Copy control variables

	    foreach x_item $save_variable { set global_vars($x_item) \
							$temp_save($x_item) }

	} else {

	    # Copy global state to temporary state
	    # ------------------------------------

		# Copy the current select_list

	    if [ info exists select_temp ] { unset select_temp }
	    foreach x_item [ array names select_list *,TYPE ] {
		regsub {,TYPE} $x_item "" x_index
		set select_temp($x_index,TYPE) $select_list($x_index,TYPE)
		set select_temp($x_index,NAME) $select_list($x_index,NAME)
		if [ info exists select_list($x_index,VAR) ] { 
		    set select_temp($x_index,VAR) $select_list($x_index,VAR)
		}
	    }

		# Copy control variables

	    foreach x_item $save_variable { set temp_save($x_item) \
							$global_vars($x_item) }
	}
    }



# Auxiliary procedures
# ====================


# Define new temporary sync-zone directory
# ----------------------------------------

    proc NewTop { p_index p_dialog { p_relpath "" } } {
			  # p_index	sync-zone index in array "sync_zones"
			  # p_dialog	""   ... definitive open for
			  #					sync operations
			  #		else ... transient open for
			  #					select dialog
			  # p_relpath	relative path below sync-zone origin, 
			  #		    only considered if p_dialog == ""

	global		sync_zones
	global		global_vars
	global		button_value

	variable	::SelectDialog::temp_save

	set x_host  $sync_zones($p_index,SV_HOST)
	set x_label $sync_zones($p_index,SV_BASE)
	set x_path  $sync_zones($p_index,CL_BASE)

	if { $p_dialog == "" } {
	    set global_vars(SERVER_HOST) $x_host
	    set global_vars(SERVER_BASE) [ AuxProcs::ExpandPath $x_label ]
	    set global_vars(CLIENT_BASE) [ AuxProcs::ExpandPath $x_path ]
	    set global_vars(SYNC_ZONE) $p_index
	    if { $p_relpath != "" } { set x_path $x_path/$p_relpath }
	} else {
	    set temp_save(SERVER_HOST) $x_host
	    set temp_save(SERVER_BASE) [ AuxProcs::ExpandPath $x_label ]
	    set temp_save(CLIENT_BASE) [ AuxProcs::ExpandPath $x_path ]
	    set temp_save(SYNC_ZONE) $p_index
	}
	OpenDirectory $x_path $p_dialog
    }



# Open a new directory
# --------------------
#	Create the permanent or the temporary node list

    proc OpenDirectory { p_path { p_dialog "" } } {
			  # p_path	the directory to be opened
			  # p_dialog	""   ... definitive open for
			  #				sync operations
			  #		else ... transient open for
			  #				select dialog

	global		global_vars
	global		select_list
	global		select_temp

	variable	::SelectDialog::temp_save

	if { $p_dialog != "" } { MainProcs::WaitMessage \
				"Getting list of remote nodes, wait ..." }
	set p_path [ AuxProcs::ExpandPath $p_path ]

	    # Attach to local variable "var_list"
	    #	- the permanent node list "select_list"   or
	    #	- the temporary node list "select_temp"
	    #
	    # Define the current path at for permanent use or temporary use

	if { $p_dialog != "" } {
	    upvar #0 select_temp var_list
	    set x_path [ string range $p_path \
				[ string length $temp_save(CLIENT_BASE) ] end ]
	    set temp_save(REL_PATH) [ string trimright $x_path "/" ]
	    set x_path $temp_save(SERVER_BASE)$x_path
	    set x_host $temp_save(SERVER_HOST)
	} else {
	    upvar #0 select_list var_list
	    set x_path [ string range $p_path [ string length \
					$global_vars(CLIENT_BASE) ] end ]
	    set global_vars(REL_PATH) [ string trimright $x_path "/" ]
	    set x_path $global_vars(SERVER_BASE)$x_path
	    set x_host $global_vars(SERVER_HOST)
	}


	    # Create list of path components
	    # ------------------------------
		#	1st = sync path origin
		#	remainder = components of path name up to current

	if [ info exists var_list ] { unset var_list }


	    # Create list of file and directory names to synchronise
	    # ------------------------------------------------------
		# For each directory and each file, add an item to select list:
		#				<x_index> = current ordinal
		#	var_list($x_index,NAME)	relative path name
		#	var_list($x_index,TYPE)	"file" or "tree" or "link"
		#	var_list($x_index,VAR)	checkbutton control var.

		# Add an extra item before first local and remote item:
		#	var_list($x_index,NAME)	"local" or "remote"
		#	var_list($x_index,TYPE)	"title"
		#	var_list($x_index,VAR)  is not defined

	if [ info exists temp_list ] { unset temp_list }

		    # Create temporary array of nodes on local host
			# First, do hidden files (pattern = .*)
			# than repeat for non-hidden files

	set x_x 0
	foreach x_pattern  { ".*" "*" } {
	    if [ catch { glob ${p_path}/$x_pattern } x_list ] continue 
	    foreach x_item [ lsort -dictionary [ glob ${p_path}/$x_pattern ] ] {
		set x_tail [ file tail $x_item ]
		if { $x_pattern == ".*" } {
		    if { ( $x_tail == ".." ) || ( $x_tail == "." ) } continue
		    if { [ string range $x_item end-1 end ] == "/." } continue
		} else {
		    if [ regexp {.*/#.+#$} $x_item ] continue
		    if [ regexp {.*/*~$} $x_item ] continue
		}
		set x_type [ file type $x_item ]
			 if { $x_type == "directory" } { set x_type "tree" }
		set temp_list($x_x,NAME) $x_tail
		set temp_list($x_x,TYPE) $x_type
		incr x_x
	    }
	}

		    # Sort temporary array into permanent array:
			# proceed in 3 passes - directories, files, links

	set x_index 0
	set var_list($x_index,NAME) "local"
	set var_list($x_index,TYPE) "title"
	incr x_index

	set local_list {}
	foreach x_type { "tree" "file" "link" } {
	    for { set x_x 0 } { [ info exists temp_list($x_x,NAME) ] != 0 } \
								{ incr x_x } {
		if { $temp_list($x_x,TYPE) != $x_type } continue
		set var_list($x_index,NAME) $temp_list($x_x,NAME)
		set var_list($x_index,TYPE) $x_type
		set var_list($x_index,VAR) 1
		incr x_index
		lappend local_list $temp_list($x_x,NAME)
	    }
	}

		    # Add nodes that exist only on remote host:
			# proceed in 2 passes, first directories, then files

	set var_list($x_index,NAME) "remote"
	set var_list($x_index,TYPE) "title"
	incr x_index

	set x_list [ Actions::RemoteList $x_host $x_path ]
	foreach x_type { "tree" "file" "link" } {
	     foreach x_item $x_list {
		if { $x_item == "./" } continue
		if { $x_item == "../" } continue
		set x_x [ string first "->" $x_item ]
		if { $x_x >= 0 } {
		    if { $x_type != "link" } continue
		    incr x_x -2
		    set x_item [ string range $x_item 0 $x_x ]
		} elseif { [ string range $x_item end end ] == "/" } {
		     if { $x_type != "tree" } continue
		     set x_item [ string range $x_item 0 end-1 ]
		} else {
		    if { $x_type != "file" } continue
		}
		if { [ lsearch -exact $local_list $x_item ] >= 0 } continue
		set var_list($x_index,NAME) $x_item
		set var_list($x_index,TYPE) $x_type
		set var_list($x_index,VAR) 1
		incr x_index
	    }
	}
	if { $p_dialog != "" } { MainProcs::WaitMessage "" }
	if { $p_dialog == "" } return
	SelectDialog::FillSelect $p_dialog
    }
}
