package provide SelectDialog



# Select panel creation and initialisation
# ========================================



namespace eval SelectDialog {


    variable	temp_save		;# temporary control variable for 
					 # operational checkbuttons
    variable	button_temp		;#temporary control variable for 
					 # file enabling checkbuttons
    variable	select_params	;# resource data-base params for main-widget
		    # (<xxx>Type)	... colours representing node types
		    # (selCols)		... number of columns
		    # (minCol)		... minimum column width
		    # (maxSize)		... max overall width and height
		    # (arrowWidth)	... witdth of column with direction 



# Auxiliary procedures
# ====================


# Fill the data zone of the selection dialog panel
# ------------------------------------------------

    proc FillSelect { p_dialog } {
			  # p_dialog	select dialog widget

	global		select_temp
	global		button_value
	global		global_vars
	global		sync_zones

	variable	temp_save
	variable	button_temp
	variable	select_params

	variable	::Config::conf_params

	set x_left 20


	    # Frame for selection buttons
	    # ---------------------------
	    #	row	0	... top space
	    #		1	... local host + origin
	    #		2	... frame with
	    #				- direction selection button
	    #				- summit node buttons
	    #				- checkbox preset menu button
	    #		3	... complementary host +origin
	    #		4	... middle space
	    #		5	... frame with node selection checkboxes
	    #
	    #	column	0	... left indent
	    #		1	... result panel		span = 3
	    #			... arrow				
	    #		2	... rubber for result panel
	    #			... path frame
	    #		3	... preset button

	set	f_data $p_dialog.data
	set	MODULE [ string range $p_dialog 1 end ]

	if [ winfo exists $f_data ] {
	    set x_save [ bind $p_dialog <Destroy> ]
	    bind $p_dialog <Destroy> {}
	    destroy $f_data
	    eval "bind $p_dialog <Destroy> [ list $x_save ]"
	    update idletasks
	}

	frame	$f_data
	pack	$f_data -padx $conf_params(windowMargin) -side bottom -fill x
	grid	columnconfigure $f_data 0 -minsize  $x_left
	grid	columnconfigure $f_data 0 -weight   0
	grid	columnconfigure $f_data 1 -weight   0 -minsize \
						$select_params(arrowWidth)
	grid	columnconfigure $f_data 2 -weight   1
	grid	columnconfigure $f_data 3 -weight   0

	grid	rowconfigure	$f_data	0 -minsize  1
	grid	rowconfigure	$f_data	4 -minsize  4
	grid	rowconfigure	$f_data	6 -minsize  $conf_params(windowMargin)


	    # Create list of path components
	    # ------------------------------
	    #	1st = sync path origin
	    #	remainder = components of path name up to current


		# Hosts and origins

	set x_server  $temp_save(SERVER_HOST)
	if { $x_server == "" } { set x_server "localhost" }

	label	$f_data.client -pady 0 -borderwidth 0 -text \
			"Client = localhost:$temp_save(CLIENT_BASE)/ ..." 
	grid	$f_data.client -column 1 -columnspan 2 -row 1 -sticky w

	label	$f_data.server -pady 0 -borderwidth 0 -text \
			"Server = ${x_server}:$temp_save(SERVER_BASE)/ ..."
	grid	$f_data.server -column 1 -columnspan 2 -row 3 -sticky w


		# List of nodes

	set x_temp [ string trim $temp_save(REL_PATH) "/" ]
	if { $x_temp != "" } {
	    set x_comps [ file split $x_temp ]
	    set x_comps [ concat $temp_save(CLIENT_BASE) $x_comps ]
	} else {
	    set x_comps [ list $temp_save(CLIENT_BASE) ]
	}


	    # Create frames for origin selection
	    # ----------------------------------

		# Frame for path components and global node selection

	set	f_comps $f_data.comps
	frame	$f_comps
	grid	$f_comps -row 2 -column 2 -sticky ew -pady 4


		# Frame for node selection panel

	set	f_select $f_data.nodes
	set	f_boxes $f_select.canvas.f
	frame	$f_select -relief ridge -borderwidth 2
	grid	$f_select -row 5 -column 1 -columnspan 3 -sticky ew


	    # Pre-select menu button and menu
	    # -------------------------------

		    # Pre-select menu button

	set	x_select $f_data.preset
	set	select_menu $x_select.menu
	menubutton $x_select -text "Preset ..." -menu $select_menu
	grid	$x_select -column 3 -row 2 -rowspan 2 -sticky s

		    # Pre-select menu

	menu $select_menu -tearoff false

	foreach x_item [ list \
		[ list "Un-select all" "SelectAux::PresetSelect $f_boxes 0" ] \
		[ list "Select all"    "SelectAux::PresetSelect $f_boxes 1" ] \
		{ } \
		{ "Initial settings"   "SelectAux::CopyState 1" } ] {

	    if { [llength $x_item ] == 0 } {
		$select_menu add separator
		continue
	    }
	    set x_text [ lindex $x_item 0 ]
	    set x_args [ lindex $x_item 1 ]
	    if { [ llength $x_args ] == 1 } {
		$select_menu add command -label $x_text -command [ list \
				MainAux::PresetProfile $x_args $f_boxes ]
	    } else {
		$select_menu add command -label $x_text -command $x_args
	    }
	}


	    # Define the items in the path component frame
	    # --------------------------------------------

		# Loop through components of path,
		#   for each component, create a button (except last - a label)

	set x_direction $f_data.direction
	button $x_direction -height 15 -bitmap "" -command [ list \
				SelectAux::ToggleDirection $x_direction ]
	grid $x_direction -column 1 -row 2 -sticky w -padx 30

	set x_x 0
	set x_last [ expr [ llength $x_comps ] - 1 ]
	set x_path ""
	foreach x_item $x_comps {
	    if { $x_x == 0 } {
		set x_label "./"
		set y_pad 8
	    } else {
		set x_label "${x_item}/"
		set y_pad 2
	    }
	    set x_path [ file join $x_path $x_item ]
	    set x_widget $f_comps.h_$x_x
	    if { $x_x < $x_last } {
		label $x_widget -text $x_label -padx $y_pad -pady 2 -relief \
							raised -borderwidth 2
		bind $x_widget <Button-1> [ list SelectAux::OpenDirectory \
							$x_path $p_dialog ]
	    } else {
		label $x_widget -text $x_label -padx 0 
	    }
	    if { [ expr $x_x % 2 ] != 0 } {
		set x_pad 4
	    } else {
		set x_pad 0
	    }
	    pack $x_widget -side left -padx $x_pad
	    incr x_x
	}


	    # Create the canvas for the node display
	    # --------------------------------------
	    #	column 0 ... data
	    #	column 1 ... vertical scrollbar

	canvas $f_select.canvas -height 0 -highlightthickness 0
	grid $f_select.canvas  -column 0 -row 0 -sticky ew
	grid columnconfigure $f_select 0 -weight 1
	grid columnconfigure $f_select 1 -weight 0

	grid rowconfigure $f_select 0 -weight 1
	grid rowconfigure $f_select 1 -weight 0

	frame $f_boxes -borderwidth 0
	$f_select.canvas create window 0 0 -window $f_boxes -anchor nw


	    # Fill the selection checkbox array
	    # ---------------------------------

	set n_cols $select_params(selCols)
	for { set x_x 0 } { $x_x < $n_cols } { incr x_x } {
	    grid columnconfigure $f_boxes $x_x -weight 1 \
						-minsize $select_params(minCol)
	}

	set x_row 0
	set x_col 0
	set x_header ""
	set x_prec ""
	set x_path [ string trimright \
			$temp_save(CLIENT_BASE)$temp_save(REL_PATH) "/" ]
	for { set x_index 0 } { [ info exists select_temp($x_index,TYPE) ] } \
							{ incr x_index } {
	    if { $x_col >= $n_cols } {
		incr x_row
		set x_col 0
	    }
	    set x_text $select_temp($x_index,NAME)
	    set x_type $select_temp($x_index,TYPE)
	    if { $x_type == "title" } {
		set x_header $x_text		;# keep pending, handle later
		set x_prec $x_type
		continue
	    }

		# File, directory or link item: check whether to be skipped

	    if { ( [ string range $x_text 0 0 ] == "." ) && \
					( $button_temp(HIDDEN_TOP) == 0 ) } {
		set select_temp($x_index,VAR) 0
		continue
	    }
	    if { ( $select_temp($x_index,TYPE) == "link" ) && \
					( $button_temp(LINK_TOP) == 0 ) } {
		set select_temp($x_index,VAR) 0
		continue
	    }

		# Handle pending header item before checkbutton creation

	    if { $x_prec == "title" } {
		if { $x_col != 0 } {
		    incr x_row
		    set x_col 0
		}
		if { $x_header == "local" } {
		    set x_title "on client"
		} elseif { $x_header == "remote" } {
		    set x_temp $f_boxes.t_$x_index
		    frame $x_temp -height 2 -borderwidth 2 -relief ridge
		    grid $x_temp -column 0 -columnspan $n_cols  -row $x_row \
							-pady 4 -sticky ew
		    incr x_row
		    set x_title "that exist only on server" 
		} else {
		    set x_title ""
		}
		set x_temp $f_boxes.l_$x_index
		label $x_temp -text "Directories and files $x_title" \
								-font standard
		grid $x_temp -column 0 -columnspan $n_cols -row $x_row \
								-sticky ew
		incr x_row
	    }

		# File, tree or link item: create the check button

	    if { $x_type != $x_prec } {
		if { $x_col != 0 } {
		    incr x_row
		    set x_col 0
		}
		grid rowconfigure $f_boxes $x_row -minsize 6
		incr x_row
	    }

	    set x_var $select_temp($x_index,VAR)
	    set x_button $f_boxes.s_$x_index
	    checkbutton $x_button -foreground  $select_params(${x_type}Type) \
			-text $x_text -variable select_temp($x_index,VAR) \
			-font normal

	    grid $x_button -column $x_col -row $x_row -sticky w -padx 3

	    if { $x_type == "tree" } {
		set x_file [ file join $x_path $x_text ]
		bind $x_button <Double-Button-1> [ list after idle \
				SelectAux::OpenDirectory $x_file $p_dialog ]
		bind $x_button <Button-3> [ list after idle \
				SelectAux::OpenDirectory $x_file $p_dialog ]
	    }

	    incr x_col
	    set x_prec $x_type
	}


	    # Adapt size and scrollbar according to contents
	    # ----------------------------------------------

	update idletasks

	set x_height [ winfo height $f_select.canvas.f ]
	set y_height $x_height
	if { $x_height > [ lindex $select_params(maxSize) 1 ] } {
	    set x_height [ lindex $select_params(maxSize) 1 ]
	    scrollbar $f_select.yscroll -orient vertical -width 14 \
				-command [ list $f_select.canvas yview ]
	    grid $f_select.yscroll -column 1 -row 0 -sticky nse

	    $f_select.canvas configure -yscrollcommand \
						[ list $f_select.yscroll set ]
	}

	set x_width [ winfo width $f_select.canvas.f ]
	set y_width $x_width
	if { $x_width > [ lindex $select_params(maxSize) 0 ] } {
	    set x_width [ lindex $select_params(maxSize) 0 ]
	    scrollbar $f_select.xscroll -orient horizontal -width 14 \
				-command [ list $f_select.canvas xview ]
	    grid $f_select.xscroll -column 0 -row 1 -sticky sew

	    $f_select.canvas configure -xscrollcommand \
						[ list $f_select.xscroll set ]
	}
	$f_select.canvas configure -width $x_width -height $x_height \
					-scrollregion "0 0 $y_width $y_height"

	set x_temp $temp_save(SYNC_ZONE)
	if { [ string first "r" $sync_zones($x_temp,FLAGS) ] >= 0 } {
	    set button_temp(XFER_DIRECTION) 0
	    $x_direction configure -state disabled
	} else {
	    $x_direction configure -state normal
	}

	SelectAux::ToggleDirection $x_direction $button_temp(XFER_DIRECTION)
	return
    }



# Select dialog
# -------------

    proc SelectDialog { p_parent } {
			  # p_parent	parent widget (calling dialog)

	global		global_vars
	global		sync_zones
	global		button_value
	global		host_table

	variable	button_temp
	variable	select_params
	variable	temp_save

	variable	::Config::conf_params

	set	MODULE "select"

	set	f_dialog .$MODULE
	set	f_control [ Dialog::DialogTop $f_dialog $p_parent [ list \
		    Cancel Accept ] "Set origin of directory, select nodes" ]
	if { $f_control == "" } { return 0 }

	if { [ array size select_params ] == 0 } {
	    AuxProcs::DefineParameters [ namespace current]::select_params \
								$f_dialog  {  \
			{ selCols	} { minCol	} { maxSize	}
			{ arrowWidth	} { fileType	} { treeType	}
			{ linkType	}				}
	}

	bind	$f_dialog <Destroy> { }
	bind	$f_dialog <Destroy> [ list Dialog::DialogActivate $f_dialog "" ]

	set	x_help $f_control.help
	button	$x_help -text "Help" -command [ list \
				HelpDialog::HelpPanel $x_help SelectDialog ]
	grid	$x_help -column 1 -row 1 -sticky e -padx 10

	SelectAux::CopyState 1


		# Synchronisation zone selection menu button and menu
		# ---------------------------------------------------

		    # Menu button

	set	x_button $f_control.top
	set	x_menu $x_button.menu
	menubutton $x_button -text "Set synchronisation zone"
	grid	$x_button -column 0 -row 1 -sticky w

		    # Menu with sync-zone paths

	set	x_menu $x_button.menu
	menu	$x_menu -tearoff false
	$x_button configure -menu $x_menu


	foreach x_item [ array names sync_zones *,NUMBER ] {
	    regsub {,NUMBER} $x_item "" x_zone
	    set x_number $sync_zones($x_item)
	    set x_aux($x_number) $x_zone
	}
	for { set x_seq 0 } { [ info exists x_aux($x_seq) ] } { incr x_seq } {
	    set x_zone $x_aux($x_seq)
	    set x_host $sync_zones($x_zone,SV_HOST)
	    set x_path $sync_zones($x_zone,CL_BASE)
	    if { [ string range  $x_path 0 0 ] == "#" } {
		set x_state disabled
	    } elseif { $x_host == "" } {
		if { [ file isdirectory [ AuxProcs::ExpandPath \
				$sync_zones($x_zone,SV_BASE) ] ] == 0 } {
		    set x_state disabled
		} else {
		    set x_state normal
		}	
	    } elseif { [ string first "x" $host_table($x_host,FLAGS) ] >= 0 } {
		set x_state disabled
	    } else {
		set x_state normal
	    }
	    set x_server $sync_zones($x_zone,SV_BASE)
	    if { $x_host != "" } { set x_server "${x_server}@$x_host" }
	    $x_menu add command -label "$x_path  <->  $x_server" -state \
		$x_state -command [ list SelectAux::NewTop $x_zone $f_dialog ]
	}


	    # Create control part of selection dialog
	    # ---------------------------------------
	    #	column	0	... spacing before left comment zone
	    #		1	... left comment zone
	    #		2	... comment zone separator
	    #		3	... right comment zone
	    #		4	... right margin

	set	f_top $f_dialog.top
	frame	$f_top
	pack	$f_top -side top -fill x

	grid	columnconfigure $f_top 0 -weight 0	-minsize 60
	grid	columnconfigure $f_top 1 -weight 0
	grid	columnconfigure $f_top 2 -weight 0	-minsize 20
	grid	columnconfigure $f_top 3 -weight 0
	grid	columnconfigure $f_top 4 -weight 1	-minsize \
						    $conf_params(windowMargin)

	set	n_cols 5		;# column 0 is left margin space
	set	x_row 0


		# Comment zone
		# ------------

	label	$f_top.title2 -pady 0 -borderwidth 0 -text \
		"Actions, presentation of synchronisation directory nodes"
	grid	$f_top.title2 -column 0 -columnspan 4 -row $x_row -sticky w \
						-padx $conf_params(windowMargin)
	incr	x_row
	grid	rowconfigure $f_top $x_row -minsize 6
	incr	x_row

		    # Open a directory

	label	$f_top.comment1a -font normal -borderwidth 0 -pady 0 -text \
			"Open a directory : "
	grid	$f_top.comment1a -row $x_row -column 1 -sticky w

	label	$f_top.comment1b -font normal -borderwidth 0 -pady 0 -text \
			"(make it the new synchronisation directory)\
					double-left-click or right-click"
	grid	$f_top.comment1b -row $x_row -column 3 -sticky w
	incr	x_row

		    # Toggle the selection

	label	$f_top.comment2a -font normal -borderwidth 0 -pady 0 -text \
					"Toggle node selection : "
	grid	$f_top.comment2a -row $x_row -column 1 -sticky w

	label	$f_top.comment2b -font normal -borderwidth 0 -pady 0 -text \
			"left-click (only selected directories and files\
					will be included in the sync action)"
	grid	$f_top.comment2b -row $x_row -column 3 -sticky w
	incr	x_row

		    # Node representation

	label	$f_top.comment4a -font normal -text "Node representation :"
	grid	$f_top.comment4a -row $x_row -column 1 -sticky w -pady 2

	set	f_show $f_top.comment4b

	frame	$f_show
	grid	$f_show -row $x_row -column 3 -sticky w


	set x_x 0
	foreach x_type {  { "file"	"node is a file"	file } \
			  { "directory"	"node is a direcory"	tree } \
			  { "link"	"node is a link"	link } } {
	    if { $x_x != 0 } { pack [ frame $f_show.s_$x_x -width 20 ] \
								-side left }

	    set	   x_temp [ lindex $x_type 2 ]
	    label  $f_show.l_$x_x -text [ lindex $x_type 0 ] \
					-fg $select_params(${x_temp}Type)
	    pack   $f_show.l_$x_x -side left

	    label  $f_show.t_$x_x -font normal -text "... [ lindex $x_type 1 ]"
	    pack   $f_show.t_$x_x -side left
	    incr   x_x
	}
	incr	x_row


		# Synchronisation directory header zone
		# -------------------------------------

	frame	$f_top.sep -borderwidth 2 -height 2 -relief raised
	grid	$f_top.sep -column 0 -columnspan $n_cols -row $x_row -pady 6 \
								-sticky ew
	incr	x_row

	label	$f_top.title3 -text "Synchronisation directory"
	grid	$f_top.title3 -column 0 -columnspan 3 -row $x_row -sticky w \
						-padx $conf_params(windowMargin)

	frame	$f_top.options
	grid	$f_top.options -column 3 -row $x_row -sticky w -padx 50
	AuxProcs::CheckboxArray $f_top.options "S" 2 0 "standard" \
					[ namespace current ]::button_temp
	foreach x_item [ winfo children $f_top.options ] {
	    $x_item configure -command [ list \
				[ namespace current ]::FillSelect $f_dialog ]
	}
	incr	x_row

	grid	rowconfigure $f_top $x_row -minsize 2


		# Create and fill selection checkbutton panel
		# -------------------------------------------

	set button_temp(XFER_DIRECTION) $button_value(XFER_DIRECTION)
	FillSelect $f_dialog


	    # Wait for user interaction
	    # -------------------------

	update idletasks
	raise $f_dialog
	set exit_code [ AuxProcs::ActionWait $f_dialog ]
	if { [ string tolower $exit_code ] == "accept" } {
	    set x_change 0
	    foreach x_item { CLIENT_BASE SERVER_BASE SERVER_HOST } \
		    { if { $global_vars($x_item) != $temp_save($x_item) } \
							{ set x_change 1 } }
	    foreach x_item { XFER_DIRECTION } \
		    { if { $button_value($x_item) != $button_temp($x_item) } \
							{ set x_change 1 } }
	    foreach x_item [ array names button_temp ] \
			{ set button_value($x_item) $button_temp($x_item) }
	    SelectAux::CopyState 0
	    MainProcs::PathLabels
	    MainProcs::SetDirection
	    if { $x_change } { MainAux::LogItem "" base }
	}
	destroy $f_dialog
	return $exit_code
    }
}
