package provide SettingDialog



# Auxiliary panels for configuration setting control
# ==================================================
#	- Setting console output verbosity level
#	- Setting rsync timeout limit
#	- Creation, modification and deletion of configuration profiles



namespace eval SettingDialog {


    variable	button_temp



# Callback procedure for general settings default button
# ------------------------------------------------------

    proc DefaultSettings { } {

	variable	button_temp

	variable	::Config::conf_default

	foreach x_item [ array names button_temp ] \
		{ set button_temp($x_item) $conf_default($x_item,VALUE) }
    }



# Auxiliary procedures
# ====================


# Panel for defining the value of rsync timeout limit
# ---------------------------------------------------

    proc GeneralSettings { p_parent } {

	global		global_vars	
	global		button_value

	variable	button_temp
	variable	setting_params	;# parameters from resource data base

	variable	::Config::conf_default
	variable	::Config::conf_params
	variable	::Config::conf_descr

	set	MODULE	"settings"
	set	f_dialog .$MODULE
	set	f_control [ Dialog::DialogTop $f_dialog $p_parent \
				{ Cancel OK+ } "Control general settings" ]
	if { $f_control == "" } { return 0 }

	AuxProcs::DefineParameters [ namespace current]::setting_params \
								$f_dialog { \
		{ leftMargin } { valueCols } { boxWidth } { valueSep } }

	bind $f_dialog <Enter> [ list focus $f_dialog ]
	bind $f_dialog <KeyPress> [ list if { "%K" == "Return" } \
				[ list Dialog::DialogDefault $f_dialog ] ]


	set	x_default $f_control.default
	button	$x_default -text "Default" -command [ list \
					[ namespace current]::DefaultSettings ]
	grid	$x_default -column 0 -row 1 -sticky w

	set	x_help $f_control.help
	button	$x_help -text "Help" -command [ list \
				HelpDialog::HelpPanel $x_help Settings  ]
	grid	$x_help -column 1 -row 1 -sticky w -padx 10


	    # Zone with numeric values
	    # -----------------------

	set	f_values $f_dialog.params
	frame	$f_values -padx $conf_params(windowMargin)
	pack	$f_values -side top -anchor w
	grid	columnconfigure $f_values 0 -minsize $setting_params(leftMargin)

	set	x_row 0

	label	$f_values.title -text "Parameter values"
	grid	$f_values.title -column 0 -columnspan 2 -row $x_row -sticky w
	incr	x_row

	set item_list ""
	set x_col 0
	foreach x_item [ array names conf_descr *,FLAGS ] {
	    set x_flags $conf_descr($x_item)
	    if { [ string first "G" $x_flags ] < 0 } continue
	    if { [ string first "v" $x_flags ] < 0 } continue
	    regsub {,FLAGS} $x_item "" x_tag
	    set button_temp($x_tag) $global_vars($x_tag)
	    set x_x [ expr $x_col * 4 ]

	    if { ( $x_row == 1 ) && ( $x_col > 0 ) } { grid columnconfigure \
			$f_values $x_x -minsize $setting_params(valueSep) }
	    incr x_x

	    set x_z "${x_col}_$x_row"
	    label $f_values.label_$x_z -font normal \
					-text "$conf_descr($x_tag,TEXT):"
	    grid $f_values.label_$x_z -column $x_x -row $x_row -sticky w
	    incr x_x
	    set	x_value $f_values.spin_$x_z
            spinbox $x_value -from [ lindex $conf_default($x_tag,PROPS) 0 ] \
			-to [ lindex $conf_default($x_tag,PROPS) 1 ] \
			-increment 1 -repeatinterval 20 -width 0 -font normal \
			-textvariable [ namespace current ]::button_temp($x_tag)
	    grid $x_value -column $x_x -row $x_row -padx 4 -sticky e
	    if { [ string first "s" $x_flags ] >= 0 } {
		incr x_x
		label $f_values.sec_$x_z -text "sec" -font normal
		grid $f_values.sec_$x_z -column $x_x -row $x_row -sticky w
	    }
	    lappend item_list $x_tag
	    incr x_col
	    if { $x_col >= $setting_params(valueCols) } {
		set x_col 0
		incr x_row
	    }
	}


	    # Zone with various checkbuttons (at present, only console echo)
	    # -------------------------------------------------------------

	frame	$f_dialog.separator -height 2 -borderwidth 2 -relief raised 
	pack	$f_dialog.separator -side top -fill x \
					-pady $conf_params(windowMargin)

	set	f_buttons $f_dialog.buttons
	frame	$f_buttons -padx $conf_params(windowMargin)
	pack	$f_buttons -side top -anchor w
	grid	columnconfigure $f_buttons 0 -minsize \
						$setting_params(leftMargin)

	set	x_row 0

	label	$f_buttons.title -text "Operational settings"
	grid	$f_buttons.title -column 0 -row $x_row -sticky w
	incr	x_row

	set x_result [ AuxProcs::CheckboxArray $f_buttons "G" 1 $x_row "" \
				[ namespace current ]::button_temp ]
	set x_col [ expr [ lindex $x_result 0 ] + 1 ]
	grid configure $f_buttons.title -columnspan $x_col

	for { set x_x 1 } { $x_x < $x_col } { incr x_x } \
		{ grid columnconfigure \
			$f_buttons $x_x -minsize $setting_params(boxWidth) }

	incr x_row
	grid rowconfigure $f_buttons $x_row -minsize $conf_params(windowMargin)


	    # Wait for user interaction
	    # -------------------------

	update idletasks
	raise $f_dialog

	set x_reply [ string tolower [ AuxProcs::ActionWait $f_dialog ] ]
	if { $x_reply == "ok" } {
	    set x_save $button_value(WRITE_LOG)
	    foreach x_tag $item_list { set global_vars($x_tag) \
							$button_temp($x_tag) }
	    foreach x_item [ array names button_temp ] \
			{ set button_value($x_item) $button_temp($x_item) }
	    log_user $button_value(CONSOLE_ECHO)
	    if {  $button_value(WRITE_LOG) != $x_save } {
		if { $button_value(WRITE_LOG) } { 
		    MainAux::LogItem "" open
		} else {
		    MainAux::LogItem "" close
		}
	    }
	}
	    
	destroy $f_dialog
	return $x_reply	
    }



# Fill the button panel
# ---------------------

    proc FillMore { p_options p_button p_all } {
			  # p_options	frame containing the checkbuttons
			  # p_button	the show / hide toggle button
			  # p_all	0 ... set for hide all
			  #		      else: toggle show/hide

	global		button_coord
	global		button_value

	variable	more_params	;# parameters from resource data base
	variable	show_all

	variable	::Config::conf_descr
	variable	::Config::conf_default

	if { $p_all == 0 } {
	    set show_all 0
	} else {
	    if { $show_all == 0 } { set x_all 1 } else { set x_all 0 }
	    set show_all $x_all
	}
	if { $show_all } { set x_label "Hide" } else { set x_label "Show" }
	$p_button configure -text "$x_label main-window buttons"

		# Sort options according to option-type groups
		#   if group is not a defined group, force group to "various"

	for { set x_x 0 } { [ info exists conf_descr($x_x) ] } { incr x_x } {
	    set x_tag $conf_descr($x_x) 
	    set x_flags $conf_descr($x_tag,FLAGS)

		    # Skip  - if not an unconfigured main-panel checkbutton

	    set x_group [ string first "M" $x_flags ]
	    if { $x_group < 0 } continue
	    if { [ string first "c" $x_flags ] < 0 } continue
	    if { $show_all == 0 } {
	        if [ info exists button_coord($x_tag) ] continue
	    }

	    incr x_group
	    set x_group [ string range $x_flags $x_group $x_group ]
	    if { [ string first $x_group $conf_descr(GROUPS) ] < 0 } \
							{ set x_group "V" } 
	    lappend x_list($x_group) $x_tag
	}


		# Create the check buttons
		#    Proceed group by group, add in group headers and spacing

	set x_limit [ expr $more_params(arrayCols) + 1 ]
	grid columnconfigure $p_options 0 -minsize $more_params(leftMargin)
	set x_x 1
	set x_y 0

	for { set x_n 0 } { [ info exists conf_descr($x_n,GROUP) ] } \
								{ incr x_n } {
	    set x_group [ lindex $conf_descr($x_n,GROUP) 0 ]
	    set x_title [ lindex $conf_descr($x_n,GROUP) 1 ]
	    if { [ info exists x_list($x_group) ] == 0 } continue
	    if { [ llength x_list($x_group) ] == 0 } continue

		    # A new group
		    #	if necessary, advance to a new line
		    #	put separating space (except on top)
		    #	put group header

	    if { $x_x > 1 } {
		set x_x 1
		incr x_y
	    }

	    if { $x_y > 0 } {
		grid rowconfigure $p_options $x_y \
						-minsize $more_params(groupSep)
		incr x_y
	    }
	    label $p_options.l_0_$x_y -text $x_title
	    grid  $p_options.l_0_$x_y -column 0 -columnspan $x_limit \
							-row $x_y -sticky w
	    incr x_y

		    # Loop through the items of this group

	    set x_extra ""
	    foreach x_item $x_list($x_group) {
		set x_button $p_options.c_${x_x}_$x_y
	        if [ info exists button_coord($x_item) ] { 
		    lappend x_extra $x_item
		    continue 
		}
		set x_text $conf_descr($x_item,TEXT)
		checkbutton $x_button -variable button_value($x_item) \
				-text $x_text -font normal
		grid $x_button -column $x_x -row $x_y -padx 4 -sticky w
		incr x_x
		if { $x_x >= $x_limit } {
		    set x_x 1
		    incr x_y
		}
	    }
	    foreach x_item $x_extra {
		set x_text $conf_descr($x_item,TEXT)
		set x_button $p_options.c_${x_x}_$x_y
		checkbutton $x_button -variable button_value($x_item) \
				-text $x_text -state disabled -font normal \
				-disabledforeground mediumpurple3
		grid $x_button -column $x_x -row $x_y -padx 4 -sticky w
		incr x_x
		if { $x_x >= $x_limit } {
		    set x_x 1
		    incr x_y
		}
	    }
	}
    }



# Toggle state of show / hide all buttons
# ---------------------------------------

    proc ToggleMore { p_options p_button } {
			  # p_options	frame containing the checkbuttons
			  # p_button	the show / hide toggle button

	set p_dialog .more
	set x_save [ bind $p_dialog <Destroy> ]
	bind $p_dialog <Destroy> {}
	foreach x_widget [ winfo children $p_options ] {
	    destroy $x_widget
	}
 	eval "bind $p_dialog <Destroy> [ list $x_save ]"
	FillMore $p_options $p_button 1
    }




# Panel for defining the value of rsync timeout limit
# ---------------------------------------------------

    proc MoreOptions { p_parent } {

	global		button_coord
	global		button_value
	global		initial_value

	variable	more_params	;# parameters from resource data base

	variable	::Config::conf_params
	variable	::Config::conf_descr
	variable	::MainPanel::main_widgets


	set	MODULE	"more"
	set	f_dialog .$MODULE
	set	f_control [ Dialog::DialogTop $f_dialog $p_parent \
					{ " Help" Cancel OK+ } "More options" ]
	if { $f_dialog == "" } { return 0 }
	bind	$f_dialog <ButtonPress-1> { HelpAux::PopupClear}
	bind $f_dialog <Enter> [ list focus $f_dialog ]
	bind $f_dialog <KeyPress> [ list if { "%K" == "Return" } \
				[ list Dialog::DialogDefault $f_dialog ] ]


	AuxProcs::DefineParameters [ namespace current]::more_params \
								$f_dialog { \
			{ arrayCols } { colSize } { groupSep } { leftMargin } }


	    # Save initial value 

	foreach x_item [ array names conf_descr *,FLAGS ] {
	    regsub {,FLAGS} $x_item "" x_item
	    set x_flags $conf_descr($x_item,FLAGS)

		# Skip 	- if the item is not for the present panel
		#	- if the item is not a checkbutton
		#	- if a main-panel item is not configured
		#		("M" flag, no button_coord value defined)

	    if { [ string first "M" $x_flags ] < 0 } continue
	    if { [ string first "c" $x_flags ] < 0 } continue
	    if [ info exists button_coord($x_item) ] continue
	    set initial_value($x_item) $button_value($x_item)
	}

	set	f_options $f_dialog.options

	set	f_left $f_control.left
	frame	$f_left
	grid	$f_left -column 0 -row 1 -sticky w

	set	x_default $f_left.default
	set	x_menu $x_default.menu
	menubutton $x_default -text "Preset" -menu $x_menu
	pack	$x_default -side left

	set	x_toggle $f_left.toggle
	button	$x_toggle -text "Toggle" -command [ list \
			[ namespace current]::ToggleMore $f_options $x_toggle ]
	pack	$x_toggle -side left -padx $conf_params(buttonSpacing)

	set	x_help [ lindex [ winfo children $f_control ] 0 ]
	$x_help	configure -command [ list HelpDialog::HelpPanel $x_help More ]
	    # The preset menu is a copy of the main-panel preset menu

	menu	$x_menu -tearoff false
	set	y_menu $main_widgets(PRESET_MENU)
	set	x_max [ $y_menu index last ]
	for { set x_x 0 } { $x_x <= $x_max } { incr x_x } {
	    if { [ $y_menu type $x_x ] == "separator" } {
		$x_menu add separator
	    } else {
		set x_label [ $y_menu entrycget $x_x -label ]
		set x_command [ $y_menu entrycget $x_x -command ]
		if { [ llength $x_command] > 2 } { set x_command \
					[ lreplace $x_command 2 2 $f_options ] }
		$x_menu add command -label $x_label -command $x_command
	    }
	}
	set	x_temp $f_control.help
	label	$x_temp -text "?" -padx 2 -pady 2 -relief raised -borderwidth 2
	grid	$x_temp -column 1 -row 1
	bind	$x_temp <ButtonPress-1> [ list HelpAux::PopupPress $x_temp ]
	bind	$x_temp <ButtonRelease-1> [ list \
				HelpAux::PopupButtonup $f_options %X %Y ]


	    # Create option checkboxes
	    # ------------------------

	frame	$f_options -padx $conf_params(windowMargin)
	pack	$f_options -side top -anchor w

	FillMore $f_options $x_toggle 0
	pack [ frame $f_dialog.bottom -height $conf_params(windowMargin) ] \
								-side top

	    # Wait for user interaction
	    # -------------------------

	update idletasks
	raise $f_dialog

	set x_reply [ string tolower [ AuxProcs::ActionWait $f_dialog ] ]
	if { $x_reply != "ok" } {

	    foreach x_item [ array names conf_descr *,FLAGS ] {
		regsub {,FLAGS} $x_item "" x_item
		set x_flags $conf_descr($x_item,FLAGS)

		# Skip 	- if the item is not for the present panel
		#	- if the item is not a checkbutton
		#	- if a main-panel item is not configured
		#		("M" flag, no button_coord value defined)

		if { [ string first "M" $x_flags ] < 0 } continue
		if { [ string first "c" $x_flags ] < 0 } continue
		if [ info exists button_coord($x_item) ] continue
		set button_value($x_item) $initial_value($x_item)
	    }
	}

	    
	destroy $f_dialog
	return $x_reply	
    }
}
