package provide TopItems


# Synchronisation zone configuration management
# =============================================
#	- Sync-zone menu item editor panel



namespace eval TopItems {

    variable	item_widgets	    ;# widget data
			# (DIALOG)	... toplevel dialog
			# (MIDDLE)	... gridded frame with text widgets
			# (COLOR1)	... background of un-selected item
			# (COLOR2)	... background of selected item
			# (SELECT)	... rel. number of selected row or ""
			# (LIMIT)	... highest field index (hor.)
			# (SEQ_NO)	... number in next temporary name

    variable	item_params	;# parameters from resource data base

    variable	host_temp	;# temporary host table (Cancel - Accept !)



# Auxiliary procedures
# ====================


# Text widgets and labels in the display are generated with path names of the
# form  <gridded-frame>.t_<col>_<row-id>
#		where	<gridded-frame>	= master containing the widgets
#			<col>		= column containing the widget (0 ... 3)
#			<row-id>	= common id attributed to all widgets of
#						a row when the row is created
#
#	proc TextWidget:	obtain widget path name given column and row-id
#	proc RowIdentifier:	obtain row-id given the relative row number
#
#
# The gridded frame has 2 leading comment rows, all following rows contain
# text and comment data
#	a "relative row" therefore is the physical row number - 2



# Return path name of the text widget for a given column and row-identifier
# -------------------------------------------------------------------------
#	Name is:	<parent_widget>.f_<column>.t_<syncid>
#	    with	    column ... the column frame number (0, 1, 2, 3)
#			    syncid ... the index into the sync_zones array

    proc TextWidget { p_column p_rowID } {
			  # p_column	column frame number
			  # p_rowID	the row-identifier

	variable	item_widgets

	set f_middle $item_widgets(MIDDLE)
	return $f_middle.t_${p_column}_${p_rowID}
    }



# Return row-identifier for a given row
# -------------------------------------

    proc RowIdentifier { p_row } {

	variable	item_widgets

	set f_middle $item_widgets(MIDDLE)
	set x_list [ grid slaves $f_middle -row [ expr $p_row + 2 ] ]
	regexp {([^_]+)$} [ lindex $x_list 0 ] "" x_rowID
	return $x_rowID
    }



# Activate a row that will now be the "selected row"
# --------------------------------------------------
#	- Restore panels that had focus to unselected state
#	- Set all text panels of the row to selected

    proc ActivateRow { p_row } {

	variable	item_widgets

	if { $p_row == $item_widgets(SELECT) } return
 
	    # Undo the old row selection

	set f_middle $item_widgets(MIDDLE)
	if { $item_widgets(SELECT) != "" } {
	    set x_row [ expr $item_widgets(SELECT) + 2 ]
	    foreach x_widget [ grid slaves $f_middle -row $x_row ] {
		if { [ winfo class $x_widget ] != "Text" } continue
		$x_widget configure -relief flat -bg $item_widgets(COLOR1)
	    }
	}


	    # Establish the new selection

	set item_widgets(SELECT) $p_row
	foreach x_widget [ grid slaves $f_middle -row [ expr $p_row + 2 ] ] {
	    if { [ winfo class $x_widget ] != "Text" } continue
	    $x_widget configure -relief sunken -bg $item_widgets(COLOR2)
	}
	set x_rowID [ RowIdentifier $p_row ]
	focus [ TextWidget 0 $x_rowID ]
    }



# Check for correct sync-zone item
# --------------------------------
#	If OK: return empty list
#	if error: return 2-item list
#			item #0 ... error text
#			item #1	... error argument

    proc CheckItem { cl_base sv_base p_host } {
			  # cl_base	client path
			  # sv_base	server path or label with offset
			  # p_host	server host or ""

	if { [ file isdirectory [ AuxProcs::ExpandPath $cl_base ] ] == 0 } {
	    set x_error [ list "Client directory does not exist" $cl_base ]
	} elseif { $p_host != "" } {

	    # Server side is a remote host

	    set x_error [ Actions::CheckHost $p_host $sv_base ]
	} elseif { [ file isdirectory [ AuxProcs::ExpandPath \
							$sv_base ] ] == 0 } {

	    # Server side is a local directory

	    set x_error [ list "Server directory does not exist" $sv_base ]
	} else {
	    set x_error {}
	}
	return $x_error
    }



# Callback procedures
# ===================


# Enter callback for toplevel window
# ----------------------------------
#	Activate selected field - if none the first

    proc EnterTop { p_dialog } {

	variable	item_widgets

	if { $item_widgets(SELECT) != "" } return
	set x_row $item_widgets(SELECT)
	if { $x_row == "" } { set x_row 0 }
	ActivateRow $x_row
    }



# Callback for button-down on a text field
# ----------------------------------------

    proc HitItem { p_x p_y } {
			  # p_x absolute x-coordinate of hit
			  # p_y absolute y-coordinate of hit

	variable	item_widgets

	set x_parent $item_widgets(MIDDLE)
	set root_x [ winfo rootx $x_parent ]
	set root_y [ winfo rooty $x_parent ]
	set x_list [ grid location $x_parent [ expr $p_x - $root_x ] \
							[ expr $p_y - $root_y ] ]
	ActivateRow [ expr [ lindex $x_list 1 ] - 2 ]
    }


# Service callback procedures
# ===========================


# Move the currently select one row up or down (circular)
# -------------------------------------------------------

    proc MoveItem { p_step } {
			  # p_step	+1 move down
			  #		-1 move up

	variable	item_widgets

	set x_row $item_widgets(SELECT)
	if { $x_row == "" } return
	set x_limit $item_widgets(LIMIT)			    ;# max column
	set x_middle $item_widgets(MIDDLE)
	set x_max [ expr [ lindex [ grid size $x_middle ] 1 ] - 3 ] ;# max row
	set y_row [ expr $x_row + $p_step ]
	if { $y_row < 0 } { 
	    set y_row $x_max
	} elseif { $y_row > $x_max } {
	    set y_row 0
	}

	    # Fill x_clBase, x_svBase, x_host with values from the current row

	set x_rowID [ RowIdentifier $x_row ]
	for { set x_x 0 } { $x_x <= $x_limit } { incr x_x } {
	    set x_widget [ TextWidget $x_x $x_rowID ]
	    if { $x_x == $x_limit } {
		set x_value($x_x) [ $x_widget cget -text ]
	    } else {
		set x_value($x_x) [ string trim [ $x_widget get 1.0 end ] ]
	    }
	}

	    # Fill y_clBase, y_svBase, y_host with values from the other row

	set y_rowID [ RowIdentifier $y_row ]
	for { set x_x 0 } { $x_x <= $x_limit } { incr x_x } {
	    set x_widget [ TextWidget $x_x $y_rowID ]
	    if { $x_x == $x_limit } {
		set y_value($x_x) [ $x_widget cget -text ]
	    } else {
		set y_value($x_x) [ string trim [ $x_widget get 1.0 end ] ]
	    }
	}

	    # Fill the original row with the field of the other row

	for { set x_x 0 } { $x_x <= $x_limit } { incr x_x } {
	    set x_widget [ TextWidget $x_x $x_rowID ]
	    if { $x_x == $x_limit } {
		$x_widget configure -text $y_value($x_x)
	    } else {
		$x_widget delete 1.0 end
		$x_widget insert 1.0 $y_value($x_x)
	    }
	}

	    # Fill the other row with the field of the original row

	for { set x_x 0 } { $x_x <= $x_limit } { incr x_x } {
	    set x_widget [ TextWidget $x_x $y_rowID ]
	    if { $x_x == $x_limit } {
		$x_widget configure -text $x_value($x_x)
	    } else {
		$x_widget delete 1.0 end
		$x_widget insert 1.0 $x_value($x_x)
	    }
	}
	ActivateRow $y_row
    }



# Delete the currently selected row
# ---------------------------------

    proc DeleteRow { p_dialog } {

	variable	item_widgets

	set x_row $item_widgets(SELECT)
	if { $x_row == "" } return
	set item_widgets(SELECT) ""
	set save_row $x_row
	set x_middle $item_widgets(MIDDLE)
	set n_rows [ lindex [ grid size $x_middle ] 1 ]

	set x_save [ bind $p_dialog <Destroy> ]
	bind $p_dialog <Destroy> {}
	incr x_row 2
	foreach x_widget [ grid slaves $x_middle -row $x_row ] \
							{ destroy $x_widget }
	incr x_row
	while { $x_row < $n_rows } {
	    set x_x [ expr $x_row - 1 ]
	    foreach x_widget [ grid slaves $x_middle -row $x_row ] \
					{ grid configure $x_widget -row $x_x }
	    incr x_row
	}
 	eval "bind $p_dialog <Destroy> [ list $x_save ]"
	if { $n_rows <= 3 } { 
	    AddRow 1
	} else {
	    ActivateRow $save_row
	}
    }



# Append a rwo with empty text and label items
# --------------------------------------------
#	Return row ID created

    proc AddRow { { p_select "" } } {
			  # p_select	"" if row does not need to be activated
			  #					else e.g. 1

	variable	item_widgets
	variable	item_params

	set x_rowID "ID$item_widgets(SEQ_NO)"
	incr item_widgets(SEQ_NO)

	set x_row [ lindex [ grid size $item_widgets(MIDDLE) ] 1 ]
	for { set x_x 0 } { $x_x <= $item_widgets(LIMIT) } { incr x_x } {
	    set x_widget [ TextWidget $x_x $x_rowID ]
	    if { [ expr $x_x & 1 ] } { set x_pad 4 } else { set x_pad 0 }
	    set x_width [ lindex $item_params(columnWidths) $x_x ]
	    if { $x_x == $item_widgets(LIMIT) } {
		label $x_widget -font normal -width $x_width -anchor center
	    } else {
		text $x_widget -height 1 -borderwidth 2 -relief flat \
			-width $x_width -padx 4 -font normal
		bind $x_widget <Return>	  [ list Dialog::DialogDefault \
							$item_params(DIALOG) ] 
		bind $x_widget <Tab>	  { break }
		bind $x_widget <Button-1>  [ list [ namespace \
						current ]::HitItem %X %Y ]
	    }
	    grid $x_widget -column $x_x -row $x_row -padx $x_pad
	}
	incr x_row -2
	if { $p_select != "" } { ActivateRow $x_row }
	return $x_rowID
    }



# Create and fill text panels for all existing synchronisation-zones
# ------------------------------------------------------------------

    proc FillItems { p_dialog } {

	global		sync_zones

	variable	host_temp
	variable	item_widgets

	set f_middle $item_widgets(MIDDLE)

	    # Destroys any old text fields

	set x_save [ bind $p_dialog <Destroy> ]
	bind $p_dialog <Destroy> {}
	set x_rows [ lindex [ grid size $f_middle ] 1 ]
	for { set x_x 2 } { $x_x < $x_rows } { incr x_x } {
	    foreach x_widget [ grid slaves $f_middle -row $x_x ] \
							{ destroy $x_widget }
	}
 	eval "bind $p_dialog <Destroy> [ list $x_save ]"
	set item_widgets(SELECT) ""
	update idletasks
	set item_widgets(SEQ_NO) 0	;# start naming at 0


	    # Create row and fill fields
		# Start by creating an auxiliary array with sync-zones
		# ordered by their sequence number

	foreach x_item [ array names sync_zones *,NUMBER ] {
	    regsub {,NUMBER} $x_item "" x_zone
	    set x_number $sync_zones($x_item)
	    set x_aux($x_number) $x_zone
	}
	for { set x_seq 0 } { [ info exists x_aux($x_seq) ] } { incr x_seq } {
	    set x_zone $x_aux($x_seq)
	    set x_rowID [ AddRow ]
	    set x_status "OK"		;# assume OK status

		# Loop through the fields of the synchronisation zone

	    set x_x 0
	    foreach x_tag { CL_BASE SV_BASE SV_HOST } {
		set x_temp $sync_zones($x_zone,$x_tag)
	        set x_widget [ TextWidget $x_x $x_rowID ]
		$x_widget insert 1.0 $x_temp
		if { [ string range $sync_zones($x_zone,CL_BASE) 0 0 ] \
								== "#" } {
		    set x_status "comment-only"
		} elseif { ( $x_tag == "SV_HOST") && ( $x_temp != "" ) } {
		    if { [ string first "x" $host_temp($x_temp,FLAGS) ] >= 0 } \
						{ set x_status "suspended" }
		} elseif { ( $x_tag == "SV_BASE" ) && \
				( $sync_zones($x_zone,SV_HOST) == "" ) } {
		    if { [ file isdirectory [ AuxProcs::ExpandPath \
				$sync_zones($x_zone,SV_BASE) ] ] == 0 } {
			set x_status "not mounted"
		    } elseif { [ string first "t" \
				     $sync_zones($x_zone,FLAGS) ] >= 0 } {
			set x_status "mounted"
		    }
		}
		incr x_x
	    }
	    set x_widget [ TextWidget $item_widgets(LIMIT) $x_rowID ]
	    $x_widget configure -text $x_status	
	}
    }



# Main service handler
# ====================


# Handler for sync-zone item editing
# ----------------------------------

    proc EditItems { p_parent } {
			  # p_parent	parent widget (calling panel)

	global		sync_zones
	global		global_vars
	global		host_table

	variable	item_widgets
	variable	item_params
	variable	host_temp

	variable	::Config::conf_params

	set	MODULE "items"
	set	f_dialog .$MODULE
	set	item_params(DIALOG) $f_dialog

	if { [ Dialog::DialogCreate $f_dialog $p_parent \
		"Add / delete / modify sync-zone items" ] == 0 } { return 0 }
	wm	resizable $f_dialog 0 0
	bind	. <ButtonPress-1> { HelpAux::PopupClear }
	wm	withdraw $f_dialog
	bind $f_dialog <Enter> [ list [ namespace current]::EnterTop $f_dialog ]

	AuxProcs::DefineParameters [ namespace current]::item_params \
				$f_dialog  { { columnWidths } { activeColour } }

	set	f_control $f_dialog.control
	frame	$f_control
	pack	$f_control -side top -padx $conf_params(windowMargin) \
				-pady $conf_params(windowMargin) -fill both

	set	f_zones $f_dialog.outer
	frame	$f_zones
	pack	$f_zones -padx $conf_params(windowMargin) -side top

	MainProcs::WaitMessage "Getting server-module data, wait ..."


	    # Create temporary host table
	    # ---------------------------

	set global_vars(NETWORK) [ Actions::CheckNetwork ]
	if [ info exists host_temp ] { unset host_temp }
	foreach x_item [ array names host_table *,FLAGS ] {
	    if { $global_vars(NETWORK) == 0 } {
		set host_temp($x_item) "x"
	    } else {
		regsub {,FLAGS} $x_item "" x_host
		if { [ Actions::HostIsup $x_host ] <= 0 } {
		    set host_temp($x_item) "x"
		} else {
		    set host_temp($x_item) ""
		}
	    }
	}


	    # Create control part of the editor dialog
	    # ----------------------------------------
		# Button list:
		#   item #0 ... label
		#   item #1 ... padx value for button
		#   item #2 ... button command

	set x_x 0
	foreach x_list {
		{ "Accept"  -0	"Dialog::DialogDone $x_button" } \
		{ "Cancel"  -6	"Dialog::DialogDone $x_button" } \
		{ "Reload" -14	"[namespace current]::FillItems $f_dialog" } \
		{ "Help"    -6	"HelpDialog::HelpPanel $x_button TopItem" } \
		{ "Add new item" 0 \
			"[ namespace current ]::AddRow 1" } \
		{ "Delete selected item" 6 \
			"[ namespace current ]::DeleteRow $f_dialog" } } {
	    set x_pad [ lindex $x_list 1 ]
	    if { [ string range $x_pad 0 0 ] == "-" } {
		set x_pad [ string range $x_pad 1 end ]
		set x_side right
	    } else {
		set x_side left
	    }
	    set x_button $f_control.b_$x_x
	    button $x_button -text [ lindex $x_list 0 ] -command \
					[ eval "list [ lindex $x_list 2 ]" ]
	    pack $x_button -side $x_side -padx $x_pad
	    if { $x_x == 0 } { $x_button configure -default active }
	    incr x_x 
	}
	set y_pad 5

	label  $f_control.label -text "   move item" -padx 0
	pack   $f_control.label -side left -padx 0

	button $f_control.up -bitmap @[ file join $global_vars(LIB_DIR) \
		up.xbm ] -command [ list [namespace current]::MoveItem -1 ]
	pack   $f_control.up -side left -padx 6 -fill y -pady $y_pad


	button $f_control.down -bitmap @[ file join $global_vars(LIB_DIR) \
		down.xbm ] -command [ list [namespace current]::MoveItem 1 ]
	pack   $f_control.down -side left -padx 0 -fill y -pady $y_pad


	    # Create gridded frame for text and comment-label widgets
	    # -------------------------------------------------------

	set	f_middle $f_zones.middle
	set	item_widgets(MIDDLE) $f_middle
	frame	$f_middle -borderwidth 2 -relief ridge
	pack	$f_middle -side top -fill x

		# Create title and separator widgets

	set x_x 0
	set x_labels {	"local path (\"client\")"  "rsync path (\"server\")" \
			"@ site (void: local)"	   "status" }
	set item_widgets(LIMIT) [ expr [ llength $x_labels ] - 1 ]
	set item_widgets(COLOR2) $item_params(activeColour)
	for { set x_x 0 } { $x_x <= $item_widgets(LIMIT) } { incr x_x } {
	    if { $x_x == $item_widgets(LIMIT) } { set x_anchor center } else \
							{ set x_anchor w }
	    label $f_middle.l_$x_x -text [ lindex $x_labels $x_x ] \
							-anchor $x_anchor
							
	    grid  $f_middle.l_$x_x -column $x_x -row 0 -sticky ew -padx 10
    	}

	frame $f_middle.sep -borderwidth 2 -height 2 -relief ridge
	grid  $f_middle.sep -column 0 -columnspan 4 -row 1 -sticky ew
	set item_widgets(COLOR1) [ $f_middle cget -bg ]

		# Fill the widgets with sync-zone data, add empty row if nothing

	FillItems $f_dialog
	if { [ array size sync_zones ] == 0 } { AddRow 1 }

	pack [ frame $f_zones.space -height $conf_params(windowMargin) ] \
								-side top
	
	update idletasks
	wm deiconify $f_dialog
	raise $f_dialog
	MainProcs::WaitMessage ""


	    # Wait for user interaction
	    # -------------------------

	while { 1 } {
	    set x_error [ list ]
	    set exit_code [ string tolower [ AuxProcs::ActionWait $f_dialog ] ]
	    if { $exit_code != "accept" } break
	    set global_vars(NETWORK) [ Actions::CheckNetwork ]


		# Loop through the now defined sync objects (one per row)
		# -------------------------------------------------------
		    # Read text panels with components of the sync object,
		    # perform consistency check and compose the "temp-objects"
		    # array.
		    #
		    # Copy the temp_zones to the sync_zones array once all
		    # checks have been successfully passed.

	    if [info exists temp_zones ]  { unset temp_zones }
	    set x_first ""		;# first valid zone seen
	    set x_list [ list ]
	    set x_limit [ expr [ lindex [ grid size $f_middle ] 1 ] - 2 ]

	    for { set x_row 0 } { $x_row < $x_limit } { incr x_row } {
		set x_rowID [ RowIdentifier $x_row ]
		set x_flags ""
		set x_status "OK"	;# assume OK status


		    # Fill x_clBase, x_svBase, x_host with values from the 
		    # current row

		set x_x 0
		foreach x_item { x_clBase x_svBase x_host } {
		    set x_widget [ TextWidget $x_x $x_rowID ]
		    upvar 0 $x_item x_dummy
		    set x_dummy [ string trim [ $x_widget get 1.0 end ] ]
		    incr x_x
		}
		ActivateRow $x_row	;# in case of an error: show the row

		    # Verify correctness of this object
			# - x_rowID	... label
			# - x_clBase	... client-side path
			# - x_svBase	... server-side path
			# - x_host	... server ip address (host) or ""
			#
			# Remember first correct zone in x_first

	        if { [ string range $x_clBase 0 0 ] == "#" }	\
				{ set x_comment 1 } else { set x_comment 0 }
		if { $x_comment == 1 } {
		    set x_check 0
		} elseif { ( $x_clBase == "" ) || ( $x_svBase == "" ) } {
		    set x_error { "Client path and server path" \
							"must not be void" }
		} elseif { [ file isdirectory [ AuxProcs::ExpandPath \
							$x_clBase ] ] == 0 } {
		    set x_error [ list \
				{ Inexistant client path (local directory)} \
				"\40path = \"$x_clBase\"" ]
		} elseif { $x_host == "" } {
		    set x_path [ AuxProcs::ExpandPath $x_svBase ]
		    foreach x_item $conf_params(removableMedia) {
			if { [ string first "t" $x_flags ] >= 0 } break
			if { [ string first $x_item $x_path ] == 0 } \
							{ append x_flags "t" }
		    }
		    if { [  string first "t" $x_flags ] < 0 } {
			if { [ file isdirectory $x_path ] == 0 } {
			    set x_error [ list "Inexistant local server path" \
						"\40path = \"$x_svBase\"" ]
			}
		    } elseif { [ file isdirectory $x_path ] == 0 } {
			set x_status "not mounted"
		    } else {
			set x_status "mounted"
		    }
		} else {
		    if { $global_vars(NETWORK) == 0 } {
			set host_temp($x_host,FLAGS) "x"
			set x_check 0
		    } elseif [ info exists host_temp($x_host,FLAGS) ] {
			if { [ string first "x" \
					$host_temp($x_host,FLAGS) ] < 0 } {
			    set x_check 1
			} else { 
			    set x_check 0
			    append x_flags "x"
			    set x_status "suspended"
			}
		    } else {
			set x_check 1
		    }
		    if { $x_check != 0 } {
			if { [ Actions::HostIsup $x_host ] < 0 } {
			    set x_reply [ Dialog::DialogMessage . Question "" \
				[ list \
				"The host does not respond" \
				"\\40service = \"${x_svBase}@$x_host\"" \
				"\\10" \
				"Accept but disable this service,\
							continue editing ?" ] \
				{ Accept Continue } ]
			    if { $x_reply > 0 } {
				set x_error 1
				break
			    }
			    append x_flags "x"
			    set x_status "suspended"
			    set x_error [ list ]
			    [ TextWidget 3 $x_rowID ] configure -font marked
			    set host_temp($x_host,FLAGS) "x"
			} else {
			    set host_temp($x_host,FLAGS) ""
			}
		    }
		    if { ( $x_error == "" ) && ( [ string first "x" \
					$host_temp($x_host,FLAGS) ] < 0 ) } { \
			set x_error [ CheckItem $x_clBase $x_svBase $x_host ]
		    }
		}

		if { $x_error != "" } {
		    set x_temp "\\40[ lindex $x_error 1 ]"
		    set x_error [ lreplace $x_error 1 1 $x_temp ]
		    Dialog::DialogMessage $f_dialog "Row \#$x_row error" "" \
							$x_error { "Continue" }
		    break		;# continue item-edit panel input
		}

		set x_widget [ TextWidget $item_widgets(LIMIT) $x_rowID ]
		$x_widget configure -text $x_status	

		set temp_zones($x_rowID,CL_BASE) $x_clBase
		set temp_zones($x_rowID,SV_BASE) $x_svBase 
		set temp_zones($x_rowID,SV_HOST) $x_host
		set temp_zones($x_rowID,FLAGS) $x_flags
		lappend x_list $x_rowID

		if { ( $x_first == "" ) && ( $x_comment == 0 ) && ( [ string \
			first "x" $x_flags ] < 0 ) } { set x_first $x_rowID }
	    } ;# foreach sync object
	    if { $x_error != "" } continue


	    # Everything is OK, finalize
	    # --------------------------

		# Transfer objects definition from temporary to permanent data

	    if [ info exists sync_zones ] { unset sync_zones }
	    if [ info exists host_table ] { unset host_table }
	    foreach x_rowID $x_list {
		AuxProcs::DefineZone $temp_zones($x_rowID,CL_BASE) \
			$temp_zones($x_rowID,SV_BASE) \
					$temp_zones($x_rowID,SV_HOST) \
						$temp_zones($x_rowID,FLAGS)
		if { $temp_zones($x_rowID,SV_HOST) == "" } continue
		set x_host $temp_zones($x_rowID,SV_HOST)
		if { $x_host == "" } continue
		set host_table($x_host,FLAGS) $host_temp($x_host,FLAGS)
	    }


		# Force updating if the active sync object has changed

	    set this_zone $global_vars(SYNC_ZONE)
	    if { $this_zone != "" } {
		set temp_array(CLIENT_BASE) [ AuxProcs::ExpandPath \
					$sync_zones($this_zone,CL_BASE) ]
		set temp_array(SERVER_BASE) [ AuxProcs::ExpandPath \
					$sync_zones($this_zone,SV_BASE) ]
		set temp_array(SERVER_HOST) [ AuxProcs::ExpandPath \
					$sync_zones($this_zone,SV_HOST) ]

		foreach x_item { CLIENT_BASE SERVER_BASE SERVER_HOST } {
		    if { $global_vars($x_item) == $temp_array($x_item) } \
								continue
		    set global_vars(SYNC_ZONE) ""
		    break
		}
	    }


		# Disable action buttons if no active sync object is left

	    set x_count 0
	    foreach x_item [ array names sync_zones *,SV_HOST ] {
		set x_host $sync_zones($x_item)
		regsub {,SV_HOST} $x_item "" x_zone
		if { [ string range $sync_zones($x_zone,CL_BASE) 0 0 ] \
							== "#" } continue
		if { $x_host != "" } {
		    if { [ string first "x" $host_table($x_host,FLAGS) ] \
								>= 0 } continue
		} else {
		    if { [ file isdirectory [ AuxProcs::ExpandPath \
			$sync_zones($x_zone,SV_BASE) ] ] == 0 } continue
		} 
		incr x_count
	    }
	    if { $x_count == 0 } { set global_vars(SYNC_ZONE) "" }

	    if { ( $this_zone == "" ) && ( $x_first != "" ) } {
	        SelectAux::NewTop $x_first ""
	        MainProcs::PathLabels
	    }
	    MainProcs::EnableButtons
	    break
	} ;# while waiting for accept
	destroy $f_dialog
	return $exit_code
    }
}
