#!/usr/bin/expectk -f
####!/usr/bin/expect -f


# Main script of the Usync front-end GUI
# --------------------------------------
#	See the copyright statement in the main-panel help display
#	(help item "About Usync)


#### Calling the expect interpreter would be preferable since this makes 
#### the package usable for platforms that do not have expectk (e.g. Suse)
#### However, that creates a - so far unexplained - problem for the 64-bit
#### Mandriva implementation of Tcl/Tk/Expect (OK with the 32-bit flavour)
####
#### Note that, when using the expect interpreter, also the last statement
#### needs to be uncommented  (  tkwait window .)



# Global variables
# ----------------

#   global_vars	    Array of general global state variables
#		    ---------------------------------------
#
#	    (LIB_DIR)		path of tcl module library
#	    (HELP_DIR)		path of help file directory
#	    (MAIN_SCRIPT)	main execution script
#	    (PID)		process ID of current expect spawn process
#	    (RELEASE)		Mandriva Release (x.y notation)
#	    (TEMP_FILE)		name of unique temporary file
#	    (CONFIG_FILE)	configuration file
#	    (NETWORK)		1 / 0 if network is / is not available 
#	    (CLIENT_HOST)	name of local host (client)
#	    (CLIENT_BASE)	abs. local path for origin for synchronisation
#	    (SERVER_HOST)	host name of server
#	    (SERVER_BASE)	rsyncd.conf label on remote host or base path
#	    (REL_PATH)		offset relative to SERVER_LABEL and CLIENT_BASE
#	    (ROOT)		1 if root login, else 0
#	    (SYNC_TIMEOUT)	rsync initialisation timeout value
#	    (MODIFY_WINDOW)	time-stamp comparison tolerance (sec)
#	    (SYNC_ZONE)		index of active synchronisation zone, or "" 
#	    (POPUP)		popup help widget, "" if it does not exist


#   sync_zone	    Array with synchronisation-zone descriptors	
#		    -------------------------------------------
#			items that belong to the same descriptor have the 
#			same tag, i.e. the same leading <tag> in their index
#			    <tag>	is the string <server>,<client>
#
#	    (<tag>,FLAGS)	list of flag chars
#					r ... read_only
#					t ... removable device
#	    (<tag>,SV_HOST)	sync server host ("" if local)
#	    (<tag>,SV_BASE)	sync server path (if remote:
#						relative to rsyncd.conf item)
#	    (<tag>,CL_BASE)	sync client path (always local)
#	    (<tag>,NUMBER)	sequence number for display/.usyncrc


#   button_value    Array with state variables
#		    --------------------------
#			each button is represented by a single item, the value
#			represents the current
#			   - state of a checkbutton (0 or 1) or the
#			   - value of a  pseudo-button (i.e. XFER_DIRECTION
#						0 ... transfer to client
#						1 ... transfer to server )
#			items are indexed by <tag> 
#				<tag> = conf_descr tag of the button


#   initial_value   Array with initial value of state variables
#		    -------------------------------------------
#			single-item entries that are a copy of the 
#			corresponding button_value entry just after reading
#			the configuration file
#
#			indexing like button_value (tag from conf_descr)


#   button_coord    Array with checkbutton placement coordinates
#		    --------------------------------------------
#			Only defined for checkbuttons configured in the main
#			panel: single-item entries, value = 2-item list
#			(#0 ... column, #1 ... row) representing the placement
#			of the checkbutton in the main panel
#
#			indexing like button_value (tag from conf_descr)
#


#    select_list    List of nodes of the current synchronisation directory
#		    ------------------------------------------------------
#			single-item entries that represent the selection
#			state of each node (0 or 1)
#
#			index = relative path of the node


#    select_temp    Auxilisary copy of select_list when the dialog is active
#		    --------------------------------------------------------
#			used as state variable for the node checkbuttons
#			the value is copied to select_list on exit with "Accept"
#
#			index = relative path of the node


#    host_table	    Array of known hosts
#		    --------------------
#			one entry for each known host, describes its state,
#			leading part of index <host> = host ip address
#
#	    (<host>,FLAGS)	list of flag characters
#					x ... host is suspended



    package require Tk 8.4
    package require Tcl 8.4

    log_user 0

    match_max 100000

    wm withdraw .		    ;# no transient windows (do wm deiconify .)



# Detect path of script being executed, define module libr"ary
# -----------------------------------------------------------

    set x_dir [ info script ]
    if { [ file type $x_dir ] == "link" } {

	    # The script sits in a linked directory

	set x_temp [file readlink $x_dir]
	if { [ string range x_temp 0 0 ] != "/" } {

		# Handle the ln -s ./<name> case

	    if { [ string range $x_temp 0 1 ] == "./" } \
				{ set x_temp [ string range $x_temp 2 end ] }
	    set parent_dir [ file dirname $x_dir ]
	    set x_dir [ file join [ file dirname $x_dir ] $x_temp ]
	} else {
	    set x_dir $x_temp
	}
    }
    if { [ string range $x_dir 0 0 ] != "/" } {

		# Handle the ./<path> case

	if { [ string compare [ string range $x_dir 0 1 ] "./" ] == 0 } {
	    set x_dir [ string range $x_dir 2 end ]
	}

		# Handle the ../<path> case

	set x_temp [pwd ]
	while { [ string range $x_dir 0 2 ] == "../" } {
	    set x_dir [ string range $x_dir 3 end ]
	    set x_temp [ file dirname $x_temp ]
	}

	set x_dir [ file join $x_temp $x_dir ]
    }

	# Convert /<item>/../ to /
    
    while { [ regsub {/[^/]+/\.\./} $x_dir "/" x_dir ] } {}

    set global_vars(MAIN_SCRIPT) $x_dir
    set global_vars(LIB_DIR) [ file join [ file dirname $x_dir ] library ]
    set global_vars(HELP_DIR) [ file join [ file dirname $x_dir ] help ]


	    # Check whether there is a network, set hostname and root flag

    set global_vars(NETWORK) [ expr ! [ catch { exec /sbin/ifconfig -a eth0 \
							| grep addr: } ] ]
    if { $env(HOME) == "/root" } { set global_vars(ROOT) 1 } else \
						{ set global_vars(ROOT) 0 }

    regsub {HOSTNAME=} [ exec /bin/grep HOSTNAME /etc/sysconfig/network ] "" \
							global_vars(CLIENT_HOST)



# Load common infrastructure modules, if needed determine user or system mode
# ---------------------------------------------------------------------------

    source [ file join $global_vars(LIB_DIR) Dialog.tcl ]
    source [ file join $global_vars(LIB_DIR) MainPanel.tcl ]
    source [ file join $global_vars(LIB_DIR) MainProcs.tcl ]
    source [ file join $global_vars(LIB_DIR) MainAux.tcl ]
    source [ file join $global_vars(LIB_DIR) SelectDialog.tcl ]
    source [ file join $global_vars(LIB_DIR) SelectAux.tcl ]
    source [ file join $global_vars(LIB_DIR) TopItems.tcl ]
    source [ file join $global_vars(LIB_DIR) SettingDialog.tcl ]
    source [ file join $global_vars(LIB_DIR) ProfileDialog.tcl ]
    source [ file join $global_vars(LIB_DIR) Actions.tcl ]
    source [ file join $global_vars(LIB_DIR) Config.tcl ]
    source [ file join $global_vars(LIB_DIR) ConfigFile.tcl ]
    source [ file join $global_vars(LIB_DIR) OptionDialog.tcl ]
    source [ file join $global_vars(LIB_DIR) AuxProcs.tcl ]
    source [ file join $global_vars(LIB_DIR) HelpDialog.tcl ]
    source [ file join $global_vars(LIB_DIR) HelpAux.tcl ]



# Execute script
# --------------

    Config::InitConfig

    MainPanel::InitPanel

    foreach y_x [ info globals x_* ] { unset $y_x }
    unset y_x

####    tkwait window .	;# this is needed since expect and not expectk is used
